      PROGRAM SAFED
C***********************************************************************
C COMPILER-SPECIFIC SUBROUTINES FOR DATE AND TIME HAVE TO BE CHOSEN
C IN LINES 803 TO 816
C
C THE FUNCTION  DLGAMA = LOG-GAMMA FUNCTION
C IS SUPPORTED BY THE VSFORTRAN COMPILER ON IBM MAINFRAME MACHINES.
C IT IS ADDED AT THE END OF THIS FILE FOR OTHER COMPILERS.
C***********************************************************************
C*                                                                     *
C* SAFE-D IS A COMPUTER PACKAGE DESIGNED TO PERFORM ANALYSIS OF COMPO- *
C* NENT FAILURE-ON-DEMAND DATA.  THE PROGRAM ANALYSIS BOTH THE HOMOGE- *
C* NEOUS BINOMIAL MODEL AND THE COMPOUND BINOMIAL-PRIOR MODELS. THE    *
C* AVAILABLE PRIOR DISTRIBUTIONS ARE BETA, LOGBETA, LOGNORMAL 1, LOG-  *
C* NORMAL 2, LOGGAMMA 1, AND LOGGAMMA 2.  THE SAFE-D PACKAGE WILL PER- *
C* FORM THE FOLLOWING:  (1) ESTIMATE P AND ITS VARIANCE USING THE HOM- *
C* OGENEOUS MODEL.(2) CALCULATE THE CONFIDENCE INTERVAL FOR P AND SAM- *
C* PLE LIKELIHOOD USING THE HOMOGENEOUS MODEL.(3) PERFORM KOLMOGOROV-  *
C* SMIRNOV AND/OR CHI-SQUARED GOODNESS-OF-FIT TESTS FOR THE HOMOGENE-  *
C* OUS MODEL.(4) ESTIMATE PRIOR DISTRIBUTION PARAMETERS (ALPHA AND     *
C* BETA) BY EITHER MATCHING MOMENTS TO THE PRIOR OR MATCHING MOMENTS   *
C* TO THE MARGINAL OR THE MAXIMUM LIKELIHOOD METHOD.(5) ESTIMATE VARI- *
C* ANCE AND COVARIANCE OF PRIOR PARAMETER ESTIMATORS.(6) CALCULATE     *
C* ALPHA AND (1-ALPHA) PERCENTILES AND DETERMINE MINIMUM INTERVAL FOR  *
C* THE ESTIMATED PRIOR DISTRIBUTION.(7) CALCULATE MOMENTS, SKEWNESS    *
C* AND KURTOSIS OF ESTIMATED PRIOR DISTRIBUTION.(8) TABULATE AND/OR    *
C* PLOT ESTIMATED PRIOR DISTRIBUTION AND ESTIMATED PRIOR CUMULATIVE    *
C* DISTRIBUTION.(9) CALCULATE MARGINAL LIKELIHOOD FOR THE SAMPLE AND   *
C* PERFORM GOODNESS-OF-FIT TESTS FOR THE MARGINAL.(10) CALCULATE       *
C* MOMENTS, SKEWNESS AND KURTOSIS OF THE POSTERIOR DISTRIBUTION OF     *
C* EACH COMPONENT AND ALSO DETERMINE MINIMUM INTERVAL FOR EACH.(11)    *
C* COMPARE HOMOGENEOUS AND COMPOUND MODELS ON A COMPONENT BY COMPONENT *
C* BASIS.(12) TABULATE AND/OR PLOT POSTERIOR DISTRIBUTION OR CUMULA-   *
C* TIVE POSTERIOR DISTRIBUTION OF EACH COMPONENT.                      *
C*                                                                     *
C* SAFE-D INPUT CONSISTS OF A SET OF COMMAND CARDS FOLLOWED BY THE     *
C* EXECUTE COMMAND. ONCE A COMMAND OR ITS PARAMETERS HAVE BEEN SPECI-  *
C* FIED WITH A PREVIOUS EXECUTE COMMAND, THE COMMAND AND ITS PARAME-   *
C* TERS REMAIN IN EFFECT (UNLESS OTHERWISE SPECIFIED) FOR THE FOLLOW-  *
C* ING EXECUTE COMMANDS. THIS ALLOWS SIMILAR ANALYSIS OF DIFFERENT     *
C* DATA SETS WITH ONLY HAVING TO CHANGE THE DATA COMMAND.  A CLEAR     *
C* COMMAND IS PROVIDED TO RESET ALL COMMANDS AND PARAMETERS, EXCEPT    *
C* THE DATA COMMAND, TO THEIR INITIAL VALUES.  THE COMMAND CARDS CON-  *
C* SIST OF A KEYWORD IN COLUMNS 1 THROUGH 8, FOLLOWED BY THE COMMAND   *
C* PARAMETERS IN FREE FORMAT STARTING IN COLUMN 10 AND SEPERATED BY    *
C* COMMAS. PARAMETERS THAT ARE NOT BEING CHANGED WOULD CONTAIN NO NUM- *
C* ERIC VALUES IN THE CORRESPONDING POSITION AND WOULD HAVE A COMMA    *
C* FOLLOWED BY ANOTHER COMMA. THE PROGRAM READS IN ALL COMMANDS UNTIL  *
C* IT ENCOUNTERS AN EXECUTE COMMAND.  AFTERWHICH, ALL COMMANDS IN      *
C* EFFECT ARE PERFORMED AND THE PROGRAM ATTEMPTS TO READ ANOTHER SET   *
C* OF COMMANDS.  FOR MORE DETAILED INFORMATION CONCERNING THE INPUT    *
C* COMMANDS AND THEIR FORMATS REFER TO THE USER'S GUIDE.               *
C*                                                                     *
C*  REQUIRED SUBROUTINES: INITD, GETRUN,         HOMOD, MAXLD, KSFIT,  *
C*                        CHISQ, HD, ABESTD, MMOM1D, MLSIMD, RANGED,   *
C*                        MMVARD, VARMLD, PRMCMD, PRCNTD, PUPJKS,      *
C*                        TABLED, PLOT, PSMOMD, PSTCPD.                *
C*                                                                     *
C*  DESCRIPTION AND CROSS-REFERENCE FOR COMMON BLOCKS:                 *
C*                                                                     *
C*    /CMD/    ICMD IS THE COMMAND INDICATOR ARRAY WHOSE ELEMENTS      *
C*             CORRESPOND TO THE VARIOUS INPUT COMMANDS. AN ELEMENT HAS*
C*             A VALUE OF 1 IF THE COMMAND IS IN THE 'ON' STATE.       *
C*             A COMMAND IN THE 'ON' STATE INDICATES THE ASSOCIATED    *
C*             CALCULATIONS ARE TO BE PERFORMED.                       *
C*    /CMD6/ THROUGH /CMD22/  THESE ARE COMMON BLOCKS WHICH CORRESPOND *
C*             TO EACH OF THE VARIOUS COMMANDS AND ARE USED TO STORE   *
C*             THE VALUES OF THE COMMAND PARAMETERS.                   *
C*             REFER TO USER GUIDE FOR IDIVIDUAL PARAMETER EXPLANATION.*
C*    /DTA/    REFER TO SUBROUTINE HOMOD.                              *
C*    /FDDATA/ REFER TO SUBROUTINE MMOM1D.                             *
C*    /DATA2D/  REFER TO SUBROUTINE VAML1D.
C*    /DATA1D/ REFER TO SUBROUTINE VAML1D.                             *
C*    /IOMM/   REFER TO SUBROUTINE MMOM1D.                             *
C*    /ADRTMI/ REFER TO SUBROUTINE MMOM1D.                             *
C*    /MEST1/  REFER TO SUBROUTINE MMOM1D.                             *
C*    /POPL/   REFER TO SUBROUTINE MMOM1D.                             *
C*    /SMPL/   REFER TO SUBROUTINE MMOM1D.                             *
C*    /AQNC7/  REFER TO SUBROUTINE MMOM1D.                             *
C*    /ASMIN/  REFER TO SUBROUTINE MMOM1D.                             *
C*    /MODL/   REFER TO FUNCTION HD.                                   *
C*    /PRIR/   PARM(1) - ALPHA, PARM(2) - BETA, PARM(3) - FIRST RANGE  *
C*                       PARAMETER, PARM(4) - SECOND RANGE PARAMETER,  *
C*             IPRIR   - DENSITY CODE,                                 *
C*             RNG(1)  - LOWER LIMIT OF FAILIER PROBABILITY,           *
C*             RNG(2)  - UPPER LIMIT OF FAILIER PROBABILITY.           *
C*    /DATAD/  USED FOR STORING INDIVIDUAL COMPONENT DATA TO BE PASSED *
C*             TO THE MARGINAL AND POSTERIOR ANALYSIS SUBROUTINES.     *
C*    /ACPRMD/ ACEPS IS THE ACCURACY USED IN THE INTEGRATION           *
C*             SUBROUTINES FOR THE MARGINAL AND POSTERIOR ANALYSIS.    *
C*    /DIST/   IMRGCL IS FLAG INDICATING THAT THE MARGINAL PROBABILITY *
C*             HAS BEEN CALCULATED FOR A SINGLE COMPONENT. THE FLAG IS *
C*             USED TO PREVENT UNECESSARY RECALCULATION.               *
C*    /RTPASS/ CONTAINS PARAMETER VALUES FOR SUBROUTINE DRTMI. SEE     *
C*             SUBROUTINE FOR DETAILS.                                 *
C*    /GENCOM/ IGOUT IS THE UNIT NUMBER TO WHICH OUTPUT IS TO BE       *
C*             BE PRINTED.                                             *
C*                                                                     *
C* MAIN PROGRAM WRITTEN BY: SCOTT D. HANSEN                            *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION SMAXL(2),Y(15),AN(2),SIGMA(3),VAR(3),V(2,2)
      DIMENSION PTS(40),OUTLN(40,3),PW(10)
      CHARACTER*11 TDST(6)
      DIMENSION A(40,2),FV(6),PMOM(10)
      CHARACTER*56 PPLTD,PPLTC,CPLT,DPLT
      INTEGER OUTPUT
      EXTERNAL HD
      COMMON /CMD/   ICMD(25)
      CHARACTER*80 TITLE
      COMMON /CMD6/  TITLE,I6P1
      CHARACTER*8 COMPID(500)
      COMMON /CMD6A/ COMPID,NFL(500),NTM(500)
      COMMON /CMD7/  R7P(4),I7P1,I7P6
      COMMON /CMD7A/ R7AP(8)
      COMMON /CMD7B/ R7BP(7)
      COMMON /CMD8/  R8P2
      COMMON /CMD9/  R9P(6)
      COMMON /CMD10/ I10P2
      COMMON /CMD11/ I11P2
C%%   COMMON /CMD12/
      COMMON /CMD13/ R13PA(40),R13PB(2),I13P2,I13PA3,I13PB3
      COMMON /CMD14/ R14P3,I14P2
      COMMON /CMD15/ R15P2
      COMMON /CMD16/ R16P2
C%%   COMMON /CMD17/
      COMMON /CMD18/ R18P2
      COMMON /CMD19/ R19PA(40),R19PB(2),I19P2,I19PA3,I19PB3
      COMMON /CMD20/ R20P3,I20P2
      COMMON /CMD21/ R21P2
      COMMON /CMD22/ I22P(500),I22PN
      CHARACTER*8 CID(500)
      COMMON /DTA/ CID,NF(500),NT(500),NN,IPRT,NITER
      COMMON /FDDATA/ FI(500),TI(500)
      COMMON /DATA2D/ KD(500),ND(500)
      COMMON /DATA1D/ NND
      COMMON /IOMM/ IDUM1(2),IOUT
      COMMON /ADRTMI/ DUM2(4),EPSS,SRANG,IEND,IDUM6
      COMMON /MEST1/ IEPRNT,IIDIST
      COMMON /POPL/ DUM1(4),IDUM5,IDIST,IMPRNT
      COMMON /SMPL/ DUM3(5),NNS,IDID,ISPRNT
      COMMON /AQNC7/ ERR,IDUM2
      COMMON /ASMIN/ FVECT(2),EPS0,ANS(2),S(2),DUM4(15),IDUM3,NEV,IDUM4
      COMMON /MODL/ RLM,IIMOD
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /DATAD/ KKF,NNT
      COMMON /ACPRMD/ ACEPS
      COMMON /DIST/ DUM5,IMRGCL
      COMMON /RTPASS/  RTXL,RTXR,RTEPS,IRTEND,IRTIER
      COMMON /GENCOM/ IGOUT
      CHARACTER*4 YAX1, YAX2, XAX*6
      DATA YAX1,YAX2,XAX /'DENS','CUMM','P     '/
      DATA TDST /'BETA    G  ','LOGBETA M  ',
     >           'LOGNORMAL 1','LOGNORMAL 2',
     >           'LOGGAMMA 1 ','LOGGAMMA 2 '/
      DATA PPLTD
     >  /'      POSTERIOR DENSITY FUNCTION FOR COMPONENT          '/
      DATA PPLTC
     >  /'   POSTERIOR CUMULATIVE FUNCTION FOR COMPONENT          '/
      DATA DPLT
     >  /'PRIOR DENSITY FUNCTION                                  '/
      DATA CPLT
     >  /'PRIOR CUMULATIVE FUNCTION                               '/
C
C     INITIALIZE EXECUTION COUNTER AND SET DATA COMMAND TO 'OFF'.
C
      IEX=0
      ICMD(6)=0
C
C     DEFINE OUTPUT AND INPUT UNIT NUMBERS
C
      OUTPUT=6
      INCMD=5
      IGOUT=OUTPUT
      WRITE(OUTPUT,1006)
1006  FORMAT('1')
      CALL DATEPR(OUTPUT)
      CALL INITD
 100  CALL GETRUN(INCMD,OUTPUT,IERR)
      IF (IERR.EQ.-1) GOTO 999
      IEX=IEX+1
      IF (IERR.EQ.1) GOTO 990
      NO=0
      CALL ECHO (OUTPUT,IEX)
      IF (ICMD(8).EQ.0) GOTO 400
C.......................................................................
C     HOMOMODL
C        CALL SUBROUTINE HOMOD TO PERFORM ANALYSIS OF HOMOGENEOUS MODEL
C.......................................................................
      WRITE (OUTPUT,3001)
      WRITE (OUTPUT,3002)
 3001 FORMAT ('1',37X,52('*'),/38X,'*',50X,'*',/38X,'*',5X,'ANALYSIS FOR
     > HOMOGENEOUS BINOMIAL MODEL:',5X,'*')
 3002 FORMAT (38X,'* ALL COMPONENTS HAVE THE SAME FAILURE PROBABILITY *'
     >,/38X,'*',50X,'*',/38X,52('*'))
      IF (ICMD(6).EQ.1) GOTO 301
      WRITE (OUTPUT,3003)
 3003 FORMAT('-','*****WARNING: DATA HAS NOT BEEN GIVEN. ANALYSIS OF HOM
     >OGENIOUS MODEL CANNOT BE PERFORMED')
      GOTO 400
 301  NN=I6P1
      DO 304 I=1,NN
      NF(I)=NFL(I)
      NT(I)=NTM(I)
      CID(I)=COMPID(I)
 304  CONTINUE
      NITER=100
      PCT=R8P2
      IPRT=OUTPUT
      CALL HOMOD(RLAMBR,VARH,RLL,RLU,PCT,IER)
      STD=DSQRT(VARH)
      PCCT=1.0D0-PCT
      WRITE (OUTPUT,3004) RLAMBR,STD,VARH
      WRITE (OUTPUT,3005) PCT,RLL,PCCT,RLU
 3004 FORMAT (///' ESTIMATED FAILURE PROBABILITY =',1PG11.4,
     >'  WITH ESTIMATED STANDARD DEVIATION =',G11.4,
     >'   (VARIANCE =',G11.4,')')
 3005 FORMAT (12X,'CONFIDENCE BOUNDS: P(',F6.4,' LEVEL) =',1PG11.4,
     >'    P(',0PF6.4,' LEVEL) =',1PG11.4)
C.......................................................................
C     LIKELIHD
C        CALL SUBROUTINE MAXLD TO CALCULATE SAMPLE LIKELIHOOD.
C.......................................................................
 305  IF (ICMD(10).EQ.0) GOTO 320
      IF (I20P2.EQ.2) GOTO 320
      N=I6P1
      DO 307 I=1,N
      FI(I)=NFL(I)
      TI(I)=NTM(I)
 307  CONTINUE
      RLAMDA=RLAMBR
      IMODL=2
      CALL MAXLD(DLIK,DLNLIK,RLAMDA,IMODL,IERR,N)
      WRITE (OUTPUT,3006) DLIK,DLNLIK
 3006 FORMAT (////' SAMPLE LIKELIHOOD (I.E. PROBABILITY OF OBSERVING',
     > ' GIVEN DATA) =',1PG15.6/
     > 10X,'LOGARITHM (NATURAL BASE) OF LIKELIHOOD =',G15.6)
C.......................................................................
C     FITTEST
C        CALL SUBROUTINE KSFIT TO PERFORM KOLMOGOROV-SMIRNOV GOODNESS-
C        OF-FIT TEST OR CALL CHISQ TO PERFORM CHI-SQUARED GOODNESS-OF-
C        FIT TEST OR BOTH DEPENDING ON IMOD.
C.......................................................................
 320  IF (ICMD(9).EQ.0) GOTO 400
      IMOD=R9P(1)
      IF (IMOD.EQ.2) GOTO 400
      IFIT=R9P(2)
      IOUT=R9P(3)
      IIMOD=1
      RLM=RLAMBR
      IF (IFIT.EQ.2) GOTO 315
      WRITE (OUTPUT,3007)
 3007 FORMAT (////' KOLMOGOROV-SMIRNOV DISCRETE GOODNESS-OF-FIT TEST:')
      CALL KSFIT(HD,D,IOUT,SIG1,SIG2)
      IF (IFIT.EQ.1) GOTO 400
 315  WRITE (OUTPUT,3008)
 3008 FORMAT (////' CHI-SQUARED GOODNESS-OF-FIT TEST:')
      LDF=R9P(4)
      EXPMIN=R9P(5)
      MAXGPS=R9P(6)
      SIGDIG=1.0D-05
      CALL CHISQ(HD,EXPMIN,MAXGPS,LDF,IOUT,SIGDIG,CHI2,NDF,SIG,NZ)
      IF (NZ.EQ.-1) WRITE (OUTPUT,3009)
 3009 FORMAT('-*****WARNING: DIMENSION OF WORK SPACE VECTOR TOO SMALL',
     > ' TO PERFORM TEST')
      IF (NZ.GT.0) WRITE (OUTPUT,3010) NZ
 3010 FORMAT('-*****WARNING: INACCURATE RESULTS IN CALCULATIONS.',
     > ' RETURN CODE=',I6)
C.......................................................................
C     PARMEST
C        CHECK TO SEE IF PRIOR DISTRIBUTION IS SPECIFIED AND WHETHER
C        PARAMETERS ARE GIVEN OR NEED TO BE ESTIMATED. CALL SUBROUTINE
C        ABESTD TO CONVERT USER RANGE PARAMETERS OF 'P' INTO
C        APPROPRIATE THETA OR A AND B DISTRIBUTION PARAMETERS.
C.......................................................................
 400  DO 401 I=11,25
      IF (ICMD(I).EQ.1) GOTO 405
 401  CONTINUE
      GOTO 888
 405  IF (ICMD(7).EQ.1) GOTO 406
      WRITE (OUTPUT,4000) IEX
 4000 FORMAT ('-*****WARNING: REQUIRED COMMAND "PARMEST" NOT SPECIFIED.'
     >,' NO FURTHER COMMANDS MAY BE PERFORMED FOR EXECUTION: ',I4)
      GOTO 888
 406  WRITE (OUTPUT,4001)
      WRITE (OUTPUT,4002)
      ITMP=IABS(I7P1)
      WRITE (OUTPUT,4003) TDST(ITMP)
 4001 FORMAT ('1',40X,47('*'),/41X,'*',45X,'*',/41X,'*',7X,
     > 'COMPOUND BINOMIAL-PRIOR MODEL:',T88,'*')
 4002 FORMAT (41X,'*  THE FAILURE PROBABILTIES ARE DISTRIBUTED   *',/
     >41X,'*',5X,'ACCORDING TO THE PRIOR DISTRIBUTION.',T88,'*')
 4003 FORMAT (41X,'*    THE PRIOR DISTRIBUTION IS: ',A11,T88,'*',/
     >41X,'*',45X,'*',/41X,47('*'))
      WRITE (OUTPUT,4017)
 4017 FORMAT(///,1X,40('*'),' PRIOR ANALYSIS ',40('*'))
      IPRIR=I7P1
      PARM(1)=R7P(1)
      PARM(2)=R7P(2)
      DLMIN=R7P(3)
      DLMAX=R7P(4)
      CALL ABESTD(DLMIN,DLMAX,IPRIR,AA,BB)
      PARM(3)=AA
      PARM(4)=BB
      IMETH=I7P6
      IF (IMETH.GT.0) GOTO 402
      WRITE (OUTPUT,4018)
 4018 FORMAT(//' USER ESTIMATES OF ALPHA AND BETA:')
      GOTO 430
C.......................................................................
C     PARMEST      7A
C        CALL SUBROUTINE MMOM1D TO CALCULATE PRIOR MATCHING MOMENTS OR
C        MARGINAL MATCHING MOMENTS ESTIMATES FOR ALPHA AND BETA DEPEN-
C        DING ON IMETH.
C.......................................................................
 402  IF (IMETH.EQ.3) GOTO 420
      NNS=I6P1
      DO 407 I=1,NNS
      FI(I)=NFL(I)
      TI(I)=NTM(I)
 407  CONTINUE
      IIDIST=IMETH
      IMPRNT=0
      ISPRNT=0
      IDID=0
      IOUT=OUTPUT
      NEV=R7AP(1)
      IEND=NEV
      ERR=R7AP(2)
      EPS0=R7AP(3)
      EPSS=R7AP(4)
      S(1)=DABS(R7AP(5))
      S(2)=DABS(R7AP(6))
      SRANG=R7AP(7)
      IEPRNT=R7AP(8)
      IF (IIDIST.EQ.1) WRITE (OUTPUT,4004)
      IF (IIDIST.EQ.2) WRITE (OUTPUT,4005)
 4004 FORMAT(//' PRIOR MATCHING MOMENTS ESTIMATION METHOD:')
 4005 FORMAT(//' MARGINAL MATCHING MOMENTS ESTIMATION METHOD:')
      CALL MMOM1D
      PARM(1)=ANS(1)
      PARM(2)=ANS(2)
      GOTO 430
C.......................................................................
C     PARMEST      7B
C        CALL SUBROUTINE MLSIMD TO CALCULATE MAXIMUM LIKELIHOOD
C        ESTIMATES OF ALPHA AND BETA USING SIMPLEX METHOD.
C.......................................................................
 420  WRITE (OUTPUT,4006)
 4006 FORMAT(//' MAXIMUM LIKELIHOOD ESTIMATION METHOD:')
      NND=I6P1
      DO 421 I=1,NND
      KD(I)=NFL(I)
      ND(I)=NTM(I)
 421  CONTINUE
      AN(1)=R7P(1)
      AN(2)=R7P(2)
      NEV=R7BP(1)
      EPSI=R7BP(2)
      ACEPS=R7BP(3)
      IPRT=R7BP(4)
      ISTEP=R7BP(5)
      IF (ISTEP.EQ.0) GOTO 425
      GOTO (422,424),ISTEP
 422  SMAXL(1)=0.0D0
      GOTO 426
 424  SMAXL(1)=DABS(R7BP(6)*100)
      GOTO 426
 425  SMAXL(1)=-DABS(R7BP(6))
      SMAXL(2)=DABS(R7BP(7))
 426  CALL MLSIMD(EPSI,AN,SMAXL,NEV,Y,IER,IPRT)
      IF (IER.NE.0) WRITE (OUTPUT,4007) IER
 4007 FORMAT('-*****WARNING: INACCURATE RESULTS IN MAXIMUM LIKELIHOOD',
     > ' METHOD. RETURN CODE=',I6)
      PARM(1)=AN(1)
      PARM(2)=AN(2)
 430  WRITE (OUTPUT,4008) PARM(1),PARM(2)
 4008 FORMAT (21X,'ESTIMATED PRIOR PARAMETERS:   ALPHA=',1PG11.4,
     > '    BETA=',G11.4)
      CALL RANGED
C.......................................................................
C     VARIANCE
C        CALL EITHER MMVARD OR VARMLD TO CALCULATE CORRESPONDING
C        VARIANCES AND COVARIANCE OF THE ESTIMATORS.
C.......................................................................
      IF (ICMD(11).EQ.0) GOTO 440
      IF (IMETH.EQ.0) GOTO 440
      IF (IMETH.EQ.3) GOTO 437
      IOUT=OUTPUT
      ERR=1.0D-6
      IMPRNT=0
      IIDIST=IMETH
      NMMV=I6P1
      CALL MMVARD(SIGMA,VAR,NMMV,IIDIST,IVERR)
      R=VAR(3)/DSQRT(VAR(1)*VAR(2))
      WRITE (OUTPUT,4009) VAR(1),VAR(2),VAR(3),R
 4009 FORMAT (21X,'VARIANCE OF THE ESTIMATORS:   VAR(ALPHA)=',1PG11.4,
     > '  VAR(BETA)=',G11.4/51X,'COVAR(ALPHA,BETA)=',G11.4,
     > '  CORRELATION R =',G11.4)
      IF (IVERR.NE.0) WRITE (OUTPUT,4010) IVERR
 4010 FORMAT('-*****WARNING: INACCURATE RESULTS IN VARIANCE/COVARIANCE',
     >' ESTIMATES. RETURN CODE=',I6)
      GOTO 440
 437  IOPT=I11P2
      ACEPS=1.0D-6
      DALPH=-10.0D0
      DBETA=0.0D0
      NSIG=4
      AP=PARM(1)
      BT=PARM(2)
      IPRT=0
      CALL VARMLD(AP,BT,DALPH,DBETA,V,NSIG,IER,IOPT,IPRT)
      R=V(1,2)/DSQRT(V(1,1)*V(2,2))
      WRITE(OUTPUT,4009) V(1,1),V(2,2),V(1,2),R
      IF (IER.EQ.2) WRITE (OUTPUT,4010) IER
      IF (IER.EQ.1) WRITE (OUTPUT,4011) NSIG
 4011 FORMAT('-*****WARNING: VARIANCE/COVARINCE ACCURATE TO ONLY ',I2
     >,' SIGNIFICANT DIGITS')
C.......................................................................
C     PRIRMOMS
C        CALL SUBROUTINE PRMCMD TO CALCULATE THE MOMENTS ABOUT ZERO
C        AND THE CENTRAL MOMENTS FOR THE PRIOR DISTRIBUTION. ALSO
C        CALCULATE SKEWNESS AND KURTOSIS.
C.......................................................................
 440  IF (ICMD(12).EQ.0) GOTO 450
      CALL PRMOMD(PW)
      WRITE (OUTPUT,4012) PW(1),PW(2),PW(3),PW(4)
 4012 FORMAT(//' PRIOR MOMENTS ABOUT ZERO',5X,'FIRST: ',1PG11.4,
     > ' SECOND: ',G11.4,' THIRD: ',G11.4,' FOURTH: ',G11.4)
      WRITE (OUTPUT,4013) PW(5),PW(6),PW(7),PW(8)
 4013 FORMAT(/' PRIOR CENTRAL MOMENTS',8X,'FIRST: ',1PG11.4,
     > ' SECOND: ',G11.4,' THIRD: ',G11.4,' FOURTH: ',G11.4)
      WRITE (OUTPUT,4019) PW(9),PW(10)
 4019 FORMAT(/' SKEWNESS: ',1PG11.4,5X,'KURTOSIS: ',G11.4)
C.......................................................................
C     PRIRPCTL
C        USE PRCNTD FUNCTION TO CALCULATE THE ALP AND 1-ALP PERCENTILES
C        FOR THE PRIOR DISTRIBUTION.
C.......................................................................
 450  IF (ICMD(16).EQ.0) GOTO 460
      ALP=R16P2
      ALP1=1.0D0-ALP
      RTXL=RNG(1)
      RTXR=RNG(2)
      RTEPS=1.0D-6
      IRTEND=250
      RLALP=PRCNTD(ALP)
      IRT=IRTIER
      RLALP1=PRCNTD(ALP1)
      IRT=IRT+IRTIER
      WRITE (OUTPUT,4020) ALP,RLALP,ALP1,RLALP1
 4020 FORMAT (//' PRIOR PROBABILITY INTERVAL:   P(',F5.3,' LEVEL)=',
     > 1PG11.4,'      P(',0PF5.3,' LEVEL)=',1PG11.4)
      IF (IRT.NE.0) WRITE (OUTPUT,4021)
 4021 FORMAT('-*****WARNING: POSSIBLE INACCURACIES IN PRIOR PROBABILITY'
     >,' INTERVAL')
C.......................................................................
C     PRIRMINT
C        CALL SUBROUTINE PUPJKS TO CALCULATE THE MINIMUM INTERVAL FOR P
C        SUCH THAT P-UPPER EQUALS P-LOWER WITH TOTAL AREA OUTSIDE THE
C        INTERVAL EQUAL TO ALPHA FOR THE PRIOR DISTRIBUTION.
C.......................................................................
 460  IF (ICMD(15).EQ.0) GOTO 470
      ALPHA=R15P2
      XZL=RNG(1)
      ACEPS=1.0D-6
      XZU=RNG(2)
      IDST=1
      WRITE (OUTPUT,4015)
      CALL PUPJKS(XZL,XZU,ALPHA,IDST,FV,IER)
      TOTAL=FV(3)+FV(4)
      TTT=1.0D0-FV(4)
      WRITE (OUTPUT,4014) FV(3),FV(1),FV(5),TTT,FV(2),FV(6),TOTAL
 4014 FORMAT (/' LOWER P(',1PG11.4,' LEVEL)=',G11.4,' WITH DENSITY=',
     >G11.4,/' UPPER P(',G11.4,' LEVEL)=',G11.4,' WITH DENSITY=',
     >G11.4,/' FOR A TOTAL AREA OUTSIDE THE INTERVAL OF ',G11.4)
 4015 FORMAT(//' PRIOR MINIMUM INTERVAL:')
      IF (IER.GT.0) WRITE (OUTPUT,4016) IER
 4016 FORMAT('-*****WARNING: INACCURATE RESULTS IN MINIMUM INTERVAL',
     > ' CALCULATIONS. RETURN CODE=',I6)
C.......................................................................
C     PRIRTABL
C        CALL SUBROUTINE TABLED TO TABULATE PRIOR DISTRIBUTION AND
C        CUMULATIVE PRIOR DISTRIBUTION.
C.......................................................................
 470  IF (ICMD(13).EQ.0) GOTO 490
      IMETH=I13P2
      IF (IMETH.EQ.4.OR.IMETH.EQ.1) GOTO 480
      PTS(1)=R13PB(1)
      PTS(2)=R13PB(2)
      NPTS=I13PB3
      IDISTT=1
      IOUT=OUTPUT
      CALL TABLED(PTS,NPTS,IMETH,IDISTT,IOUT,OUTLN,'        ')
      GOTO 490
 480  NPTS=I13PA3
      DO 485 I=1,NPTS
      PTS(I)=R13PA(I)
 485  CONTINUE
      IDISTT=1
      IOUT=OUTPUT
      CALL TABLED(PTS,NPTS,IMETH,IDISTT,IOUT,OUTLN,'        ')
C.......................................................................
C     PRIRPLOT
C        USING VALUES OBTAINED FROM SUBROUTINE TABLED CALL SUBROUTINE
C        PLOT TO CREATE LINE PRINTER PLOTS OF THE PRIOR DISTRIBUTION
C        AND/OR THE CUMULATIVE PRIOR DISTRIBUTION.
C.......................................................................
 490  IF (ICMD(14).EQ.0) GOTO 500
      PTS(1)=R14P3
      PTS(2)=1.0D0-R14P3
      NPTS=40
      IDISTT=1
      IOUT=0
      IMETH=3
      CALL TABLED(PTS,NPTS,IMETH,IDISTT,IOUT,OUTLN,'        ')
      IDC=I14P2
      IF (IDC.EQ.2) GOTO 495
      DO 491 I=1,NPTS
      A(I,1)=OUTLN(I,1)
      A(I,2)=OUTLN(I,2)
 491  CONTINUE
      N=NPTS
      M=2
      NL=0
      NO=NO+1
      NS=1
      IXOUT=4
      AXMX=0.0D0
      AXMN=0.0D0
      IOUT=OUTPUT
      CALL PLOT(NO,A,N,M,NL,NS,DPLT,XAX,YAX1,IXOUT,AXMX,AXMN,IOUT)
      IF (IDC.EQ.1) GOTO 500
 495  DO 496 I=1,NPTS
      A(I,1)=OUTLN(I,1)
      A(I,2)=OUTLN(I,3)
 496  CONTINUE
      N=NPTS
      M=2
      NL=0
      NO=NO+1
      NS=1
      IXOUT=4
      AXMX=0.0D0
      AXMN=0.0D0
      IOUT=OUTPUT
      CALL PLOT(NO,A,N,M,NL,NS,CPLT,XAX,YAX2,IXOUT,AXMX,AXMN,IOUT)
C.......................................................................
C     MARGINAL ANALYSIS
C        CHECK TO SEE IF MARGINAL ANALYSIS HAS BEEN REQUESTED.
C.......................................................................
 500  IF (ICMD(9).EQ.1.AND.R9P(1).NE.1) GOTO 501
      IF (ICMD(10).EQ.1.AND.I10P2.NE.1) GOTO 501
      GOTO 600
 501  WRITE (OUTPUT,5001)
 5001 FORMAT(///,'1',40('*'),' MARGINAL ANALYSIS ',40('*'))
C.......................................................................
C     MARGINAL LIKELIHD
C        CALL SUBROUTINE MAXLD TO CALCULATE THE MARGINAL LIKELIHOOD.
C.......................................................................
      IF (ICMD(10).EQ.0) GOTO 520
      IF (I20P2.EQ.1) GOTO 520
      N=I6P1
      DO 507 I=1,N
      FI(I)=NFL(I)
      TI(I)=NTM(I)
 507  CONTINUE
      IMODL=1
      ACEPS=1.0D-8
      CALL MAXLD(DLIK,DLNLIK,RLAMDA,IMODL,IER,N)
      WRITE (OUTPUT,3006) DLIK,DLNLIK
      IF (IER.NE.1) WRITE (OUTPUT,4007) IER
C.......................................................................
C     MARGINAL FITTEST
C        CALL SUBROUTINE KSFIT TO PERFORM KOLMOGOROV-SMIRNOV GOODNESS-
C        OF-FIT TEST OR CALL CHISQ TO PERFORM CHI-SQUARED GOODNESS-OF-
C        FIT TEST OR BOTH DEPENDING ON IMOD.
C.......................................................................
 520  IF (ICMD(9).EQ.0) GOTO 600
      IMOD=R9P(1)
      IF (IMOD.EQ.1) GOTO 600
      IPRT=OUTPUT
      IFIT=R9P(2)
      IOUT=R9P(3)
      IIMOD=2
      NN=I6P1
      ACEPS=1.0D-8
      DO 523 I=1,NN
      NF(I)=NFL(I)
      NT(I)=NTM(I)
      CID(I)=COMPID(I)
 523  CONTINUE
      IF (IFIT.EQ.2) GOTO 525
      WRITE (OUTPUT,3007)
      CALL KSFIT(HD,D,IOUT,SIG1,SIG2)
      IF (IFIT.EQ.1) GOTO 600
 525  LDF=R9P(4)
      EXPMIN=R9P(5)
      MAXGPS=R9P(6)
      SIGDIG=1.0D-5
      WRITE (OUTPUT,3008)
      CALL CHISQ(HD,EXPMIN,MAXGPS,LDF,IOUT,SIGDIG,CHI2,NDF,SIG,NZ)
      IF (NZ.GT.0) WRITE (OUTPUT,3010) NZ
      IF (NZ.EQ.-1) WRITE (OUTPUT,3009)
C.......................................................................
C     COMPONTS
C        CHECK TO SEE IF DATA HAS BEEN GIVEN AND IF COMPONENTS SPECIFIED
C        FOR POSTERIOR ANALYSIS EXIST.
C.......................................................................
 600  DO 601 I=17,21
      IF (ICMD(I).EQ.1) GOTO 602
 601  CONTINUE
      GOTO 700
 602  WRITE (OUTPUT,6000)
 6000 FORMAT('1',40('*'),' POSTERIOR ANALYSIS ',40('*'))
      WRITE (OUTPUT,4008) PARM(1),PARM(2)
      IF (ICMD(6).EQ.1) GOTO 603
      WRITE (OUTPUT,6007)
 6007 FORMAT('-*****WARNING: DATA HAS NOT BEEN GIVEN. POSTERIOR',
     > ' ANALYSIS CANNOT NOT BE PERFORMED')
      GOTO 700
 603  IF (ICMD(22).EQ.1) GOTO 604
      IPIDX=I6P1
      GOTO 605
 604  IPIDX=I22PN
 605  DO 606 I=1,IPIDX
      IF (I22P(I).LE.0.OR.I22P(I).GT.I6P1) GOTO 607
 606  CONTINUE
      GOTO 610
 607  WRITE (OUTPUT,6006) I
 6006 FORMAT('-','*****WARNING: COMPONENT NUMBER ',I4,' DOES NOT EXIST I
     >N DATA SET.'/' POSTERIOR ANALYSIS TERMINATED')
      GOTO 700
C.......................................................................
C     POSTMOMS
C        CALL SUBROUTINE PSMOMD TO CALCULATE THE MOMENTS ABOUT ZERO
C        AND THE CENTRAL MOMENTS FOR THE POSTERIOR DISTRIBUTION OF
C        EACH COMPONENT SPECIFIED. ALSO CALCULATE SKEWNESS AND KURTOSIS.
C.......................................................................
 610  IF (ICMD(17).EQ.0) GOTO 625
      ACEPS=1.0D-6
      DO 620 I=1,IPIDX
      KK=I22P(I)
      KKF=NFL(KK)
      NNT=NTM(KK)
      WRITE (OUTPUT,6004) COMPID(KK),KKF,NNT
 6004 FORMAT(///' FOR COMPONENT: ',A8,' F= ',I7,' T= ',I7)
      CALL PSMOMD (PMOM,IERR)
      WRITE (OUTPUT,6001) PMOM(7),PMOM(8),PMOM(9),PMOM(10)
 6001 FORMAT(' POSTERIOR MOMENTS ABOUT ZERO',5X,'FIRST: ',G11.4,' SECOND
     >: ',G11.4,' THIRD: ',G11.4,' FOURTH: ',G11.4)
      WRITE (OUTPUT,6002) PMOM(1),PMOM(2),PMOM(3),PMOM(4)
 6002 FORMAT(/' POSTERIOR CENTRAL MOMENTS ',8X,'FIRST: ',G11.4,' SECOND:
     > ',G11.4,' THIRD: ',G11.4,' FOURTH: ',G11.4)
      WRITE (OUTPUT,4019) PMOM(5),PMOM(6)
      IF (IERR.NE.0) WRITE (OUTPUT,6003) IERR
 6003 FORMAT('-','*****WARNING: POSSIBLE INACCURATE VALUES IN MOMENTS ',
     >I1,' AND HIGHER')
 620  CONTINUE
C.......................................................................
C     POSTCOMP
C        CALL SUBROUTINE PSTCPD TO PERFORM COMPARISON OF HOMOGENEOUS AND
C        POSTERIOR MODELS FOR EACH COMPONENT SPECIFIED.
C.......................................................................
 625  IF (ICMD(18).EQ.0) GOTO 640
      NN=IPIDX
      DO 626 K=1,NN
      I=I22P(K)
      CID(K)=COMPID(I)
      NF(K)=NFL(I)
      NT(K)=NTM(I)
 626  CONTINUE
      ACEPS=1.0D-6
      NITER=250
      PCT=R18P2
      IPRT=OUTPUT
      EPSDR=1.0D-6
      CALL PSTCPD(PCT,EPSDR)
C.......................................................................
C     POSTMINT
C        CALL SUBROUTINE PUPJKS TO CALCULATE THE MINIMUM INTERVAL FOR P
C        SUCH THAT P-UPPER EQUALS P-LOWER WITH TOTAL AREA OUTSIDE THE
C        INTERVAL EQUAL TO ALPHA FOR THE POSTERIOR DISTRIBUTION OF EACH
C        COMPONENT SPECIFIED.
C.......................................................................
 640  IF (ICMD(21).EQ.0) GOTO 655
      ALPHA=R21P2
      WRITE (OUTPUT,6005)
 6005 FORMAT(//' POSTERIOR MINIMUM INTERVAL:')
      ACEPS=1.0D-6
      IDST=2
      XZL=RNG(1)
      ACEPS=1.0D-6
      DO 650 I=1,IPIDX
      KK=I22P(I)
      KKF=NFL(KK)
      NNT=NTM(KK)
      WRITE (OUTPUT,6004) COMPID(KK),KKF,NNT
      IMRGCL=0
      XZU=RNG(2)
      CALL PUPJKS (XZL,XZU,ALPHA,IDST,FV,IER)
      TOTAL=FV(3)+FV(4)
      TTT=1.0D0-FV(4)
      WRITE (OUTPUT,4014) FV(3),FV(1),FV(5),TTT,FV(2),FV(6),TOTAL
      IF (IER.GT.0) WRITE (OUTPUT,4016) IER
 650  CONTINUE
C.......................................................................
C     POSTTABL
C        CALL SUBROUTINE TABLED TO TABULATE POSTERIOR DISTRIBUTION AND
C        CUMULATIVE POSTERIOR DISTRIBUTION OF EACH COMPONENT SPECIFIED.
C.......................................................................
 655  IF (ICMD(19).EQ.0) GOTO 670
      IMETH=I19P2
      IOUT=OUTPUT
      IDISTT=2
      ACEPS=1.0D-6
      IF (IMETH.EQ.4.OR.IMETH.EQ.1) GOTO 660
      PTS(1)=R19PB(1)
      PTS(2)=R19PB(2)
      NPTS=I19PB3
      GOTO 665
 660  NPTS=I19PA3
      DO 662  I=1,NPTS
      PTS(I)=R13PA(I)
 662  CONTINUE
 665  DO 667 I=1,IPIDX
      KK=I22P(I)
      KKF=NFL(KK)
      NNT=NTM(KK)
      IMRGCL=0
      CALL TABLED(PTS,NPTS,IMETH,IDISTT,IOUT,OUTLN,COMPID(KK))
 667  CONTINUE
C.......................................................................
C     POSTPLOT
C        USING VALUES OBTAINED FROM SUBROUTINE TABLED CALL SUBROUTINE
C        PLOT TO CREATE LINE PRINTER PLOTS OF THE POSTERIOR DISTRIBUTION
C        AND/OR THE CUMULATIVE POSTERIOR DISTRIBUTION OF EACH COMPONENT
C        SPECIFIED.
C.......................................................................
 670  IF (ICMD(20).EQ.0) GOTO 700
      PTS(1)=R20P3
      PTS(2)=1.0D0-R20P3
      NPTS=40
      IDISTT=2
      IOUT=0
      ACEPS=1.0D-6
      IMETH=3
      IDC=I20P2
      N=NPTS
      M=2
      NL=0
      NS=1
      IXOUT=4
      AXMX=0.0D0
      AXMN=0.0D0
      IPOUT=OUTPUT
      DO 680 I=1,IPIDX
      KK=I22P(I)
      KKF=NFL(KK)
      NNT=NTM(KK)
      IMRGCL=0
      CALL TABLED(PTS,NPTS,IMETH,IDISTT,IOUT,OUTLN,'        ')
      IF (IDC.EQ.2) GOTO 675
      DO 671 J=1,NPTS
      A(J,1)=OUTLN(J,1)
      A(J,2)=OUTLN(J,2)
 671  CONTINUE
      PPLTD(49:56)=COMPID(KK)
      NO=NO+1
      CALL PLOT(NO,A,N,M,NL,NS,PPLTD,XAX,YAX1,IXOUT,AXMX,AXMN,IPOUT)
      IF (IDC.EQ.1) GOTO 680
 675  CONTINUE
      DO 676 J=1,NPTS
      A(J,1)=OUTLN(J,1)
      A(J,2)=OUTLN(J,3)
 676  CONTINUE
      NO=NO+1
      PPLTC(49:56)=COMPID(KK)
      CALL PLOT(NO,A,N,M,NL,NS,PPLTC,XAX,YAX2,IXOUT,AXMX,AXMN,IPOUT)
 680  CONTINUE
 700  GOTO 888
 888  WRITE (OUTPUT,8001) IEX
 8001 FORMAT('0','END PROCESSING FOR EXECUTION ',I4)
      CALL DATEPR(OUTPUT)
      GOTO 100
 990  WRITE (OUTPUT,9001) IEX
 9001 FORMAT('0','COMMAND INPUT ERRORS FOR EXECUTION ',I4)
      WRITE (OUTPUT,9002)
 9002 FORMAT('0','ABNORMAL TERMINATION OF PROGRAM')
      CALL DATEPR(OUTPUT)
      STOP
 999  WRITE (OUTPUT,9003) IEX
 9003 FORMAT('0NORMAL TERMINATION OF PROGRAM AFTER ',I4,' EXECUTIONS')
      CALL DATEPR(OUTPUT)
      STOP
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE DATEPR(OUTPUT)
C*                                                                     *
C***********************************************************************
      INTEGER     OUTPUT
C FOLLOWING 4 STATEMENTS FOR MICROSOFT FORTRAN
C     INTEGER*2   IHS,ISEC,IMIN,IH,ITG,IMO,IJA
C     CALL GETDAT(IJA,IMO,ITG)
C     CALL GETTIM(IH,IMIN,ISEC,IHS)
C     WRITE(OUTPUT,1007) ITG, IMO, IJA, IH, IMIN, ISEC
C FOLLOWING 3 STATEMENTS FOR IBM MAINFRAME VSFORTRAN
C     INTEGER     NDT(8)
C     CALL DATIM (NDT)
C     WRITE(OUTPUT,1007) NDT(6),NDT(7),NDT(8),NDT(5),NDT(4),NDT(3)
C FOLLOWING X STATEMENTS FOR FORTRAN 90
      CHARACTER  CDATE*8, CTIME*10, CZONE*5
      INTEGER    I(8)
      CALL DATE_AND_TIME(CDATE,CTIME,CZONE,I)
      WRITE(OUTPUT,1007) I(3),I(2),I(1),I(5),I(6),I(7)
C
1007  FORMAT('0',20('*'),7X,'PROGRAM SAFE-D',8X,20('*')/
     > ' ',20('*'),4X,I2,'.',I2,'.',I4,
     > 3X,I2.2,':',I2.2,':',I2.2,4X,20('*'))
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE INITD
C*                                                                     *
C***********************************************************************
C*                                                                     *
C*  THIS ROUTINE SETS THE DEFAULT PARAMETER VALUES FOR EACH COMMAND.   *
C*  THESE VALUES ARE USED FOR DEFAULT VALUES WHENEVER A PARAMETER IS   *
C*  UNSPECIFIED IN A COMMAND. THE PARAMETERS DO HAVE A NAMING SCHEME   *
C*  FOR IDENTIFICATION. REFER TO THE USER'S GUIDE FOR MORE INFORMATION.*
C*  REFER TO THE USER'S GUIDE FOR COMMON BLOCK NAMES ALSO.             *
C*                                                                     *
C*  NOTE:  THESE DEFAULT VALUES REMAIN IN EFFECT UNTIL THE USER        *
C*         SPECIFIES A DIFFERENT VALUE FOR THE COMMAND PARAMETER.      *
C*                                                                     *
C*                                                                     *
C*  WRITTEN BY: S. D. HANSEN                                           *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CMD/   ICMD(25)
      CHARACTER*80 TITLE
      COMMON /CMD6/  TITLE,I6P1
      CHARACTER*8 COMPID(500)
      COMMON /CMD6A/ COMPID,NFL(500),NTM(500)
      COMMON /CMD7/  R7P(4),I7P1,I7P6
      COMMON /CMD7A/ R7AP(8)
      COMMON /CMD7B/ R7BP(7)
      COMMON /CMD8/  R8P2
      COMMON /CMD9/  R9P(6)
      COMMON /CMD10/ I10P2
      COMMON /CMD11/ I11P2
C%%   COMMON /CMD12/
      COMMON /CMD13/ R13PA(40),R13PB(2),I13P2,I13PA3,I13PB3
      COMMON /CMD14/ R14P3,I14P2
      COMMON /CMD15/ R15P2
      COMMON /CMD16/ R16P2
C%%   COMMON /CMD17/
      COMMON /CMD18/ R18P2
      COMMON /CMD19/ R19PA(40),R19PB(2),I19P2,I19PA3,I19PB3
      COMMON /CMD20/ R20P3,I20P2
      COMMON /CMD21/ R21P2
      COMMON /CMD22/ I22P(500),I22PN
      ICMD(1)=0
      ICMD(2)=0
      ICMD(3)=0
      ICMD(4)=0
      ICMD(5)=0
      DO 444 I=7,25
      ICMD(I)=0
 444  CONTINUE
C
C     DATA         6
C
C*********************************
C
C     PARMEST      7
C
      I7P6=0
C*********************************
C
C     PARMEST 7A
C
      R7AP(1)=500
      R7AP(2)=1.0D-10
      R7AP(3)=1.0D-7
      R7AP(4)=1.0D-10
      R7AP(5)=0.0D0
      R7AP(6)=0.0D0
      R7AP(7)=20.0D0
      R7AP(8)=0
C*********************************
C
C     PARMEST 7B
C
      R7BP(1)=500
      R7BP(2)=1.0D-5
      R7BP(3)=1.0D-6
      R7BP(4)=6
      R7BP(5)=1
      R7BP(6)=5
C*********************************
C
C     HOMOMODL     8
C
      R8P2=5.0D-2
C*********************************
C
C     FITTEST      9
C
      R9P(1)=1
      R9P(2)=1
      R9P(3)=0
      R9P(4)=1
      R9P(5)=3.0
      R9P(6)=16
C*********************************
C
C     LIKELIHD     10
C
      I10P2=1
C*********************************
C
C     VARIANCE     11
C
      I11P2=1
C*********************************
C
C     PRIRMOMS     12
C
C
C     PRIRTABL     13
C
      I13P2=3
      R13PA(1)=5.0D-2
      R13PA(2)=.10D0
      R13PA(3)=.50D00
      R13PA(4)=.90D00
      R13PA(5)=.95D0
      I13PA3=5
      R13PB(1)=5.0D-2
      R13PB(2)=.95D0
      I13PB3=10
C*********************************
C
C     PRIRPLOT     14
      I14P2=1
      R14P3=0.05D0
C*********************************
C
C
C     PRIRMINT     15
C
      R15P2=.10D0
C*********************************
C
C     PRIRPCTL     16
C
      R16P2=5.0D-02
C*********************************
C
C     POSTMOMS     17
C
C
C     POSTCOMP     18
C
      R18P2=.05D0
C*********************************
C
C     POSTTABL     19
C
      I19P2=3
      R19PA(1)=5.0D-2
      R19PA(2)=.10D0
      R19PA(3)=.50D0
      R19PA(4)=.90D0
      R19PA(5)=.95D0
      I19PA3=5
      R19PB(1)=5.0D-2
      R19PB(2)=.95D0
      I19PB3=10
C*********************************
C
C     POSTPLOT     20
C
      I20P2=1
      R20P3=5.0D-2
C*********************************
C
C     POSTMINT     21
C
      R21P2=.10D0
C*********************************
C
C     COMPONTS     22
C
      DO 10 I=1,500
      I22P(I)=I
 10   CONTINUE
      I22PN=500
C*********************************
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE GETRUN(INCOM,IOUT,IRUNOK)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE INPUTS ALL THE COMMANDS UP TO AND INCLUDING THE     *
C* NEXT 'EXECUTE' COMMAND.  THE ROUTINE CALLS SUBROUTINE GETCOM TO     *
C* READ EACH COMMAND AND STORE THE SPECIFIED PARAMETER VALUES IN THE   *
C* DAT ARRAY.  THE ROUTINE REPLACES DEFAULT VALUES SET BY SUBROUTINE   *
C* INITD WITH THE VALUES OF PARAMETERS SPECIFIED IN THE INPUT COMMANDS.*
C* THE VALUES ARE STORED IN THEIR APPROPRIATE COMMON BLOCK VARIABLES   *
C* IN THE /CMD.../ COMMON BLOCKS. FOR MORE INFORMATION ON THE /CMD.../ *
C* COMMON BLOCKS AND THEIR VARIABLES SEE THE USER'S GUIDE.             *
C*                                                                     *
C* INPUT PARAMETERS:                                                   *
C*    INCOM: INPUT UNIT NUMBER FOR COMMAND INPUT CARDS.                *
C*    IOUT:  OUTPUT UNIT NUMBER FOR PRINTING.                          *
C*    IRUNOK: RETURN CODE. =-1 END OF FILE ON UNIT INCOM.              *
C*                         =1  COMMAND INPUT ERRORS FOUND.             *
C*                         =0  COMPLETE RUN FOUND. NO INPUT ERRORS.    *
C*                                                                     *
C* REQUIRED SUBROUTINES: INITD, GETCOM                                 *
C*                                                                     *
C*                                                                     *
C*                                                                     *
C* WRITTEN BY: S. D. HANSEN                                            *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION DAT(100),IDT(100)
      COMMON /CMD/   ICMD(25)
      CHARACTER*80 TITLE
      COMMON /CMD6/  TITLE,I6P1
      CHARACTER*8 COMPID(500)
      COMMON /CMD6A/ COMPID,NFL(500),NTM(500)
      COMMON /CMD7/  R7P(4),I7P1,I7P6
      COMMON /CMD7A/ R7AP(8)
      COMMON /CMD7B/ R7BP(7)
      COMMON /CMD8/  R8P2
      COMMON /CMD9/  R9P(6)
      COMMON /CMD10/ I10P2
      COMMON /CMD11/ I11P2
C%%   COMMON /CMD12/
      COMMON /CMD13/ R13PA(40),R13PB(2),I13P2,I13PA3,I13PB3
      COMMON /CMD14/ R14P3,I14P2
      COMMON /CMD15/ R15P2
      COMMON /CMD16/ R16P2
C%%   COMMON /CMD17/
      COMMON /CMD18/ R18P2
      COMMON /CMD19/ R19PA(40),R19PB(2),I19P2,I19PA3,I19PB3
      COMMON /CMD20/ R20P3,I20P2
      COMMON /CMD21/ R21P2
      COMMON /CMD22/ I22P(500),I22PN
      CHARACTER*8  BLK, COM
      CHARACTER*89 FRM
      DATA BLK /'        '/
C
      FRM(1:8)='( A8,1X,'
      FRM(89:89)=')'
      IRUNOK=0
      ICNT=0
      DO 887 I=1,100
      DAT(I)=0.0D0
 887  CONTINUE
C
 888  CALL GETCOM(IOUT,INCOM,ICOM,COM,DAT,IDT,IDX,IEOF)
      ICNT=ICNT+1
      IF(IEOF.EQ.1) GOTO 992
      IF (ICOM.EQ.-1) GOTO 998
      ICMD(ICOM)=1
      GOTO (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,
     >      24,25), ICOM
C
C     CLEAR
C
 1    CALL INITD
      GOTO 888
C
C     EXECUTE
C
 2    RETURN
C*****************************
 3    GOTO 998
 4    GOTO 998
 5    GOTO 998
C****************************
C
C     DATA
C
 6    IPRM=1
      IF (IDT(1).EQ.0) GOTO 995
      I6P1=DAT(1)
      READ(INCOM,1005,END=999) FRM(9:88)
 1005 FORMAT(A80)
      READ(INCOM,1005,END=999) TITLE
      READ(INCOM,FRM,END=999) (COMPID(I),NFL(I),NTM(I),I=1,I6P1)
      I1=I6P1+1
      DO 61 I=I1,500
      COMPID(I)=BLK
      NFL(I)=0.0D0
      NTM(I)=0.0D0
 61   CONTINUE
      WRITE(IOUT,1007) TITLE
1007  FORMAT('0',20('*'),5X,'DATA HAS BEEN GIVEN',5X,20('*')/
     > ' ',10('*'),5X,A80,5X,10('*'))
      WRITE(IOUT,1008)
1008  FORMAT('0',2X,'COMP. ID',7X,'FAILURES',13X,'TIME',16X,'COMP. ID',
     > 7X,'FAILURES',13X,'TIME',/,3X,8('-'),2(3X,16('-')),10X,8('-'),
     > 2(3X,16('-')))
      WRITE(IOUT,1009) (COMPID(I),NFL(I),NTM(I),I=1,I6P1)
1009  FORMAT(' ',2X,A8,3X,8X,I8,3X,I16,10X,A8,3X,I8,11X,I16)
      GOTO 888
C
C     PARMEST
C
 7    DO 71 I=1,5
      IF (IDT(I).EQ.1) GOTO 71
      IPRM=I
      GOTO 995
 71   CONTINUE
      I7P1=-DABS(DAT(1))
      DO 72 I=1,4
      R7P(I)=DAT(I+1)
 72   CONTINUE
      IF (IDT(6).EQ.1) I7P6=DAT(6)
      IF (I7P6.EQ.0) GOTO 888
      IF (I7P6.EQ.3) GOTO 75
 70   DO 74 I=1,8
      IF (IDT(6+I).EQ.1) R7AP(I)=DAT(6+I)
 74   CONTINUE
      GOTO 888
 75   DO 76 I=1,7
      IF (IDT(6+I).EQ.1) R7BP(I)=DAT(6+I)
 76   CONTINUE
      GOTO 888
C
C     HOMOMODL
C
 8    IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      IF (IDT(2).EQ.1) R8P2=DAT(2)
      GOTO 888
C
C     FITTEST
C
 9    IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      DO 91 I=1,6
      IF (IDT(I+1).EQ.1) R9P(I)=DAT(I+1)
 91   CONTINUE
      GOTO 888
C
C     LIKELIHD
C
 10   IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      GOTO 888
C
C     VARIANCE
C
 11   IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      IF (IDT(2).EQ.1) I11P2=DAT(2)
      GOTO 888
C
C     PRIRMOMS
C
 12   GOTO 888
C
C     PRIRTABL
C
 13   IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      IF (IDT(2).EQ.1) I13P2=DAT(2)
      IF (I13P2.EQ.1.OR.I13P2.EQ.4) GOTO 135
      IF (IDT(3).EQ.1) I13PB3=DAT(3)
      IF (IDT(4).EQ.1) R13PB(1)=DAT(4)
      IF (IDT(5).EQ.1) R13PB(2)=DAT(5)
      GOTO 888
 135  IF (IDT(3).EQ.1) I13PA3=DAT(3)
      DO 136 I=1,I13PA3
      IF (IDT(3+I).EQ.1) R13PA(I)=DAT(3+I)
 136  CONTINUE
      GOTO 888
C
C     PRIRPLOT
C
 14   IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      IF (IDT(2).EQ.1) I14P2=DAT(2)
      IF (IDT(3).EQ.1) R14P3=DAT(3)
      GOTO 888
C
C     PRIRMINT
C
 15   IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      IF (IDT(2).EQ.1) R15P2=DAT(2)
      GOTO 888
C
C     PRIRPCTL
C
 16   IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      IF (IDT(2).EQ.1) R16P2=DAT(2)
      GOTO 888
C
C     POSTMOMS
C
 17   IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      GOTO 888
C
C     POSTCOMP
C
 18   IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      IF (IDT(2).EQ.1) R18P2=DAT(2)
      GOTO 888
C
C     POSTTABL
C
 19   IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      IF (IDT(2).EQ.1) I19P2=DAT(2)
      IF (I19P2.EQ.1.OR.I19P2.EQ.4) GOTO 195
      IF (IDT(3).EQ.1) I19PB3=DAT(3)
      IF (IDT(4).EQ.1) R19PB(1)=DAT(4)
      IF (IDT(5).EQ.1) R19PB(2)=DAT(5)
      GOTO 888
 195  IF (IDT(3).EQ.1) I19PA3=DAT(3)
      DO 196 I=1,I19PA3
      IF (IDT(3+I).EQ.1) R19PA(I)=DAT(3+I)
 196  CONTINUE
      GOTO 888
C
C     POSTPLOT
C
 20   IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      IF (IDT(2).EQ.1) I20P2=DAT(2)
      IF (IDT(3).EQ.1) R20P3=DAT(3)
      GOTO 888
C
C     POSTMINT
C
 21   IF (IDT(1).EQ.1.AND.DAT(1).EQ.0) GOTO 991
      IF (IDT(2).EQ.1) R21P2=DAT(2)
      GOTO 888
C
C     COMPONTS
C
 22   IF (IDX.EQ.1) GOTO 224
      I22PN=0
      DO 221 I=1,IDX
      IF (IDT(I).EQ.0) GOTO 221
      I22PN=I22PN+1
      I22P(I22PN)=DAT(I)
 221  CONTINUE
      GOTO 888
 224  DO 225 I=1,500
      I22P(I)=I
 225  CONTINUE
      I22PN=500
      ICMD(22)=0
      GOTO 991
C***********************************
 23   GOTO 998
 24   GOTO 998
 25   GOTO 998
 991  ICMD(ICOM)=0
      GOTO 888
 992  IF (ICNT.GT.1) GOTO 999
      IRUNOK=-1
      RETURN
 999  WRITE (IOUT,1001)
      WRITE (IOUT,1000)
 1000 FORMAT(' UNEXPECTED ENDFILE ENCOUNTERED OR MISSING EXECUTE',
     >' STATEMENT')
      IRUNOK=1
      RETURN
 998  WRITE(IOUT,1001)
 1001 FORMAT('->>>>>ERROR: ')
      WRITE(IOUT,1002) COM
 1002 FORMAT(' "',A8,'" IS NOT A VALID COMMAND')
      IRUNOK=1
      RETURN
 995  WRITE(IOUT,1001)
      WRITE(IOUT,1003) COM,IPRM
 1003 FORMAT(' FOR COMMAND: "',A8,'" PARAMETER ',I4,
     >' HAS NO DEFAULT AND IS MISSING')
      IRUNOK=1
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE GETCOM(IOUT,INCOM,ICOM,COM,DATIN,IDTFLG,IDX,IEOF)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE READS AND INTERPRETS INDIVIDUAL COMMAND CARDS. THE  *
C* ROUTINE FIRST DETERMINES THE COMMAND KEYWORD INDEX NUMBER. IT THEN  *
C* INTERPRETS THE PARAMETER FIELDS OF THE COMMANDS VIA THE SUBROUTINES *
C* NUMSZE AND SUBSCP.  THE ROUTINE ALSO PRINTS OUT APPROPRIATE ERROR   *
C* MESSAGES FOR IMPROPERLY DEFINED PARAMETERS.  THE ROUTINE READS ALL  *
C* CARDS CORRESPONDING TO A SINGLE COMMAND UNTIL THE FIRST NON-CONTIN- *
C* UED END OF CARD (OR COMMAND DELIMITER) IS ENCOUNTERED.              *
C*                                                                     *
C* REQUIRED SUBROUTINES: NUMSZE,SUBSCP                                 *
C*                                                                     *
C*                                                                     *
C* SUBROUTINE PARAMETERS:                                              *
C*    IOUT: UNIT NUMBER OF OUTPUT DEVICE FOR PRINTING.                 *
C*    INCOM: UNIT NUMBER OF INPUT DEVICE FOR READING COMMANDS.         *
C*    ICOM: INDEX NUMBER OF THE COMMAND KEYWORD. SEE USER'S GUIDE FOR  *
C*          MORE INFORMATION. ICOM=-1 INDICATES THE KEYWORD FOUND IS   *
C*          INVALID.                                                   *
C*    COM: COMMAND KEYWORD FROM INPUT.                                 *
C*    DATIN: ARRAY CONTAINING THE INTERPRETED PARAMETER VALUES         *
C*           IF A COMMAND PARAMETER IS MISSING THAT ARRAY ELEMENT=0.0D0*
C*    IDTFLG: ARRAY OF FLAGS INDICATING WHETHER A PARAMETER IS MISSING *
C*            OR HAS BEEN DEFINED. THIS ARRAY CORRESPONDS TO THE DATIN *
C*            ARRAY. A 1 INDICATES A VALUE HAS BEEN FOUND AND IS STORED*
C*            IN DATIN. A 0 INDICATES THE PARAMETER WAS MISSING.       *
C*    IDX: NUMBER OF PARAMETERS FOUND FOR THE PARTICULAR COMMAND. THIS *
C*         INCLUDES ALL SPECIFICATIONS WHETHER BLANK OR NUMERICAL      *
C*         VALUES.                                                     *
C*    IEOF: INDICATES WHETHER AN END OF FILE HAS OCCURED ON THE INPUT  *
C*          INPUT. 0= NO ENDFILE, 1= ENDFILE HAS OCCURED.              *
C*                                                                     *
C*                                                                     *
C* WRITTEN BY: S. D. HANSEN                                            *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*1 BLK,PNT,HEADER(18), IPARM(80)
      CHARACTER*8 CMDV(25),COM
      DIMENSION DATIN(100),IDTFLG(100)
      DATA BLK/' '/,PNT/'|'/
      DATA HEADER /'I','M','P','R','O','P','E','R',' ','C','H','A'
     >,'R','A','C','T','E','R'/
      DATA CMDV/'CLEAR   ','EXECUTE ','$$$$$$$$','&&&&&&&&','********',
     >'DATA    ','PARMEST ','HOMOMODL','FITTEST ','LIKELIHD','VARIANCE',
     >'PRIRMOMS','PRIRTABL','PRIRPLOT','PRIRMINT','PRIRPCTL','POSTMOMS',
     >'POSTCOMP','POSTTABL','POSTPLOT','POSTMINT','COMPONTS','$&$&$&$&',
     >'<><><><>','.*.*.*.*'/
      DO 5 I=70,80
      IPARM(I)=BLK
 5    CONTINUE
      IEOF=0
      IRD=1
      ICOM=0
      READ (INCOM,1000,END=888) COM,(IPARM(I),I=1,70)
 1000 FORMAT(A8,1X,70A1)
      DO 10 I=1,100
      IDTFLG(I)=0
 10   CONTINUE
      IDX=0
 15   IPT2=1
 16   IPT1=IPT2
      CALL NUMSZE(IPARM,IPT2,ISIZE)
      IF (IPT2.GT.-1) GOTO 20
      IPT2=-IPT2
      IDX=IDX+1
      DATIN(IDX)=0.0D0
      WRITE (IOUT,1004)
 1004 FORMAT('-','>>>>>ERROR:')
      WRITE (IOUT,106) COM,IRD
 106  FORMAT(' FOR COMMAND: "',A8,'"',/
     >' IMPROPER CHARACTER OR CHARACTER SEQUNCE IN CARD NUMBER ',I4)
      IF (IRD.GT.1) GOTO 70
      WRITE (IOUT,101) COM,(IPARM(I),I=1,70)
      WRITE (IOUT,100) (BLK,I=1,IPT2),PNT
      WRITE(IOUT,1003) (BLK,I=1,IPT2),(HEADER(I),I=1,18)
 101  FORMAT(/' INVALID INPUT CARD: ',A8,1X,70A1)
 100  FORMAT(' ',28X,100A1)
 1003 FORMAT(' ',20X,90A1)
      RETURN
 70   WRITE(IOUT,103) (IPARM(I),I=1,80)
 103  FORMAT(/' INVALID INPUT CARD: ',80A1)
      WRITE(IOUT, 104) (BLK,I=1,IPT2),PNT
      WRITE(IOUT,1002) (BLK,I=1,IPT2),(HEADER(I),I=1,18)
 104  FORMAT(' ',19X,100A1)
 1002 FORMAT(' ',11X,90A1)
      RETURN
 20   IF (ISIZE.NE.0) GOTO 30
      IDX=IDX+1
      DATIN(IDX)=0.0D0
      IF (IPT2.EQ.0) GOTO 555
      GOTO 16
 30   CALL SUBSCP(IPT1,VAL,IDX,ISIZE,IPARM)
      DATIN(IDX)=VAL
      IDTFLG(IDX)=1
      IF (IPT2.EQ.0) GOTO 555
      IF (IPT2.NE.81) GOTO 16
      IRD=IRD+1
      READ(INCOM,1001,END=888) (IPARM(I),I=1,80)
 1001 FORMAT(80A1)
      GOTO 15
 888  IEOF=1
      RETURN
 555  DO 556 I=1,25
      IF (COM.NE.CMDV(I)) GOTO 556
      ICOM=I
      RETURN
 556  CONTINUE
      ICOM=-1
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE NUMSZE(IPARM,IJ,ISIZE)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE DETERMINES THE LENGTH OF THE FIELDS OF AN INDIVI-   *
C* DUAL PARAMETER ON A CARD. THE ROUTINE SCANS FOR COMMAS TO DETERMINE *
C* THE LENGTH OF THE FIELD. IN ADDITION IT ALSO ERROR CHECKS (THOUGH   *
C* NOT EXHAUSTIVE) FOR IMPROPER CHARACTERS OR IMPROPERLY DEFINED NUM-  *
C* ERICAL VALUES.  NO SUBROUTINES ARE REQUIRED.                        *
C*                                                                     *
C* SUBROUTINE PARAMETERS:                                              *
C*    IPARM: CARD IMAGE TO BE SCANNED.                                 *
C*    IJ:    STARTING COLUMN FROM WHICH SCANNING IS TO PROCEED ON      *
C*           INPUT. ON OUTPUT, THIS IS A RETURN CODE FLAG.             *
C*           =0 END OF CARD WAS FOUND, =81 CONTINUATION CHARACTER WAS  *
C*           FOUND, <0= ERROR: COLUMN NUMBER OF IMPROPER CHARACTER,    *
C*           >0= NEXT COLUMN PAST THE COMMA DELIMITER FOR THIS FIELD   *
C*    ISIZE: LENGTH OF SCANNED FIELD.                                  *
C*           =0 INDICATES THE PARAMETER VALUE WAS MISSING.             *
C*                                                                     *
C* WRITTEN BY: S. D. HANSEN                                            *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*1 IPARM(80)
      CHARACTER*1 ZERO,NINE,CHAR,PERIOD,SIGN,BLK,COMMA,ASTER,COLON
      DATA ZERO/'0'/,NINE/'9'/,PERIOD/'.'/,SIGN/'-'/,BLK/' '/
      DATA COMMA/','/,ASTER/'*'/,COLON/';'/
      ISIZE=0
      INUM=0
      IS=0
      IPER=0
    2 CHAR=IPARM(IJ)
      IF (CHAR.EQ.COMMA) GOTO 4
      IF (CHAR.EQ.COLON) GOTO 170
      IF (CHAR.GE.ZERO.AND.CHAR.LE.NINE) GOTO 10
      IF (CHAR.NE.PERIOD) GOTO 150
      IF (IPER.EQ.1) GOTO 888
      IPER=1
      GOTO 3
 150  IF(CHAR.NE.SIGN) GO TO 160
      IF (INUM.EQ.1) GOTO 888
      IF (IS.EQ.1) GOTO 888
      IF (IPER.EQ.1) GOTO 888
      IS=1
      GOTO 3
 160  IF (CHAR.EQ.BLK) GOTO 3
 30   IF (CHAR.NE.ASTER) GOTO 888
      IJ=81
      RETURN
 888  IJ=-IJ
      RETURN
 10   INUM=1
    3 ISIZE=ISIZE+1
      IJ=IJ+1
      IF (IJ.LT.81) GOTO 2
 170  IJ=0
      IF (INUM.EQ.0) ISIZE=0
      RETURN
 4    IF (INUM.EQ.0) ISIZE=0
      IJ=IJ+1
    1 RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE SUBSCP(ISTRT,TJT,IDX,ILEN,IPARM)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C* THIS ROUTINE CALCULATES A NUMERICAL VALUE FROM THE CHARACTER REPRE- *
C* SENTATION OF THE VALUE. THE ROUTINE CAN INTERPRET DECIMAL OR        *
C* INTEGER VALUES.  THE SUMMATION OF POWERS METHOD IS USED FOR DETER-  *
C* MINING THE VALUE.  THE ROUTINE CAN HANDLE NEGATIVE VALUES BUT NOT   *
C* SIGNED POSITIVE VALUES.  IN ADDITION, IT ALLOWS FOR IMBEDDED BLANKS *
C* WITHIN THE NUMBER FOR AID IN DISTINGUISHING MAGNITUDE.  NO ADDI-    *
C* TIONAL SUBROUTINES ARE REQUIRED.                                    *
C*                                                                     *
C* SUBROUTINE PARAMETERS:                                              *
C*    ISTRT: FIRST COLUMN OF THE CHARACTER FIELD.                      *
C*    TJT: VALUE CALCULATED FROM THE CHARACTER REPRESENTATION.         *
C*    IDX: COUNTER INCREMENTED BY 1 FOR EACH SUBROUTINE CALL.          *
C*         IDX MUST BE INITIALIZED IN THE CALLING ROUTINE.             *
C*    ILEN: THE LENGTH OF THE FIELD TO BE INTERPRETTED INCLUDING THE   *
C*          ISTRT COLUMN.                                              *
C*    IPARM: ARRAY FOR THE CARD IMAGE WHERE THE CHARACTER FIELD IS     *
C*           STORED.                                                   *
C*                                                                     *
C* WRITTEN BY: S. D. HANSEN                                            *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*1 IPARM(80)
      CHARACTER*1 PERIOD,SIGN,BLANK,COMMA,NUM(10)
      DATA PERIOD/'.'/,SIGN/'-'/,BLANK/' '/,COMMA/','/
      DATA NUM/'0','1','2','3','4','5','6','7','8','9'/
      TJT=0.0D0
      NT=0
      NS=0
      IF (ILEN.NE.0) GOTO 5
      IDX=IDX+1
      RETURN
 5    KK=ISTRT+NS
      IQ=0
      DO 1 IJ=1,ILEN
      ISUB=ISTRT+IJ-1
      IF(IPARM(ISUB).EQ.SIGN) NS=1
      DO 3 KEN=1,10
      IF (IPARM(ISUB).EQ.BLANK) GOTO 1
      IF(IPARM(ISUB ).EQ.NUM(KEN)) GO TO 2
      IF((IPARM(ISUB ).EQ.PERIOD).AND.(IPARM(ISUB +1).NE.COMMA.OR.
     >    IPARM(ISUB+1).NE.BLANK)) NT=1
    3 CONTINUE
      GO TO 1
    2 IF(NT.EQ.1) GO TO 4
      YKEN=KEN-1
      TJT=TJT*10.+YKEN
      GO TO 1
    4 IQ=IQ+1
      TJT=TJT+(KEN-1)*0.10D0**IQ
    1 CONTINUE
      IDX=IDX+1
      IF(NS.EQ.1) TJT=-TJT
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE ECHO (IOUT,IEX)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE ECHOES THE CALCULATIONS TO BE PERFORMED AT THE      *
C* START OF EACH RUN.  THE ROUTINE SIMPLY EXAMINES THE COMMAND INDICA- *
C* TOR ARRAY ICMD(25) AND WRITES THE APPROPRIATE MESSAGE IF THE COM-   *
C* MAND IS IN THE 'ON' STATE (A 1 IN THE CORRESPONDING ELEMENT). NO    *
C* OTHER SUBROUTINES ARE REQUIRED. FOR MORE INFORMATION ON THE /CMD/   *
C* COMMON BLOCK SEE THE USER'S GUIDE.                                  *
C*                                                                     *
C* SUBROUTINE PARAMETERS:                                              *
C*    IOUT: UNIT NUMBER OF THE OUPUT DEVICE FOR PRINTING.              *
C*    IEX: NUMBER OF CURRENT RUN.                                      *
C*                                                                     *
C*                                                                     *
C* WRITTEN BY: S. D. HANSEN                                            *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER ICMD(25)
      COMMON /CMD/ ICMD
      WRITE (IOUT,1000) IEX
 1000 FORMAT ('1FOR EXECUTION ',I4,' THE FOLLOWING IS TO BE PERFORMED:')
      IF (ICMD(8) .EQ. 1) WRITE (IOUT,1008)
1008  FORMAT (' ANALYSIS OF HOMOGENIOUS MODEL')
      IF (ICMD(10) .EQ. 1) WRITE (IOUT,1010)
 1010 FORMAT (' LIKLIHOOD  CALCULATED')
      IF (ICMD(9) .EQ. 1) WRITE (IOUT,1009)
 1009 FORMAT (' CHI-SQUARED AND/OR KOLMOGOROV-SMIRNOV GOODNESS OF FIT',
     >' TESTS')
      IF (ICMD(7) .EQ. 1) WRITE (IOUT,1007)
 1007 FORMAT (' ALPHA AND BETA ESTIMATED (OR HAVE BEEN SPECIFIED)')
      IF (ICMD(11) .EQ. 1) WRITE (IOUT,1011)
 1011 FORMAT (' VARIANCE OF ALPHA AND BETA CALCULATED')
      IF (ICMD(12) .EQ. 1) WRITE (IOUT,1012)
 1012 FORMAT (' PRIOR MOMENTS CALCULATED')
      IF (ICMD(16) .EQ. 1) WRITE (IOUT,1016)
 1016 FORMAT (' PRIOR UPPER AND LOWER PERCENTILES CALCULATED')
      IF (ICMD(15) .EQ. 1) WRITE (IOUT,1015)
 1015 FORMAT (' PRIOR MINIMUM INTERVAL CALCULATED')
      IF (ICMD(13) .EQ. 1) WRITE (IOUT,1013)
 1013 FORMAT (' PRIOR DISTRIBUTION TABULATED')
      IF (ICMD(14) .EQ. 1) WRITE (IOUT,1014)
 1014 FORMAT (' PRIOR DISTRIBUTION PLOTTED')
      IF (ICMD(17) .EQ. 1) WRITE (IOUT,1017)
 1017 FORMAT (' POSTERIOR MOMENTS CALCULATED')
      IF (ICMD(18) .EQ. 1) WRITE (IOUT,1018)
 1018 FORMAT (' COMPARISON OF HOMOGENIOUS AND POSTERIOR MODELS')
      IF (ICMD(21) .EQ. 1) WRITE (IOUT,1021)
 1021 FORMAT (' POSTERIOR MINIMUM INTERVAL CALCULATED')
      IF (ICMD(19) .EQ. 1) WRITE (IOUT,1019)
 1019 FORMAT (' POSTERIOR DISTRIBUTION TABULATED')
      IF (ICMD(20) .EQ. 1) WRITE (IOUT,1020)
 1020 FORMAT (' POSTERIOR DISTRIBUTIONS PLOTTED')
      IF (ICMD(22) .EQ. 1) WRITE (IOUT,1022)
 1022 FORMAT (' COMPONENTS FOR POSTERIOR ANALYSIS HAVE BEEN SPECIFIED')
      IF (ICMD(1) .EQ. 1) WRITE (IOUT,1001)
 1001 FORMAT (' THE ''CLEAR'' COMMAND WAS EXECUTED')
      RETURN
      END
C+***************************** HOMOD **********************************
C*                                                                     *
      SUBROUTINE HOMOD(PBAR,VAR,PLOW,PUP,PCT,IERROR)
C*                                                                     *
C****************************** HOMOD **********************************
C*                                                                     *
C* THIS SUBROUTINE USES A HOMOGENEOUS BINOMIAL MODEL TO ANALYZE GIVEN  *
C* FAILURE-ON-DEMAND DATA. ESTIMATORS FOR THE FAILURE PROBABILITY, P,  *
C* AND ITS VARIANCE ARE CALCULATED TOGETHER WITH THE EQUAL-TAIL CON-   *
C* FIDENCE INTERVAL.                                                   *
C*                                                                     *
C* PROGRAM VARIABLES:                                                  *
C*    PBAR   = RETURNED VALUE FOR ESTIMATED FAILURE PROBABILITY        *
C*    VAR    = RETURNED VALUE OF THE VARIANCE OF PBAR                  *
C*    PLOW   = RETURNED VALUE OF THE LOWER CONFIDENCE LIMIT            *
C*    PUP    = RETURNED VALUE FOR THE UPPER CONFIDENCE LIMIT           *
C*    PCT    = INPUT VALUE FOR THE PROBABILITY TO BE ASSOCIATED WITH   *
C*             EACH TAIL FOR CALCULATION OF THE CONFIDENCE INTERVAL    *
C*    IERROR = RETURNED ERROR CODE: =0 NORMAL RETURN; =1 POSSIBLE      *
C*             ERROR IN LOWER CONFIDENCE LIMIT; =2 POSSIBLE ERROR      *
C*             IN UPPER CONFIDENCE LIMIT.                              *
C*                                                                     *
C* COMMON /DTA/  SUPPLIES FAILURE DATA TO SUBROUTINE                   *
C*    ID(I)  = IDENTIFICATION FOR I-TH COMPONENT - NOT USED            *
C*    NF(I)  = NUMBER OF FAILURES FOR I-TH COMPONENT                   *
C*    NT(I)  = NUMBER OF DEMANDS FOR I-TH COMPONENT                    *
C*    N      = TOTAL NUMBER OF COMPONENTS                              *
C*    IPRT   = UNIT NUMBER FOR PRINTING OF WARNING MESSAGES            *
C*    NITER  = MAXIMUM NUMBER OF ITERATIONS TO BE USED FOR SOLVING     *
C*             FOR THE CONFIDENCE LIMITS                               *
C*                                                                     *
C* SUBROUTINES NEEDED:  FBETA, MDBETA, DRTMI                           *
C*                                                                     *
C* WRITTEN BY J.K.SHULTIS, KANSAS STATE UNIVERSITY, 8/80               *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*8 CID(500)
      COMMON /DTA/ CID,NF(500),NT(500),N,IPRT,NITER
      COMMON /FBET/ARG1,ARG2,PCTL,IPRTT
      EXTERNAL FBETA
C
      EPS=1.0D-5
      IERROR=0
      IPRTT=IPRT
      SUMF=0.0
      SUMT=0.0
C
C CALCULATE SUM OF FAILURES AND TIMES
      DO 10 I=1,N
      SUMF=SUMF+NF(I)
   10 SUMT=SUMT+NT(I)
C
C CALCULATE ESTIMATORS FOR LAMBDA AND ITS VARIANCE
      PBAR=SUMF/SUMT
      VAR=PBAR*(1.D0-PBAR)/SUMT
C
C CALCULATE UPPER AND LOWER CONFIDENCE LIMITS
      IF(SUMF.EQ.0.0) GO TO 30
      IF(SUMF.EQ.SUMT) GO TO 40
C CALCULATE LOWER CONFIDENCE LIMIT
      PCTL=PCT
      ARG1=SUMF
      ARG2=SUMT-SUMF+1.D0
      CALL DRTMI(PLOW,FNVAL,FBETA,0.0D0,1.D0,EPS,NITER,IER)
      IF(IER.EQ.0) GO TO 20
      IERROR=1
      WRITE(IPRT,11)
   11 FORMAT(' *** WARNING: POSSIBLE ERROR IN CALCULATION OF THE',
     >' LOWER CONFIDENCE LIMIT IN SUBROUTINE HOMOD')
C CALCULATE UPPER CONFIDENCE LIMIT
   20 PCTL=1.0 - PCT
      ARG1=SUMF+1.D0
      ARG2=SUMT-SUMF
      CALL DRTMI(PUP,FNVAL,FBETA,0.0D0,1.D0,EPS,NITER,IERR)
      IF(IERR.EQ.0) RETURN
      IERROR=2
      WRITE(IPRT,12)
   12 FORMAT(' *** WARNING* POSSIBLE ERROR IN CALCULATION OF THE',
     >' UPPER CONFIDENCE LIMIT IN SUBROUTINE HOMOD')
      RETURN
C
C SPECIAL CASE FOR SUMF=0
   30 PLOW=0.0
      PUP=1.D0 - PCT**(1.D0/SUMT)
      RETURN
C SPECIAL CASE IF SUMF = SUMT
   40 PUP=1.0
      PLOW=(1.D0-PCT)**(1.D0/SUMT)
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      DOUBLE PRECISION FUNCTION FBETA(X)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C* THIS FUNCTION IS USED ONLY BY THE SUBROUTINES HOMOD AND PSTCPD TO   *
C* FIND THE CONFIDENCE LIMITS OF THE HOMOGENOUS BINOMIAL MODEL.        *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /FBET/ARG1,ARG2,PCT,IPRT
      IF((X.EQ.1.D0).OR.(X.EQ.0.0D0)) GO TO 20
      XX=X
      A=ARG1
      B=ARG2
      CALL MDBETA(XX,A,B,VAL,NZ)
      IF (NZ.NE.0) WRITE (IPRT,1)
      FBETA=VAL-PCT
      RETURN
   20 FBETA=X-PCT
      RETURN
    1 FORMAT(' *** WARNING: POSSIBLE ERROR IN EVALUATION OF THE',
     >' INCOMPLETE BETA FUNCTION IN SUBROUTINE FBETA')
      END
C+***************************** KSFIT **********************************
C*                                                                     *
      SUBROUTINE KSFIT(H,D,IOUT,SIG1,SIG2)
C*                                                                     *
C****************************** KSFIT **********************************
C*                                                                     *
C* PURPOSE:                                                            *
C* THIS SUBROUTINE CALCULATES THE KOLMOGOROV-SMIRNOV GOODNESS-OF-FIT   *
C* STATISTIC BETWEEN GIVEN FAILURE-RATE ATTRIBUTE DATA AND A SPECIFIED *
C* FAILURE RATE STATISTICAL MODEL.  BOUNDS FOR THE SIGNIFICANCE LEVEL  *
C* OF THE FIT STATISTIC ARE ESTIMATED FROM THE EXACT BOUNDS FOR SMALL  *
C* SAMPLE SIZES ( <51 ).                                               *
C*                                                                     *
C* SUBROUTINE VARIABLES:                                               *
C*    H     = NAME OF EXTERNAL FUNCTION WHICH GIVES THE PROBABILITY    *
C*            OF OBSERVING FF FAILURES IN TIME TT, I.E. H(FF,TT)       *
C*    D     = VALUE OF THE GOODNESS-OF-FIT STATISTIC                   *
C*    IOUT  = PARAMETER CONTROLLING INTERMEDIATE OUTPUT; IF = 0 NO     *
C*            INTERMEDIATE OUTPUT GIVEN; IF > 0 INTERMEDIATE OUTPUT    *
C*            PRINTED                                                  *
C*    SIG1  = LOWER BOUND ON THE SIGNIFICANCE LEVEL                    *
C*    SIG2  = UPPER BOUND ON THE SIGNIFICANCE LEVEL                    *
C*                                                                     *
C* COMMON BLOCK /DTA/ CID(500),NF(500),NT(500),N,NIPRT                 *
C*    THIS BLOCK SUPPLIES THE FAILURE DATA AGAINST WHICH THE MODEL     *
C*    IS TO BE TESTED.                                                 *
C*      CID(I)  = IDENTIFICATION FOR I-TH COMPONENT (CHARACTER*8)
C*      NF(I)   = NUMBER OF FAILURES FOR I-TH COMPONENT CLASS          *
C*      NT(I)   = TEST TIME FOR I-TH COMPONENT CLASS                   *
C*      N       = TOTAL NUMBER OF COMPONENTS                           *
C*      IPRT    = UNIT NUMBER FOR PRINTED OUTPUT (NOT USED)            *
C*                                                                     *
C* SUBROUTINES NEEDED:  DKSLVL, KSSUM, DXP                             *
C*                    DOUBLE FUNCTION H(FF,TT) MUST BE SUPPLIED        *
C*                                                                     *
C* WRITTEN BY J. K. SHULTIS, KANSAS STATE UNIVERSITY, 6/80             *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION G(150)
      CHARACTER*8 CID(500)
      COMMON /DTA/ CID,NF(500),NT(500),N,IPRT,NITER
C
      NSIZE = 150
      SIG1 = 0.0
      SIG2 = 1.0
      SUMOB = 0.0
      SUME = 0.0
      FACT = 1./N
      D = 0.0
C
C CALCULATE THE CUMULANT PROBABILITIES FOR THE OBSERVED AND EXPECTED FAI
C FAILURES
      IF (IOUT.GT.0) WRITE (IPRT,8)
      KK = 0
    1 KK = KK+1
      IF (KK.GT.NSIZE) GO TO 3
      K = KK-1
      FF = K
      SUMK = 0.0
      DO 2 I=1,N
C  CALCULATE CUMULANT OF OBSERVED
      IF (NF(I).EQ.K) SUMOB = SUMOB+1
C  CALCULATE CUMULANT OF EXPECTED
      TT = NT(I)
    2 SUMK = SUMK+H(FF,TT)
      SUME = SUME+SUMK
      G(KK) = SUME*FACT
      SKK = SUMOB*FACT
C  CALCULATE MAXIMUM DEVIATION
      DIFF = DABS(G(KK)-SKK)
      IF (D.LT.DIFF) D = DIFF
      IF (IOUT.GT.0) WRITE (IPRT,9) K,SKK,G(KK)
C  TEST IF G(K) IS SUFFICIENTLY CLOSE TO UNITY TO TRUNCATE SEQUENCE
      IF (((1.-G(KK)).LT.D).AND.((1.-SKK).LT.D)) GO TO 4
      GO TO 1
C  NOT ENOUGH STORAGE FOR G(K) CUMULANTS - VECTOR G(K) MUST BE BIGGER
    3 WRITE (IPRT,10) NSIZE,G(NSIZE)
      IF (ILVL.LT.0) WRITE (IPRT,11) D
      RETURN
C
C TEST IF SUFFICIENT TERMS HAVE BEEN CALCULATED
    4 D1 = 1.-D
      NEND = N*D1
      IF (NEND-N*D1) 6,5,5
    5 IF (G(KK).GE.(1.-FACT)) GO TO 7
      GO TO 1
    6 IF (G(KK).GE.(D+NEND*FACT)) GO TO 7
      GO TO 1
    7 IF (IOUT.GT.0) WRITE (IPRT,12) D
C
C CALCULATE SIGNIFICANCE LEVEL
      NTOT = N
      CALL DKSLVL (D,NTOT,G,NSIZE,IOUT,SIG1,SIG2,ILVL)
      IF (ILVL.EQ.0) WRITE (IPRT,13) D,SIG1,SIG2
      RETURN
    8 FORMAT (//9X,'K',8X,'CUMULANTS:  OBSERVED     EXPECTED')
    9 FORMAT (1X,I9,12X,G20.6,G13.6)
   10 FORMAT (/' IN K-S FIT TEST, VECTOR G(K) MUST BE INCREASED IN',
     >' SIZE: G(',I3,') =',G15.6)
   11 FORMAT (' DISCRETE KOLMOGOROV-SMIRNOV GOODNESS-OF-FIT',
     >' STATISTIC =',G12.5)
   12 FORMAT (/' KOLMOGOROV-SMIRNOV FIT STATISTIC D=',G15.6)
   13 FORMAT (//' DISCRETE KOLMOGOROV-SMIRNOV GOODNESS-OF-FIT',
     >' STATISTIC =',G12.5/
     >' PROBABILITY OF OBTAINING A VALUE AT LEAST AS LARGE',
     >' AS THIS STATISTIC, WHEN THE NULL HYPOTHESIS IS TRUE,',/
     >' IS BETWEEN',G11.3,' AND',G11.3)
      END
C+****************************** DKSLVL ********************************
C
      SUBROUTINE DKSLVL(D,N,G,MAXG,IOUT,PMIN,PMAX,ILVL)
C
C******************************* DKSLVL ********************************
C*                                                                     *
C* PURPOSE:                                                            *
C* THIS SUBROUTINE CALCULATES BOUNDS FOR THE SIGNIFICANCE LEVEL FOR    *
C* A SPECIFIED KOLMOGOROV-SMIRNOV GOODNESS-OF-FIT STATISTIC FOR A      *
C* DISCRETE PROBABILITY MODEL. UPPER AND LOWER BOUNDS ARE COMPUTED     *
C* ONLY IF THE NUMBER OF DATA IS LESS THAN 51.                         *
C*                                                                     *
C* SUBROUTINE VARIABLES:                                               *
C*    D     = SPECIFIED K-S DISCRETE GOODNESS-OF-FIT STATISTIC         *
C*    N     = TOTAL NUMBER OF COMPONENTS IN SAMPLE                     *
C*    G     = INPUT VECTOR GIVING EXPECTATIONS, I.E. G(K+1) EXPECT-    *
C*            ATION OF OBSERVING K OR FEWER FAILURES FOR SAMPLE        *
C*    MAXG  = SIZE OF G VECTOR                                         *
C*    IOUT  = PARAMETER CONTROLLING INTERMEDIATE OUTPUT: IF = 0 NO     *
C*            INTERMEDIATE OUTPUT PRODUCED; IF > 0 INTERMEDIATE OUT-   *
C*            PUT PRODUCED ON UNIT NUMBER 'IOUT'                       *
C*    PMIN  = LOWER BOUND ON THE SIGNIFICANCE LEVEL                    *
C*    PMAX  = UPPER BOUND ON THE SIGNIFICANCE LEVEL                    *
C*    ILVL  = ERROR RETURN CODE: =0 EXACT RESULT OBTAINED; =1 IF TOO   *
C*            MANY DATA ARE USED; <0 IF ERROR ENCOUNTERED              *
C*                                                                     *
C* SUBROUTINES NEEDED: KSSUM, DXP                                      *
C*                                                                     *
C* WRITTEN BY J. K. SHULTIS,  KANSAS STATE UNIVERSITY,  6/80           *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION G(MAXG),C(150),F(150),B(150)
      DOUBLE PRECISION BLN(150),E(150),ELN(150)
      EQUIVALENCE (C(1),F(1)), (B(1),E(1)), (BLN(1),ELN(1))
      CHARACTER*8 CID(500)
      COMMON /DTA/ CID,IDUMMY(1001),IPRT,NITER
C
      NASY = 51
      KMAX = 149
      IDEBUG = 0
C
C*** CALCULATE WHETHER SMALL SAMPLE LEVEL OR ASYMPTOTIC LEVEL REQUIRED
      ILVL = 0
      IF (N.GE.NASY) GO TO 27
      STOP = N*(1.-D)
      NSTOP = STOP+1
      DELN = 1./N
      IF (NSTOP.GT.KMAX) GO TO 25
C
C*** CALCULATE THE C(K) COEFFICIENTS:
      J = 0
      K = 0
    1 IF (G(J+1).GE.(D+K*DELN)) GO TO 2
      J = J+1
      IF (J.GT.MAXG) GO TO 24
      GO TO 1
    2 C(K+1) = 1.-G(J+1)
      K = K+1
      IF (K-STOP) 1,3,4
    3 C(K+1) = 0.0
    4 IF (IDEBUG.GE.1) WRITE (IPRT,28) (C(I),I=1,NSTOP)
C FIND THE LAST NON-ZERO C(K):
      DO 5 K=1,NSTOP
      KK = NSTOP-K+1
      IF (C(KK).GT.0.0) GO TO 6
    5 CONTINUE
    6 MAXC = KK-1
      IF (MAXC.GT.0) GO TO 7
      PMIN = 0.0
      GO TO 11
C CALCULATE THE LOGS OF C(K):
    7 DO 8 K=1,KK
    8 C(K) = DLG(C(K))
C
C CALCULATE THE B(K) COEFFICIENTS:
      BLN(1) = 0.0
      DO 9 KK=2,NSTOP
      K = KK-2
      KKK = KK-1
      JJ = MIN0(K,MAXC)
      CALL KSSUM (SUM,JJ,KKK,C,BLN,KMAX)
      COMSUM = 1.-SUM
      IF (COMSUM) 26,26,9
    9 BLN(KK) = DLG(COMSUM)
      B(1) = 1.0
      DO 10 K=2,NSTOP
   10 B(K) = DXP(BLN(K))
      IF (IDEBUG.GE.1) WRITE (IPRT,29) (B(I),I=1,NSTOP)
C
C CALCULATE PROBABILITY(DMINUS > D)
      JJ = NSTOP-1
      JJ = MIN0(JJ,MAXC)
      CALL KSSUM (PMIN,JJ,N,C,BLN,KMAX)
   11 IF (IOUT.GE.1) WRITE (IPRT,30) PMIN
C
C CALCULATE THE F(K) COEFFICIENTS:
      K = 0
      J = 0
   12 IF (G(J+1).GT.(1.00001D0-D-K*DELN)) GO TO 13
      J = J+1
      IF (J.GT.MAXG) GO TO 24
      GO TO 12
   13 IF (J) 14,14,15
   14 F(K+1) = 0.0
      K = K+1
      IF (K-STOP) 14,14,16
   15 F(K+1) = G(J)
      J = J-1
      K = K+1
      IF (K-STOP) 12,12,16
   16 IF (IDEBUG.GE.1) WRITE (IPRT,31) (F(I),I=1,NSTOP)
C FIND THE LAST NON-ZERO F(I)
      DO 17 K=1,NSTOP
      KK = NSTOP-K+1
      IF (F(KK).GT.0.0) GO TO 18
   17 CONTINUE
   18 MAXF = KK-1
      IF (MAXF.GT.0) GO TO 19
      PMAX = 0.0
      GO TO 23
C CALCULATION OF THE LOGS OF F(K):
   19 DO 20 K=1,KK
   20 F(K) = DLG(F(K))
C
C CALCULATE E(K) COEFFICIENTS:
      ELN(1) = 0.0
      DO 21 KK=2,NSTOP
      K = KK-2
      KKK = KK-1
      JJ = MIN0(K,MAXF)
      CALL KSSUM (SUM,JJ,KKK,F,ELN,KMAX)
      COMSUM = 1.-SUM
      IF (COMSUM) 26,26,21
   21 ELN(KK) = DLG(COMSUM)
      E(1) = 1.0
      DO 22 K=2,NSTOP
   22 E(K) = DXP(ELN(K))
      IF (IDEBUG.GE.1) WRITE (IPRT,32) (E(I),I=1,NSTOP)
C
C CALCULATE PROB(DPLUS > D)
      JJ = NSTOP-1
      JJ = MIN0(JJ,MAXF)
      CALL KSSUM (PMAX,JJ,N,F,ELN,KMAX)
   23 IF (IOUT.GE.1) WRITE (IPRT,33) PMAX
C
C CALCULATE BOUNDS ON THE SIGNIFICANCE LEVEL
      ARG = PMIN*PMAX
      PMAX = PMAX+PMIN
      PMIN = PMAX-ARG
      IF (PMAX.GT.1.0) PMAX = 1.0
      RETURN
C
C
   24 WRITE (IPRT,34)
      ILVL = -1
      RETURN
   25 WRITE (IPRT,35)
      ILVL = -2
   26 WRITE (IPRT,36)
      ILVL = -3
      RETURN
C
C*** CALCULATE SIGNIFICANCE LEVEL FROM ASYMPTOTIC SERIES
   27 ILVL = 1
      PMAX = 1.0
      PMIN = 1.0
      WRITE (IPRT,37)
      RETURN
   28 FORMAT (//' C(K) COEFFICIENTS:'/(1X ,8G15.7))
   29 FORMAT (/' B(I) COEFFICIENTS:'/(1X ,8G15.7))
   30 FORMAT (/' CALCULATED PROB(DMINUS > D) =',G15.7)
   31 FORMAT (/' F(I) COEFFICIENTS:'/(1X,8G15.7))
   32 FORMAT (/' E(I) COEFFICIENTS:'/(1X,8G15.7))
   33 FORMAT (/' CALCULATED PROB(DPLUS > D) =',G15.7)
   34 FORMAT (' TOO FEW G(K) SUPPLIED TO DKSLVL')
   35 FORMAT (' NEED BIGGER ARRAYS IN DKSLVL FOR COEFFICIENTS')
   36 FORMAT (' ROUND OFF ERROR PREVENTED ACCURATE CALCULATION OF',
     >' BOUNDS  FOR THE SIGNIFICANCE LEVEL')
   37 FORMAT (' TOO MANY DATA TO PERFORM KOLMOGOROV-SMIRNOV FIT TEST')
      END
C+**********************************************************************
C
      SUBROUTINE KSSUM(SUM,JJ,N,ALN,BLN,MAX)
C
C***********************************************************************
C* THIS SUBROUTINE PERFORMS SUMMATIONS NEEDED IN SUBROUTINE DKSLVL     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION ALN(MAX),BLN(MAX)
C
      PLN = 0.0
      SUM = DXP(N*ALN(1)+BLN(1))
      IF (JJ.EQ.0) RETURN
      JJJ = JJ+1
      DO 1 J=2,JJJ
      ARG = N-J+2
      PLN = PLN+DLG(ARG/(J-1.))
    1 SUM = SUM+DXP(PLN+(N-J+1.)*ALN(J)+BLN(J))
      RETURN
      END
C+******************************** CHISQ *******************************
C*                                                                     *
      SUBROUTINE CHISQ(H,EXPMIN,MAXGPS,LDF,INTER,EPS,CHI2,NDF,SIG,NZ)
C*                                                                     *
C********************************* CHISQ *******************************
C*                                                                     *
C* THIS PROGRAM PERFORMS A CHI-SQUARED GOODNESS OF FIT TEST ON GIVEN   *
C* FAILURE RATE ATTRIBUTE DATA WITH A COMPOUND POISSON-GAMMA FAILURE   *
C* RATE MODEL.  DIFFERENT TEST TIMES FOR THE COMPONENTS ARE ALLOWED.   *
C* FAILURE DATA ARE SUPPLIED TO THE SUBPROGRAM THROUGH THE COMMON      *
C* BLOCK 'DTA'.                                                        *
C*                                                                     *
C* INPUT PARAMETERS:                                                   *
C*    H     = EXTERNAL FUNCTION WHICH GIVES THE PROBABILITY OF         *
C*            OBTAINING FF FAILURES IN TEST TIME TT                    *
C*    EXPMIN= MINIMUM VALUE OF THE EXPECTED NUMBER OF FAILURES IN      *
C*            EACH CLASS                                               *
C*    MAXGPS= MAXIMUM NUMBER OF CLASSES DESIRED FOR THE CHI-SQUARED    *
C*            TEST                                                     *
C*    LDF   = NUMBER OF DEGREES OF FREEDOM TO BE SUBTRACTED FROM       *
C*            THE TOTAL NUMBER OF CLASSES TO GIVE THE NUMBER OF        *
C*            DEGREES OF FREEDOM FOR THE FIT TEST                      *
C*    INTER = 0 IF NO INTERMEDIATE OUTPUT IS DESIRED; =1 IF INTER-     *
C*            MEDIATE OUTPUT IS DESIRED                                *
C*    EPS   = RELATIVE ACCURACY OF THE SIGNIFICANCE LEVEL, I.E.        *
C*            1.D-S SPECIFIES S SIGNIFICANT FIGURES ARE DESIRED        *
C*                                                                     *
C* OUTPUT PARAMETERS:                                                  *
C*    CHI2  = VALUE OF THE CHI-SQUARED TEST STATISTIC                  *
C*    NDF   = NUMBER OF DEGREES OF FREEDOM USED IN THE TEST            *
C*    SIG   = SIGNIFICANCE LEVEL FOR THE CHI-SQUARED STATISTIC         *
C*    NZ    = ERROR RETURN CODE: =0 NORMAL RETURN, =-1 EXPECTATION     *
C*            VECTOR MUST BE MADE LARGER, >0 ERROR IN CALCULATION      *
C*            OF THE SIGNIFICANCE LEVEL (SEE SUBROUTINE 'GAMIC'        *
C*            ERROR RETURN CODES)                                      *
C*                                                                     *
C* COMMON BLOCK   /DTA/ CID,NF,NT,N,IPRT,NITER                         *
C*   CID (4000 BYTES), CONTAINS IDENTIFICATION LABELS FOR COMPONENTS   *
C*                     (NOT USED IN PROGRAM)                           *
C*   NF  (2000 BYTES), CONTAINS NUMBER OF FAILURES FOR THE COMPONENTS  *
C*   NT  (2000 BYTES), CONTAINS TEST TIME FOR COMPONENTS               *
C*   N      (4 BYTES), TOTAL NUMBER OF COMPONENTS                      *
C*   IPRT   (4 BYTES), UNIT NUMBER FOR INTERMEDIATE OUTPUT             *
C*   NITER  (4 BYTES), NOT USED IN SUBROUTINE                          *
C*                                                                     *
C* SUBROUTINES NEEDED:  H, GAMIC                                       *
C*                                                                     *
C* WRITTEN BY J. K. SHULTIS, KANSAS STATE UNIVERSITY,  6/80            *
C*                                                                     *
C***********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION E(100),EHAT(100),Y(2),CHI(50)
      DIMENSION NFHAT(100),NOB(100)
      CHARACTER*8 CID(500)
      COMMON /DTA/ CID,NF(500),NT(500),N,IPRT,NITER
C
C*** CALCULATE TOTAL NUMBER OF OBSERVED FAILURES
      KMAX=100
      DO 81 I=1,KMAX
   81 NOB(I)=0.0
      NFTOT=0
      DO 82 I=1,N
      NFF=NF(I) + 1
      NFTOT= NFTOT + 1
   82 NOB(NFF)=NOB(NFF) + 1
C
C*** CALCULATE THE EXPECTATIONS FOR EACH T(I)
      SUM=0.0
      IF(INTER.EQ.1) WRITE(IPRT,43)
   43 FORMAT(////' INTERMEDIATE OUTPUT FOR CHI-SQUARED TEST OF FIT'/
     >4X,'NO. FAIL.',4X,'OBSERVED',4X,'EXPECTED',4X,'CUMULATIVE EXPT.')
C
      DO 41 K=1,KMAX
      E(K)=0.0
      IF=K-1
      FF=IF
      DO 40 I=1,N
      TT=NT(I)
   40 E(K)=E(K) + H(FF,TT)
      SUM=SUM + E(K)
      NFTOT=NFTOT-NOB(K)
      IF(INTER.EQ.1) WRITE(IPRT,55) IF,NOB(K),E(K),SUM
   55 FORMAT(6X,I3,11X,I3,G17.5,G15.5)
      SUMM=N-SUM
      IF(SUMM.GT.EXPMIN) GO TO 41
      LAST=K
      KSTAR=K-2
      E(K)=N-(SUM-E(K))
      IF=IF+1
      IF(INTER.EQ.1) WRITE(IPRT,56) IF,NFTOT,SUMM,N
   56 FORMAT(4X,'>=',I3,11X,I3,G17.5,I8)
      GO TO 42
   41 CONTINUE
C  RETURN IF THE SIZE OF THE EXPECTATION VECTOR, E(K), > 100
      NZ=-1
      RETURN
C
C*** GROUP THE EXPECTATIONS SO THAT AT LEAST EXPMIN ARE IN EACH BIN
   42 KSTRT=1
      LAST=KSTAR+2
      KG=1
      NFHAT(1)=-1
   12 SUM=0.0
      DO 10 J=KSTRT,LAST
      SUM=SUM + E(J)
      IF (SUM.LT.EXPMIN) GO TO 10
      EHAT(KG)=SUM
      NFHAT(KG+1)=J-1
      GO TO 11
   10 CONTINUE
   11 KSTRT=KSTRT + (NFHAT(KG+1) - NFHAT(KG))
      IF(NFHAT(KG+1).GE.(KSTAR+1)) GO TO 15
      KG=KG+1
      GO TO 12
C PRINT OUT VALUES FOR EACH CLASS
   15 IF (KG.LE.2) GO TO 99
      IF (INTER.EQ.0) GO TO 59
      WRITE(IPRT,18) EXPMIN
   18 FORMAT(///' COLLAPSE CLASSES SO THAT EACH CLASS HAS AN EXPECTATION
     > GREATER THAN',G11.4/' CLASS    NO. FAILURES',7X,'EXPECTATION')
      KK=KG-1
      DO 16 I=1,KK
      II=NFHAT(I) + 1
   16 WRITE(IPRT,17) I,II,NFHAT(I+1),EHAT(I)
   17 FORMAT(' ',I4,6X,I3,' - ',I3,9X,G12.5)
      II=NFHAT(KG)+1
      WRITE(IPRT,19) I,II,EHAT(KG)
   19 FORMAT(' ',I4,9X,'>= ',I3,9X,G12.5)
C
C  COLLAPSE CLASSES FURTHER INTO MAXGPS BINS
   59 NEXTRA=KG-MAXGPS
      IF(NEXTRA) 97,97,20
   20 DO 29 I=1,NEXTRA
C FIND THE SMALLEST EXPECTATION AND BIN NUMBER
      EMIN=EHAT(1)
      KMIN=1
      DO 21 K=2,KG
      IF (EHAT(K).GT.EMIN) GO TO 21
      EMIN=EHAT(K)
      KMIN=K
   21 CONTINUE
C FIND THE SMALLEST NEIGHBOUR (I.E. NEIGHBOURING BIN WITH SMALLEST EXP.
      IF (KMIN.GT.1) GO TO 22
      KNEXT=2
      GO TO 28
   22 IF (KMIN.LT.KG) GO TO 23
      KNEXT=KG-1
      GO TO 28
   23 KNEXT=KMIN+1
      IF(EHAT(KMIN-1).LT.EHAT(KNEXT)) KNEXT=KMIN-1
C COMBINE THE SMALLEST CLASS WITH ITS SMALLEST NEIGHBOUR
   28 EHAT(KG+1)=0.0
      KSTRT=KMIN + (KNEXT-KMIN+1)/2
      EHAT(KSTRT-1)=EHAT(KMIN)+EHAT(KNEXT)
      DO 27 K=KSTRT,KG
      EHAT(K)=EHAT(K+1)
   27 NFHAT(K)=NFHAT(K+1)
   29 KG=KG-1
C
C*** CALCULATE THE OBSERVED NUMBER IN EACH BIN
C INITIALIZE THE OBSERVED VECTOR
   97 DO 30 I=1,LAST
   30 NOB(I)=0
      DO 31 I=1,N
      NFF=NF(I)
      KK=KG-1
      DO 32 K=1,KK
      IF(NFF.GT.NFHAT(K+1)) GO TO 32
      NOB(K)=NOB(K) + 1
      GO TO 31
   32 CONTINUE
      NOB(KG) = NOB(KG) + 1
   31 CONTINUE
C
C CALCULATE CHI-SQUARED CONTRIBUTION FOR EACH BIN
      CHI2=0.0
      DO 92 I=1,KG
      CHI(I)=(NOB(I)-EHAT(I))**2/EHAT(I)
   92 CHI2=CHI2 + CHI(I)
C PRINT OUT THE COLLAPSED GROUP RESULTS AND OBSERVED VALUES
      IF(INTER.EQ.0) GO TO 93
      KK=KG-1
      WRITE(IPRT,26) MAXGPS
   26 FORMAT(///' COLLAPSE THE NUMBER OF CLASSES SO TOTAL IS LESS THAN
     > OR EQUAL TO',I3/' GROUP   NUMBER OF FAILURES   EXPECTATIONS   OBS
     >ERVED   EXP-OBS   CHI-SQ CONTRIBUTION')
      DO 25 I=1,KK
      II=NFHAT(I)+1
      DIFF=EHAT(I)-NOB(I)
   25 WRITE(IPRT,35) I,II,NFHAT(I+1),EHAT(I),NOB(I),DIFF,CHI(I)
   35 FORMAT(I5,6X,I3,' - ',I3,10X,G12.5,I9,5X,G12.5,G15.5)
      II=NFHAT(KG) + 1
      DIFF=EHAT(KG)-NOB(KG)
      WRITE (IPRT,36) KG,II,EHAT(KG),NOB(KG),DIFF,CHI(KG)
   36 FORMAT(' ',I4,9X,'>= ',I3,10X,G12.5,I9,5X,G12.5,G15.5)
C
C*** CALCULATE THE SIGNIFICANCE LEVEL
   93 NDF=KG-LDF
      X=CHI2*0.5
      ALP=NDF*0.5
      CALL GAMIC(X,ALP,EPS,1,Y,NZ)
      SIG=1.0 - Y(1)
      WRITE(IPRT,46) CHI2
   46 FORMAT(//' CHI-SQUARED STATISTIC =',G15.6)
      WRITE(IPRT,45) NDF,SIG
   45 FORMAT(' PROBABILITY OF OBTAINING A VALUE AT LEAST AS LARGE AS THI
     >S STATISTIC WITH',I4,' DEGREES OF FREEDOM,'/' WHEN THE NULL HYPOTH
     >ESIS IS TRUE, IS EQUAL TO',G10.3)
      RETURN
C
   99 WRITE(IPRT,72)
   72 FORMAT(' TOO FEW CLASSES TO PEFORM CHI-SQUARED TEST OF FIT')
      NZ=2
      RETURN
      END
C+********************************* HD *********************************
C*                                                                     *
      DOUBLE PRECISION FUNCTION HD(FF,DD)
C*                                                                     *
C********************************** HD *********************************
C* THIS SUBROUTINE CALCULATES THE PROBABILITY OF OBTAINING FF FAILURES *
C* IN DD DEMANDS FROM A HOMOGENEOUS BINOMIAL MODEL (MPDEL=1) OR FROM   *
C* A COMPOUND BINOMIAL-PRIOR (MARGINAL) FAILURE MODEL (MODEL=2).       *
C*                                                                     *
C* INPUT VARIABLES:                                                    *
C*     FF = NUMBER OF FAILURES                                         *
C*     DD = NUMBER OF DEMANDS ON THE COMPONENT                         *
C*                                                                     *
C* COMMON BLOCK /MODL/ P, MODEL                                        *
C*     P     = FAILURE PROBABILTY FOR BINOMIAL (USED ONLY IF MODEL=1)  *
C*     MODEL = 1 FOR HOMOGENEOUS BINOMIAL MODEL; =2 FOR COMPOUND MODEL *
C*                                                                     *
C* SUBROUTINES NEEDED:  DXP, MARCLD                                    *
C*                                                                     *
C***********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /MODL/ P,MODEL
      COMMON /DATAD/ K,N
      COMMON /GENCOM/ IGOUT
      HD=0.0D0
      IF(DD.LT.FF) RETURN
      IF (MODEL.EQ.2) GO TO 2
C
C*** HOMOGENEOUS MODEL CALCULATION
    1 HD=DXP(DLGAMA(DD+1.D0)-DLGAMA(DD-FF+1.D0)-DLGAMA(FF+1.D0)
     > +FF*DLG(P) + (DD-FF)*DLG(1.D0-P))
      RETURN
C
C*** COMPOUND MODEL CALCULATION
    2 K=FF
      N=DD
      CALL MRGCLD(ANS,IERR)
      IF(IERR.NE.1) WRITE(IGOUT,100)
  100 FORMAT(' *** WARNING: POSSIBLE ERROR IN CALCULATION OF THE',
     >' MARGINAL DISTRIBUTION IN SUBROUTINE HD')
      HD=ANS
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE ABESTD(PMIN,PMAX,IPRIOR,A,B)
C
C***********************************************************************
C* COMPUTE RANGE PARAMETERS A AND B FROM LOWER AND UPPER LIMITS
C* PMIN AND PMAX OF P FOR FAILURE-ON-DEMAND CASE
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IP=-IPRIOR
      GO TO (100,200,300,400,500,600),IP
C* BETA DISTRIBUTION
100   A=PMIN
      B=PMAX
      RETURN
C* LOG-BETA DISTRIBUTION
200   IF(PMAX.LT.1.D0) GO TO 205
      A=0.D0
      GO TO 210
205   A=-DLG(PMAX)
210   B=-DLG(DMAX1(PMIN,1.0D-12))
      RETURN
C* LOG-NORMAL-1 DISTRIBUTION
300   IF(PMIN.GT.0.D0) GO TO 305
      A=1.D70
      GO TO 310
305   A=DLG(1.D0/PMIN-1.D0)
310   IF(PMAX.LT.1.D0) GO TO 315
      B=-1.D70
      RETURN
315   B=DLG(1.D0/PMAX-1.D0)
      RETURN
C* LOG-NORMAL-2 DISTRIBUTION
400   IF(PMAX.LT.1.D0) GO TO 401
      B=0.D0
      GO TO 403
401   B=-DLG(PMAX)
403   IF(PMIN.GT.0.D0) GO TO 405
      A=1.D70
      RETURN
405   A=-DLG(PMIN)
      RETURN
C* LOG-GAMMA-1 DISTRIBUTION
500   B=0.D0
      IF(PMAX.LT.1.D0) GO TO 505
      A=0.D0
      RETURN
505   A=-DLG(PMAX)
      RETURN
C* LOG-GAMMA-2 DISTRIBUTION
600   B=0.D0
      IF(PMIN.GT.0.D0) GO TO 605
      A=0.D0
      RETURN
605   A=-DLG(1.D0-PMIN)
      RETURN
      END
C+**********************************************************************
C*
      SUBROUTINE MMOM1D
C*
C*----------------------------------------------------------------------
C*
C*       THIS PROGRAM PACKAGE COMPUTES ESTIMATES OF THE PARAMETERS
C*     ALPHA AND BETA OF THE PRIOR AND MARGINAL DENSITIES USED IN
C*     'FAILURE ON DEMMAND' CASE, USING THE METHOD OF MOMENTS.
C*       THE MOMENT EQUATIONS ARE SOLVED BY USING THREE DIFFERENT
C*       TECHNIQUES, DEPENDING ON THE PRIOR DISTRIBUTION OF INTEREST.
C*
C*       THE PROGRAM PACKAGE CONSISTS OF THE FOLLOWING SUBROUTINES:-
C*
C*        1. MMOM1D      - INTEPHASE BETWEEN USER'S MAIN PROGRAM AND THE
C*                         PROGRAM PACKAGE.
C*        2. ESTMTD      - INITIATES ESTIMATION PROCEDURE
C*        3. MMTEQT      - YIELDS A VECTOR OF DIM=2 CONTAINING THE FUNCT
C*                         L VALUES OF(SAMPLE MT - POP MT)AT A GIVEN VAL
C*                         ALPHA & BETA.
C*        4. POPMMD      - COMPUTES THEORETICAL MOMENTS (ABOUT ZERO) OF
C*                         DISTRIBUTION FOR GIVEN VALUES OF ALPHA & BETA
C*        5. SMPLMM      - COMPUTES THE SAMPLE MOMENTS FROM DATA.
C*        6. PRIRFD      - RETURNS FUNCTIONAL VALUE OF P**IPOWER*F(P)
C*                         AT GIVEN P FOR A GIVEN INTEGER VALUE OF
C*                         IPOWER. F(P) IS THE VALUE OF THE DENSITY FUNC
C*                         EVALUATED AT P.
C*        7. PRIORD      - RETURNS THE VALUE F(P) AT A GIVEN P.
C*        8. DXP         - DOUBLE PRECISION EXPONENTIAL FUNCTION
C*        9. GAMIC       - COMPUTES N MEMBER SEQUENCE OF INCOMPLETE GAMM
C*                         FUNCTIONS NORMALIZED.
C*       10. MDBETA      - EVALUATE THE INCOMPLETE BETA DISTR. FUNCTION.
C*       11. DERFF       - BETTER VERSION OF IBM DERF FUNCTION.
C*       12. QNC7        - DOUBLE PRECISION NUMERICAL INTEGRATION OVER
C*                         FINITE INTERVALS.
C*       13. DRTMI       - ROUTINE THAT SOLVES SINGLE UNKOWN NON-LINEAR
C*       14. SIMIN        - FINDS THE SOLUTION VECTOR THAT MINIMIZES
C*                         THE SUM OF SUARES (F(ALPHA,BETA))**2
C*                        + (G(ALPHA,BETA))**2
C*                         WHERE F AND G ARE THE "MOMENT EQUATIONS".
C*                        (I.E. THE ELEMENTS OF THE VECTOR GIVEN BY
C*                        MMTEQ. NOTE THAT THIS SUM OF SQUARES IS
C*                        GIVEN TO US BY THE REAL FUNCTION SUMSQD.)
C*       15. DHEADD      - PRINTS DISTRIBUTION NAME AS TITLE
C*       16. OTHER IBM SUPPLIED FUNCTIONS.
C*
C*----------------------------------------------------------------------
C*
C*    SUBROUTINES NEEDED:- ESTMTD
C*
C*
C*    COMMON BLOCKS USED:-
C*
C*        1. COMMON /PRIR/PARM(4),RNG(2),IPRIR
C*
C*                       PARM(1) - INTIAL (STARTING) ESTIMATE OF ALPHA
C*                       PARM(2) - INTIAL (STARTING) ESTIMATE OF ABETA
C*                       PARM(3) & PARM(4) - END POINTS OF SUPPORT OF
C*                                           DISTRIBUTION
C*
C*                       RNG(1) & RNG(2)   -  GIVES END POINTS OF RAGE O
C*                                            P.
C*
C*                       IPRIR   - DISTRIBUTION INDEX
C*                               =  -1 FOR BETA DIST
C*                                  -2 FOR LOGBETA
C*                                  -3 FOR LOGNORMAL 1
C*                                  -4 FOR LOGNORMAL 2
C*                                  -5 FOR LOGGAMMA 1
C*                                  -6 FOR LOGGAMMA 2
C*
C*
C*        2. COMMON /SMPL/ PHAT,PHATSQ,SPSQ,RNSTAR,RN,N,IDID,IPRINT
C*
C*                       PHAT 4 SPSQ ARE THE SAMPLE MEAN 4 VARIANCE OF F
C*                       PHATSQ IS SQUARE OF PHAT
C*                       RNSTAR = N/SUM OF NI WHERE N=TOTAL # OBS
C*                                             AND NI=TOTAL 'TRIALS' IN
C*                                             OBS.
C*                       ISPRNT =0 IF SAMPLE MOMENTS ARE NOT TO BE
C*                                 PRINTED.
C*                              =1 IF THEY ARE TO BE PRINTED.
C*
C*        3. COMMON /POPL/ W(2),IPOWER,IDIST,IMPRNT
C*
C*                      W(1) = 1ST MOMENT OF DISTR.
C*                      W(2) = 2ND MOMENT OF DISTR. (ABOUT ZERO).
C*                      IPOWER = INDEX INDICATING WHETHER 1ST OR 2ND
C*                               MOMENT IS NEEDED (TELLS PRIRFD).
C*                      IDIST =1 IF MATCHING MOMENTS TO BE DONE FOR PRIO
C*                      IDIST =2 IF MATCHING MOMENTS TO BE DONE FOR MARG
C*                      IMPRNT =-1 IF NO MOMENTS OR ERROR MESSEGES
C*                                 ASSOCIATED WITH THEM ARE TO BE PRINTE
C*                             =0 IF THE 1ST & THE 2ND MOMENTS ARE NOT
C*                                 TO BE PRINTED, BUT ERROR MESSEGES
C*                                 ARE TO BE PRINTED.
C*                             =1 IF THE MOMENTS OF THE DISTRIBUTION
C*                                 EVALUATED AT THE FINAL ESTIMATE
C*                                 (SOLUTION) ARE TO BE PRINTED.
C*                                 ERROR MESSEGES WILL BE PRINTED AS WEL
C*                             =2 IF THE MOMENTS (EVALUATED AT EACH
C*                                 ITERATION STEP) ARE TO BE PRINTED
C*                                 AT EACH ITERATION INCLUDING THE
C*                                 FINAL STEP. ERROR MESSEGES WILL
C*                                 ALSO BE PRINTED.
C*
C*        4. COMMON /AQNC7/ ERR,IERR2
C*
C*
C*                      ERR - USER SUPPLIED ERROR LEVEL FOR QNC3
C*                      IERR2 - ERROR CODE PASSED BY QNC3
C*                            =1 IF OKAY
C*                            =2 IF ANSWER NOT ACCURATE
C*
C*
C*        5. COMMON /ASMIN/ FVECTT(2),EPS,ANS(2),S(2),Y(15),K,NEV,ICONT
C*                      FVECTT - TWO DIMENTIONAL ARRAY GIVING THE
C*                               VALUES OF THE MOMENT EQUATIONS AT EACH
C*                               ITERATION STEP.
C*                      FOR DESCRIPTION OF THE OTHER VARIABLES IN THIS
C*                      COMMON SEE DOCUMENTATION  GIVEN UNDER SECTION ON
C*                      INPUT PARAMETERS. ALSO SEE DOCUMENTATION ON
C*                      SIMIN. (NOTE:- DOCUMENTATION ON SIMIN SAYS THAT
C*                      S(1) SHOULD BE NEGATIVE. HOWEVER WE READ THIS
C*                      AS A POSITIVE NUMBER AND THE PROGRAM MMOM
C*                      MAKES IT NEGATIVE.)
C*
C*
C*        6. COMMON /ADRTMI/ X,F,XLI,XRI,EPSS,SRANG,IEND,IERR
C*                      FOR DESCRIPTION OF THESE VARIABLES
C+                      SEE DOCUMENTATION ON DRTMI.
C*                      ALSO SEE DOCUMENTATION ON INPUT PARAMTERS.
C*
C*
C*
C*        7. COMMON /FDDATA/ FI(500),NI(500)
C*
C*                      FI & NI CONTAINS DATA
C*
C*
C*        8. COMMON /IOMM/INPARM,INDATA,OUTPUT
C*
C*                      IPARM = UNIT FROM WHICH PARAMETERS ARE READ
C*                      INDATA =UNIT FROM WHICH DATA IS READ
C*                      OUTPUT = UNIT TO WHICH RESULTS TO BE OUTPUTTED
C*
C*
C*
C*        9. COMMON /MEST1/ IEPRNT,IIDIST
C*                      IEPRNT =-1 IF THE FINAL SOLUTION IS NOT
C*                                 TO BE PRINTED. ALSO ALL OTHER OUTPUT
C*                                 WILL BE SUPRESSED INCLUDING THE
C*                                 ERROR CODES. (NOTE:- IEPRNT WILL HAVE
C*                                 NO EFFECT ON THE PRINTING OF
C*                                 MOMENTS OR ERROR CODES ASSOCIATED
C*                                 THEM. IMPRNT SHOULD BE USED TO
C*                                 CONTROL THE PRINTING OF THESE.)
C*                             =0 IF ONLY THE FINAL SOLUTION TO THE
C*                                 MOMENTS EQUATIONS IS TO BE
C*                                 PRINTED. THE VALUES OF THE MOMENTS
C*                                 EQUATIONS AT THE SOLUTION WILL ALSO
C*                                 BE PRINTED.
C*                             =1  IF VALUES OF THE CURRENT ESTIMATES
C*                                 ARE TO BE PRINTED AT EACH ITERATION.
C*                                 THE VALUES OF THE MOMENTS EQUATIONS
C*                                 AT THE CURRENT ESTIMATE WILL
C*                                 ALSO BE PRINTED.
C*                      IIDIST = 1 IF ESTIMATION IS TO BE DONE FOR
C*                                 PRIOR DISTRIBUTION ONLY.
C*                             = 2 IF ESTIMATION IS TO BE DONE FOR
C*                                 MARGINAL DISTRIBUTION ONLY
C*                             = 3 IF ESTIMATION IS TO BE DONE FOR BOTH
C*                                 MARGINAL AND PRIOR DISTRIBUTIONS.
C*
C*       10. COMMON /PBNDS/  SX(2),SF(2),IFIRST,WHICHX
C*                        SX    = CONTAINS THE VALUE OF THE VECTOR
C*                                X AT THE PREVIOUS ITERATION.
C*                        FX    = CONTAINS THE VALUE OF THE VECTOR
C*                                FVECT AT THE PREVIOUS ITERATION.
C*                        IFIRST = 1 IF THE PREVIOUS VALUE OF THE
C*                                    X VECTOR IS WITHIN BOUNDS
C*                               = 0 OTHERWISE.
C*                        WHICHX = 1 IF X(1) IS OUTSIDE BOUNDS
C*                                   AND X(2) IS NOT.
C*                               = 2 IF X(2) IS OUTSIDE BOUNDS
C*                                   THESE ABOVE VALUES ARE PASSED TO
C*                                    THE SUBROUTINE OUTBND BY
C*                                    SUBROUTINE MMTEQD IF THE X
C*                                    ARE STEPPING OUTSIFE THEIR BOUNDS
C*
C*
C*     PARAMETERS NEEDED:-
C*     INPUT PARAMETERS NEEDED:-
C*
C*        1. ACCURACY & SEARCH PARAMETRS
C*                       ERR - MAXIMUM ERROR ALLOWED IN QNC7-USER SUPPLI
C*                             DEFAULT IS 10**-10
C*
C*
C*                       EPS - THE CONVERGENCE CRITERION.
C*                             THE MINIMIZATION PROCESS FIRST CALL
C*                             3 POINTS IN (ALPHA,BETA) SPACE AND
C*                             ITERATES BY REPLACING THE POINT WITH THE
C*                             LARGEST FUNCTION VALUE BY A NEW POINT
C*                             WITH A SMALLER FUNCTION VALUE.
C*                             NOW, IF YAVG IS THE AVERAGE VALUE OF
C*                             THE FUNCTION AT THE 3 POINTS AND R IS
C*                             THEIR STANDARD ERROR (I.E. THE
C*                             ROOT-MEAN-SQUARE OF THE SUM OF
C*                             (Y(I) - YAVG), I=1,2,3), THEN,
C*                             IF EPS .GT. 0  CONVERGENCE IS OBTAINED
C*                             IF R .LE. EPS.
C*                             IF EPS .LT. 0, CONVERGENCE IS OBTAINED
C*                             IF R .LE. ABS(EPS*YAVG).
C*                             IF EPS = 0, THE PROCESS WILL STOP
C*                             AFTER NEV ITERATIONS.
C*                             DEFAULT IS 1.0E-7
C*
C*                     S    - LENGTH OF EACH SIDE OF INITAIL SIMPLEX
C*                            (I.E. INITAIL STEP SIZE IN EACH DIMENSION)
C*                            DEFAULT IS INITIAL GUESS*0.05
C*
C*                     NEV -  LIMIT ON THE NUMBER OF ITERATIONS.
C*                            DEFAULT IS 1000.
C*
C*
C*                     EPSS- ACURACY REQUIRED IN THE SOLUTION
C*                           OF THE NON-LINEAR EQUATION WITH
C*                           ONE UNKNOWN (BETA). THIS IS USED
C*                           FOR WEIBULL DISTRIBUTION.
C*
C*
C*        2. DISTRIBUTION PARAMETERS
C*
C*                       PARM(1),....PARM(4) -SEE DOCUM ON COMMON /IPRIR
C*                       IPRIR - DISTR. INDEX.SEE DOCUM ON COMMON /IPRIR
C*                       N     - # OF OBSERVATIONS FOR THIS DISTR.
C*                       IIDIST - =1 IF ESTIMATES TO BE FOUND FOR THE
C*                               PRIOR DISTRIBUTION ONLY.
C*                               =2 IF ESTIMATES TO BE FOUND FOR THE
C*                               MARGINAL DISTRIBUTION ONLY.
C*                               =3 IF ESTIMATES ARE TO BE FOUND FOR
C*                               BOTH PRIOR AND MARGINAL.
C*                               STEP, INCLUDING THE FINAL STEP.
C*        3. OUTPUT CONTROL PARAMETERS
C*
C*                       IEPRNT  - SEE DESCRIPTION ON MEST1 COMMON
C*                       IMPRNT  - SEE DESCRIPTION ON POPL COMMON
C*                       ISPRNT  - SEE DESCRIPTION ON SMPL COMMON
C*
C*
C*----------------------------------------------------------------------
C*
C*        WRITTEN BY:- V.A. SAMARANAYAKE
C*                     KANSAS STATE UNIVERSITY
C*                     MANHATTAN, KANSAS
C*
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION SSTOR(2)
      INTEGER OUTPUT,WHICHX
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /SMPL/ PHAT,PHATSQ,SPSQ,RNSTAR,RN,N,IDID,ISPRNT
      COMMON /POPL/ W(4),IPOWER,IDIST,IMPRNT
      COMMON /AQNC7/ ERR,IERR2
      COMMON /ASMIN/ FVECTT(2),EPS,ANS(2),S(2),Y(15),K,NEV,ICONT
      COMMON /ADRTMI/ X,F,XLI,XRI,EPSS,SRANG,IEND,IERR
      COMMON /FDDATA/ FI(500),XNI(500)
      COMMON /IOMM/ INPARM,INDATA,OUTPUT
      COMMON /MEST1/ IEPRNT,IIDIST
      COMMON /PBNDS/ SX(2),SF(2),IFIRST,WHICHX
C
C     NOTE THE EQUIVALENCE STATEMENT
C
      EQUIVALENCE (PARM(1),ALPHA),(PARM(2),BETA),(PARM(3),A,THETA),
     > (PARM(4),B)
C
C     INITIALIZE
C
      K = 2
      SX(1) = 0.D0
      SX(2) = 0.D0
      SF(1) = 9.D17
      SF(2) = 9.D17
      IFIRST = 1
C
C
C     IF VALUES NOT GIVEN ASSIGN DEFAULT VALUES
C
      IF (ERR .LT. 1.0D-40) ERR = 1.0D-7
      AEPS = DABS(EPS)
      IF (AEPS .LT. 1.0D-20 ) EPS  = 1.0D-15
      IF (S(1) .LT. 1.0D-20 ) S(1) = PARM(1)*5.0D-2
      IF (S(2) .LT. 1.0D-20 ) S(2) = PARM(2)*5.0D-2
      IF (NEV .EQ. 0) NEV = 1000
      IF (EPSS .LT. 1.0D-20) EPSS = 1.0D-10
      IF(IEND .EQ. 0) IEND = 1000
      ISNEV = NEV
      S(1) = -S(1)
      SSTOR(1) = S(1)
      SSTOR(2) = S(2)
C
C     WRITE USER SUPPLIED BOUNDS AND ACCURACY PARAMETERS FOR
C     QNC3 & LMGENV
C
C     IF (IEPRNT .LT. 0) GO TO 10
C
C
C  10 CONTINUE
C
C     STORE INITIAL EST OF ALPHA AND BETA.
C
      PARM1 = PARM(1)
      PARM2 = PARM(2)
      ANS(1) = PARM1
      ANS(2) = PARM2
C
      RN = DBLE(N)
C
C
C
C     IF PROCESSING IS TO BE DONE FOR MARGINAL DISTRIBUTION ONLY,
C     THEN GO TO SECTION OF SUBROUTINE THAT DOES PROCESSING FOR
C     MARGINALS
C
      IF (IIDIST .EQ. 2) GO TO 31
C
C
C     CALL SUBROUTINE THAT WRITES THE NAME OF DISTRIBUTION CURRENTLY BEI
C     PROCESSED
C
      IF (IEPRNT .LT. 0) GO TO 15
C
C     WRITE STARTING VALUES FOR ALPHA & BETA.
C     ALSO CHECK TO SEE IF STARTING VALUES ARE WITHIN THE RESPECTIVE
C     RANGES FEASIBLE FOR ALPHA AND BETA
C
C
C
      IF (BETA .GT. 0.D0) GO TO 2
    1 CONTINUE
      WRITE (OUTPUT,211)
  211 FORMAT ('0 ***********   ERROR   **********  STARTING BETA IS NEGA
     >TIVE. GIVE POSITIVE VALUE.')
      RETURN
C
    2 CONTINUE
      IF (ALPHA .GT. 0.D0 .OR. IPRIR .EQ. -3 .OR. IPRIR .EQ. -4)GOTO 4
    3 CONTINUE
      WRITE (OUTPUT,212)
  212 FORMAT ('0 **********    ERROR   ********** SATRTING ALPHA IS NEGA
     >TIVE . GIVE POSITIVE VALUE')
      RETURN
C
C
C     INITIATE ESTIMATION PROCEDURE FOR PRIOR DISTR.
C
C
    4 CONTINUE
C
   15 CONTINUE
C
      IDIST = 1
      CALL ESTMTD
C
C     IF NO ERROR THEN GO TO SECTION OF CODE THAT PROCESS MARGINAL
C     DISTRIBUTION, OR RETURN.
C
      IF (NEV .GT. ISNEV) GO TO 22
      IF (IIDIST .EQ. 1) RETURN
      GO TO 30
C
C
C     AN ERROR HAS OCCURED DURING THE SOLUTION OF MOMENT EQUATIONS.
C
   22 CONTINUE
      IF (IEPRNT .GE. 0) WRITE (OUTPUT,202)
  202 FORMAT ('0',10X,'****** ITERATION LIMIT EXCEEDED. SEARCH DIDNOT CO
     >NVERGE TO A MINIMUM')
C
C     CHECK IF ONLY PRIOR DISTRIBUTION ESTIMATES ARE REQUESTED
C
      IF (IIDIST .EQ. 1) RETURN
C
C     RELOAD INITIAL ESTIMATES
C
      PARM(1) = PARM1
      PARM(2) = PARM2
      ANS(1) = PARM1
      ANS(2) = PARM2
C
C     INITIATE ESTIMATION PROCEDURE FOR MARGINAL DISTR.
C
   30 CONTINUE
      NEV = ISNEV
      S(1) = SSTOR(1)
      S(2) = SSTOR(2)
      IFIRST = 1
      SF(1) = 9.D17
      SF(2) = 9.D17
      SX(1) = 0.D0
      SX(2) = 0.D0
   31 CONTINUE
C
C
      IF (BETA .LE. 0.D0) GO TO 1
      IF (ALPHA .LE. 0.D0 .AND. (IPRIR .NE. -3 .OR. IPRIR .NE. -4))GOTO
     > 3
      IDIST=2
      CALL ESTMTD
      IF (IEPRNT .LT. 0) GO TO 999
  999 CONTINUE
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE ESTMTD
C
C***********************************************************************
C*
C*        THIS SUBROUTINE INITIATES THE ESTIMATION PROCEDURE
C*
C*        SUBROUTINES NEEDED:-  1. SMPLMM
C*                              2. LMGENV
C*
C*----------------------------------------------------------------------
C*
C*    WRITTEN BY:- V.A.SAMARANAYAKE
C*                 KANSAS STATE UNIVERSITY,
C*                 MANHATTAN, KANSAS 66506.
C*
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER OUTPUT
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /SMPL/ PHAT,PHATSQ,SPSQ,RNSTAR,RN,N,IDID,ISPRNT
      COMMON /POPL/ W(4),IPOWER,IDIST,IMPRNT
      COMMON /AQNC7/ ERR,IERR2
      COMMON /ASMIN/ FVECTT(2),EPS,ANS(2),S(2),Y(15),K,NEV,ICONT
      COMMON /ADRTMI/ X,F,XLI,XRI,EPSS,SRANG,IEND,IERR
      COMMON /IOMM/ INPARM,INDATA,OUTPUT
C
C
      EQUIVALENCE (PARM(1),ALPHA),(PARM(2),BETA),(PARM(3),A,THETA),
     > (PARM(4),B)
      EXTERNAL SUMSQD
      EXTERNAL MFUNCD
C     IF SAMPLE MOMENTS ALREADY ESTIMATED SKIP CALLING SMPLMM
C
      IF (IDID .EQ. 1) GO TO 5
      CALL SMPLMM
      IDID = 1
    5 CONTINUE
C
C
C     DIFFERENT DISTRIBUTIONS HAVE DIFFERENT SOLVING TECHNIQUES
C     (CLOSED FORM SOLUTION, SOLVING NON-LINEAR SINGLE UNKNOWN
C     SINGLE EQUATION & SOLVING TWO UNKNOWN SIMULTANEOUS EQUATIONS)
C     GO TO APROPRIATE TECHNIQUE
C
C
      II = -IPRIR
      GO TO (10,20,20,20,50,50),II
C
C     IF BETA DISTRIBUTION ESTIMATE IS GIVEN IN CLOSED FORM
C     HENCE COMPUTE IT DIRECTLY
C
   10 CONTINUE
      PMA = PHAT-A
      BMP = B-PHAT
      BMA = B-A
      IF (IDIST .EQ. 2) GO TO 15
      XX = (PMA*BMP/SPSQ) - 1.D0
      GO TO 16
   15 CONTINUE
      XX = (PMA*BMP - SPSQ)/(SPSQ-(PHAT*(1.D0 - PHAT)/RNSTAR))
   16 CONTINUE
      ANS(1) = (PMA/BMA)*XX
      ANS(2) = (BMP/BMA)*XX
      RETURN
C
C     FIND SOLUTION FOR LOGBETA,LOGNORMAL1 AND LOGNORMAL2 DISTR.
C
   20 CONTINUE
      ICONT = 0
      CALL SIMIN (SUMSQD,K,EPS,ANS,S,NEV,ICONT,Y)
      RETURN
C
C      FIND SOLUTION FOR LOGGAMMA 1 DISTRIBUTION.
C      DRTMI IS USED TO FIND THE SOLUTION
   50 CONTINUE
      XLI = BETA/SRANG
      XRI = BETA/SRANG
      CALL DRTMI (X,F,MFUNCD,XLI,XRI,EPSS,IEND,IERR)
      IF (IERR .NE. 0) WRITE(OUTPUT,200) IERR
  200 FORMAT ('0 ******   ERROR HAS OCCURED WHILE EXECUTING THE SOLUTION
     > FINDING ROUTINE DRTMI. IERR = ',I2)
      IF (II .EQ. 6) GO TO 60
      ANS(1) = -(THETA + DLG(PHAT))/DLG(X + 1.D0)
      ANS(2) = X
      RETURN
C
C      FIND SOLUTION FOR LOGGAMMA 2 DISTRIBUTION
C
   60 CONTINUE
      ANS(1) = -(THETA +DLG(1.D0-PHAT))/DLG(1.D0+X)
      ANS(2) = X
      RETURN
      END
C+**********************************************************************
C*
      DOUBLE PRECISION FUNCTION PRIRFD(P)
C*
C***********************************************************************
C*
C*     THIS FUNCTION IS USED BY QNC3 TO EVALUATE THE VALUE OF
C*     FUNCTION IT IS IN THE PROCESS OF INTEGRATING, AT A GIVEN VALUE P
C*
C*     SUBROUTINES NEEDED:-     1. PRIORD
C*
C*----------------------------------------------------------------------
C*
C*    WRITTEN BY:- V.A.SAMARANAYAKE
C*                 KANSAS STATE UNIVERSITY,
C*                 MANHATTAN, KANSAS 66506.
C*
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /POPL/ W(4),IPOWER,IDIST,IMPRNT
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      PRIRFD = 0.0D0
      IF (P .LT. 1.D0-70) GO TO 10
      PRIORF = PRIORD(P)
      IF ( PRIORF .LT. 1.0D-70) GO TO 10
      PRIRFD = DXP(IPOWER*DLG(P) + DLG(PRIORF))
   10 CONTINUE
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE MMTEQD (NNVAR,NNFUNS,X,FVECTT,IERRR)
C
C***********************************************************************
C*
C*        THIS SUBROUTINE IS USED BY SUMSQD TO DETERMINE THE
C*     VALUES OF THE MATCHING MOMENT EQUATIONS FOR A GIVEN VALUE OF
C*     THE PARAMETERS TO BE ESTIMATED
C*
C*     SUBROUTINES NEEDED:-     1. POPMMD
C*                              2. DXP
C*                              3. DERFF
C*
C*----------------------------------------------------------------------
C*
C*    WRITTEN BY:- V.A.SAMARANAYAKE
C*                 KANSAS STATE UNIVERSITY,
C*                 MANHATTAN, KANSAS 66506.
C*
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER OUTPUT,WHICHX
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /SMPL/ PHAT,PHATSQ,SPSQ,RNSTAR,RN,N,IDID,ISPRNT
      COMMON /POPL/ W(4),IPOWER,IDIST,IMPRNT
      COMMON /AQNC7/ ERR,IERR2
      COMMON /MEST1/ IEPRNT,IIDIST
      COMMON /IOMM/ INPARM,INDATA,OUTPUT
      COMMON /PBNDS/ SX(2),SF(2),IFIRST,WHICHX
      DIMENSION X(2),FVECTT(2),WW(2)
      EQUIVALENCE (W(1),WW(1),W1),(W(2),W2),(PARM(3),AA),(PARM(4),BB)
C
C
C     STORE ORIGINAL VALUES OF PARM(1), PARM(2) AND LOAD VALUES IN
C     X(1) AND X(2) INTO THEM. NOTE PRIRFD USES THE 4 VALUES IN ARRAY
C     PARM AS THE PARAMETERS OF THE DISTRIBUTIONAND THAT X HAS THE
C     LATEST VALUES FOR ALPHA AND BETA.
C
      PARM1 = PARM(1)
      PARM2 = PARM(2)
      PARM(1) = X(1)
      PARM(2) = X(2)
C
      X1 = X(1)
      X2 = X(2)
      IF ( X2 .LE. 1.0D-73) GO TO 91
C
C
      II=-IPRIR
      GO TO (10,20,21,21,20,20),II
   10 CONTINUE
      RETURN
C
   20 CONTINUE
      IF ( X1 .LE. 1.0D-73) GO TO 90
   21 CONTINUE
      IFIRST = 1
      IPOWER = 2
      CALL POPMMD
      FVECTT(1) = PHAT - W1
      FVECTT(2) = PHATSQ + SPSQ -W2
      IF (IDIST .EQ. 2) FVECTT(2) = FVECTT(2) -(W1-W2)/RNSTAR
C
C
  999 CONTINUE
      IF (IEPRNT .EQ. 1) WRITE (OUTPUT,200) X,FVECTT
  200 FORMAT ('0',6X,'ALPHA = ',F20.12,4X,' BETA = ',F20.12,
     >6X,' F1 = ',D20.14,4X,' F2 = ',D20.14)
      IF (IMPRNT .EQ. 2) WRITE(OUTPUT,201) WW
  201 FORMAT (' ',10X,'1ST MOMENT= ',D20.14,6X,' 2ND MOMENT= ',D20.14)
C      RELOAD THE PARM VALUES THAT WERE SAVED.
C
      PARM(1) = PARM1
      PARM(2) = PARM2
      SX(1) = X1
      SX(2) = X2
      SF(1) = FVECTT(1)
      SF(2) = FVECTT(2)
      RETURN
C
C     ESTIMATES HAVE STEPPED OUTSIDE THEIR PERMISSABLE RANGE.
C     CALL SUBROUTINE OUTBND WHICH ASSIGNS APPROPRIATE VALUES
C     TO THE ARRAY FVECTT SUCH THAT THE DIRECTION OF SEARCH
C     WILL BE CHANGED AND THE ESTIMATES WILL REENTER THEIR
C     PERMISIBLE RANGE.
C
C     THIS COPRRECTION MAY BE ALL YOU NEED TO PREVENT GETTING
C     UNFEASIBLE SOLUTIONS FOR THE MOMENT EQUATIONS.
C     HOWEVER IN SOME CASES YOU MAY HAVE AN ABEND.
C     IN SUCH CASES, EITHER GIVE A SMALLER VALUE FOR
C     DIST PARAMETER(SAY 1/10 OF PRESENT VALUE) OR
C     START AT A DIFFERENT INITIAL POINT.
C
C
   90 CONTINUE
      WHICHX = 1
      CALL OUTBND (FVECTT,X)
      W1 = 0.D0
      W2 = 0.D0
      GO TO 999
   91 CONTINUE
      WHICHX = 2
      CALL OUTBND (FVECTT,X)
      W1 = 0.D0
      W2 = 0.D0
      GO TO 999
      END
C+*********************************************************************
C
      SUBROUTINE OUTBND (FVECTT,X)
C
C**********************************************************************
C*
C*     THIS SUBROUTINE IS USED BY THE SUBROUTINE MMTEQD/MMTEQ
C*     WHEN THE THE PARAMETER VALUES (ALPHA & BETA) PASSED BY
C*     THE MINIMIZATION ROUTINE (LMGENV OR SIMIN) ARE OUTSIDE
C*     THEIR RESPECTIVE RANGES.
C*
C*     THIS SUBROUTINE ASSIGNS APPROPRIATE VALUES TO THE
C*     ARRAY FVECTT SUCH THAT THE MINIMIZATION ROUTINE
C*     WILL CHANGE ITS DIRECTION OF SEARCH AND
C*     RE-ENTER THE REGION OF PERMISIBLE VALUES.
C*
C*----------------------------------------------------------------------
C*
C*     WRITTEN BY :-
C*              V.A.SAMARANAYAKE
C*              KANSAS STATE UNIVERSITY,
C*              MANHATTAN, KANSAS 66502.
C*
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER WHICHX
      COMMON /PBNDS/ SX(2),SF(2),IFIRST,WHICHX
      DIMENSION X(2),FVECTT(2)
C
C     IF PREVIOUS X VALUES WERE POSITIVE GO TO 200
C
      IF (IFIRST .EQ. 1) GO TO 200
C
C     CHECK WHICH X IS NEGATIVE
C
      IF (WHICHX .EQ. 2) GO TO 20
C
C      X(1) IS NEGATIVE
C
C     NOW, IF PRESENT X(1) VALUE IS > THAN LAST X(1)
C     VALUE LET SEARCH PROCEED IN PRESENT DIRECTION
C     THIS IS DONE BY LETTING FVECTT VALUES TO BE
C     SMALLER THAN THE PREVIOUS VALUES.
C
C     IF PRESENT X(1) VALUE IS < THAN LAST X(1)
C     VALUE, THEN REVERSE SEARCH DIRECTION BY LETTING
C     THE FVECT VALUES TO BE LARGER THAN PREVIOUS VALUES.
C
      IF (X(1) .LT. SX(1)) GO TO 100
      FVECTT(1) = SF(1)/2.D0
      FVECTT(2) = SF(2)/2.D0
      GO TO 999
   20 CONTINUE
C
C     PROCESSING IS THE SAME AS FOR X(1)
C
      IF (X(2) .LT. SX(2)) GO TO 100
      FVECTT(1) = SF(1)/2.D0
      FVECTT(2) = SF(2)/2.D0
      GO TO 999
C
C     X VALUES SMALLER (I.E. WORSE) THAN PREVIOUS VALUES
C
  100 CONTINUE
      FVECTT(1) = SF(1)*2.D0
      FVECTT(2) = SF(2)*2.D0
      GO TO 999
C
C    LAST X VALUES POSITIVE. NEG VALUES ENTERED IN THIS STEP
C
  200 CONTINUE
      FVECTT(1) = SF(1)*4.D0
      FVECTT(2) = SF(2)*4.D0
      IFIRST = 0
C
C
  999 CONTINUE
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE POPMMD
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER OUTPUT
      COMMON  /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /POPL/ W(4),IPOWER,IDIST,IMPRNT
      COMMON /AQNC7/ ERR,IERR2
      COMMON /IOMM/ INPARM,INDATA,OUTPUT
C
      EQUIVALENCE (W(1),W1),(W(2),W2),(W(3),W3),(W(4),W4),
     >(PARM(1),ALPHA),(PARM(2),BETA),(PARM(3),THETA,A),(PARM(4),B)
      EXTERNAL PRIRFD
C
C
      II = -IPRIR
      GO TO (10,20,20,40,50,60),II
C
C
C     BETA DISTRIBUTION
   10 CONTINUE
      ALBT = ALPHA+BETA
      BMA = B-A
      W1 = BMA * ALPHA/ALBT + A
      IF (IPOWER .EQ. 1) RETURN
      BMAS = BMA*BMA
      AL1 = ALPHA + 1.D0
      ALBT1 = ALBT + 1.D0
      AS = A*A
      W2 = BMAS*ALPHA*AL1/(ALBT*ALBT1) + 2.D0*A*W1 -AS
      IF (IPOWER .EQ. 2) RETURN
      AL2 = AL1 + 1.D0
      ALBT2 = ALBT1 + 1.D0
      W3 = BMAS*BMA*ALPHA*AL1*AL2/(ALBT*ALBT1*ALBT2)
     > + 3.D0*A*W2 -3.D0*AS*W1 + AS*A
      IF (IPOWER .EQ. 3) RETURN
      W4 = ALPHA*AL1*AL2*(AL2+1.D0)*BMAS*BMAS/
     > (ALBT*ALBT1*ALBT2*(ALBT2+1.D0))
     > + 4.D0*A*W3 -6.D0*AS*W2 +4.D0*AS*A*W1 -AS*AS
      RETURN
C
C     LOGBETA AND LOGNORMAL 1 DISTRIBUTIONS
C     MOMENTS FOUND BY NUMERICAL INTEGRATION
C
   20 CONTINUE
      CALL RANGED
      IPOWR = IPOWER
      XLI = RNG(1)
      XRI = RNG(2)
      DO 21 IPOWER=1,IPOWR
      CALL QNC7 (PRIRFD,XLI,XRI,ERR,W(IPOWER),IERR2)
      IF (IMPRNT  .GE. 0 .AND. IERR2 .EQ. 2) WRITE (OUTPUT,101)
  101 FORMAT ('0 ********   ERROR   ******** NUMERICAL INTEGRATION',
     >' ROUTINE FAILED TO GIVE REQUIRED ACCURACY, IN COMPUTING MOMENTS')
   21 CONTINUE
      RETURN
C
C     LOGNORMAL 2  DISTRIBUTION
C
C
   40 CONTINUE
      SQRT2 = 1.D0/DSQRT(2.D0)
      AMA = (A-ALPHA)/BETA * SQRT2
      BMA = (B-ALPHA)/BETA * SQRT2
      ISA = SGN(AMA)
      ISB = SGN(BMA)
      ABSA = DABS(AMA)
      ABSB = DABS(BMA)
      DRFA = 0.5D0*ISA*DERFF(ABSA)
      DRFB = 0.5D0*ISB*DERFF(ABSB)
C FOLLOWING 3 STATEMENTS DEACTIVATED. 24.MAR 86 HENNINGS
C     WRITE(6,777) A,B,AMA,AMB,ISA,ISB,ABSA,ABSB,DRFA,DRFB
C 777 FORMAT(' INSIDE ',4(D20.14,1X)/' SGN ',2(I3,1X)/
C    1 ' ABS ',4(D20.14,1X))
      G = DRFA-DRFB
      BETAS = BETA*BETA
      AMA = AMA + BETA*SQRT2
      BMA = BMA + BETA*SQRT2
      ISA = SGN(AMA)
      ISB = SGN (BMA)
      ABSA = DABS(AMA)
      ABSB = DABS(BMA)
      DRFA = 0.5D0*ISA*DERFF(ABSA)
      DRFB = 0.5D0*ISB*DERFF(ABSB)
      W1 = DXP(BETAS*0.5D0 - ALPHA)*(DRFA-DRFB)/G
      IF (IPOWER .EQ. 1) RETURN
C
      AMA = AMA + BETA*SQRT2
      BMA = BMA + BETA*SQRT2
      ISA = SGN(AMA)
      ISB = SGN (BMA)
      ABSA = DABS(AMA)
      ABSB = DABS(BMA)
      DRFA = 0.5D0*ISA*DERFF(ABSA)
      DRFB = 0.5D0*ISB*DERFF(ABSB)
      W2 = DXP(2.D0*BETAS -2.D0*ALPHA)*(DRFA-DRFB)/G
      IF (IPOWER .EQ. 2) RETURN
      AMA = AMA + BETA*SQRT2
      BMA = BMA + BETA*SQRT2
      ISA = SGN(AMA)
      ISB = SGN (BMA)
      ABSA = DABS(AMA)
      ABSB = DABS(BMA)
      DRFA = 0.5D0*ISA*DERFF(ABSA)
      DRFB = 0.5D0*ISB*DERFF(ABSB)
      W3 = DXP(4.5D0*BETAS - 3.D0*ALPHA)*(DRFA-DRFB)/G
      IF (IPOWER .EQ. 3) RETURN
      AMA = AMA + BETA*SQRT2
      BMA = BMA + BETA*SQRT2
      ISA = SGN(AMA)
      ISB = SGN (BMA)
      ABSA = DABS(AMA)
      ABSB = DABS(BMA)
      DRFA = 0.5D0*ISA*DERFF(ABSA)
      DRFB = 0.5D0*ISB*DERFF(ABSB)
      W4 = DXP(8.D0*BETAS - 4.D0*ALPHA)*(DRFA-DRFB)/G
      RETURN
C
C
C     LOGGAMMA 1 DISTRIBUTION
C
   50 CONTINUE
      W1 = DXP(-THETA -ALPHA*DLG(1.D0+BETA))
      IF (IPOWER .EQ. 1) RETURN
      W2 = DXP(-2.D0*THETA - ALPHA*DLG(1.D0+2.D0*BETA))
      IF (IPOWER .EQ. 2) RETURN
      W3 = DXP(-3.D0*THETA - ALPHA*DLG(1.D0+3.D0*BETA))
      IF (IPOWER .EQ. 3) RETURN
      W4 = DXP(-4.D0*THETA - ALPHA*DLG(1.D0+4.D0*BETA))
      RETURN
C
C     LOGGAMMA 2 DISTRIBUTION
C
   60 CONTINUE
      W1 = 1.D0 - DXP(-THETA -ALPHA*DLG(1.D0+BETA))
      IF (IPOWER .EQ. 1) RETURN
      W2 = 1.D0 - 2.D0*DXP(-THETA -ALPHA*DLG(1.D0+BETA))
     >+ DXP (-2.D0*THETA - ALPHA*DLG(1.D0+2.D0*BETA))
      IF (IPOWER .EQ. 2) RETURN
      W3 = 1.D0 - 3.D0*W1 + 3.D0*W2 -DXP(-3.D0*THETA
     > -ALPHA*DLG(1.D0+3.D0*BETA))
      IF (IPOWER .EQ. 3) RETURN
      W4 = DXP(-4.D0*THETA -ALPHA*DLG(1.D0 +4.D0*BETA))
     > -1.D0 + 4.D0*W1 -6.D0*W2 +4.D0*W3
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE SMPLMM
C
C***********************************************************************
C*
C*        THIS SUBROUTINE COMPUTES THE SAMPLE MOMENTS OF A SET OF DATA
C*     THE DATA SHOULD BE PASSED THROUGH THE COMMON /FDDATA/
C*
C*----------------------------------------------------------------------
C*
C*    WRITTEN BY:- V.A.SAMARANAYAKE
C*                 KANSAS STATE UNIVERSITY,
C*                 MANHATTAN, KANSAS 66506.
C*
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER OUTPUT
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /SMPL/ PHAT,PHATSQ,SPSQ,RNSTAR,RN,N,IDID,ISPRNT
      COMMON /FDDATA/ FI(500),XNI(500)
      COMMON /IOMM/ INPARM,INDATA,OUTPUT
      PHAT = 0.D0
      SPSQ = 0.D0
      RNSTAR = 0.D0
      DO 10 I=1,N
      PI = FI(I)/XNI(I)
      PHAT = PHAT + PI
      SPSQ = SPSQ + PI*PI
      RNSTAR = RNSTAR + 1.D0/XNI(I)
   10 CONTINUE
      PHAT = PHAT/RN
      SPSQ = ( SPSQ - RN*PHAT*PHAT)/(RN-1.D0)
      RNSTAR = RN/RNSTAR
      PHATSQ = PHAT*PHAT
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE DHEADD
C
C***********************************************************************
C*
C*    THIS SUBROUTINE PRINTS THE THE NAME OF A GIVEN DISTRIBUTION
C*    AS A HEADING. THE DISTRIBUTIONS INCLUDED IN THIS SUBROUTINE
C*    ARE THOSE USED IN THE FAILIURE ON DEMAND CASE AND THE DISTRIBUTION
C*    CODE MUST BE PASSED THROUGH THE VARIABLE IPRIR IN COMMON BLOCK
C*    PRIR. THE UNIT NUMBER OF THE OUTPUT DEVICE SHOULD BE PASSED
C*    THROUGH THE VARIABLE OUTPUT IN COMMON BLOCK IOMM.
C*
C*----------------------------------------------------------------------
C*
C*    WRITTEN BY:- V.A.SAMARANAYAKE
C*                 KANSAS STATE UNIVERSITY,
C*                 MANHATTAN, KANSAS 66506.
C*
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /IOMM/ INPARM,INDATA,OUTPUT
      INTEGER OUTPUT
C
C     PRINT HEADING OF THE APPROPRIATE DISTRIBUTION
C
      II = -IPRIR
      GO TO (1,2,3,4,5,6),II
    1 CONTINUE
      WRITE (OUTPUT,2001)
 2001 FORMAT ('1',30X,'******   BETA DISTRIBUTION   ******')
      GO TO 7
    2 CONTINUE
      WRITE (OUTPUT,2002)
 2002 FORMAT ('1',30X,'******   LOG BETA DISTRIBUTION   ******')
      GO TO 7
    3 CONTINUE
      WRITE (OUTPUT,2003)
 2003 FORMAT ('1',30X,'******   LOGNORMAL 1 DISTRIBUTION   ******')
      GO TO 7
    4 CONTINUE
      WRITE (OUTPUT,2004)
 2004 FORMAT ('1',30X,'******   LOGNORMAL 2 DISTRIBUTION   ******')
      GO TO 7
    5 CONTINUE
      WRITE (OUTPUT,2005)
 2005 FORMAT ('1',30X,'******   LOGGAMMA 1 DISTRIBUTION   ******')
      GO TO 7
    6 CONTINUE
      WRITE (OUTPUT,2006)
 2006 FORMAT ('1',30X,'******   LOGGAMMA 2 DISTRIBUTION   ******')
C
    7 CONTINUE
      RETURN
      END
C+*****************************************************************
C
      DOUBLE PRECISION FUNCTION DERFF(X)
C
C******************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION A(36)
      LOGICAL LARGE
      DATA A/1.095471299777623,-.289175401126989,0.110456398633795,
     >-.041253188227857,.014082838070652,-.004329295447431,.001198271901
     >592,-.000299972962353,.000068325860379,-.000014246988455,
     >.000002735408773,-.000000486191287,.000000080387276,-.000000012418
     >418,.000000001799533,-.245479D-9,.31625D-10,-.3859D-11,.447D-12,
     >-.49D-13,.5D-14,.975083423708556,-.024049393850415,.82045224088D-3
     >,-.43429308130D-4,.3018447034D-5,-.254473319D-6,.24858353D-7,
     >-.2731720D-8,.330847D-9,-.43505D-10,.6141D-11,-.922D-12,.146D-12,
     >-.24D-13,.4D-14/
      DATA NMAX1,NMIN1,NMAX2,NMIN2/21,1,36,23/
      DATA CONSTT/1.128379167095512/
C*  INITIALIZATION
      B2=0.0D0
      B1=0.0D0
      XABS=DABS(X)
      IF(XABS-3.0D0)1,1,2
    1 LARGE=.FALSE.
      XX=X/3.0D0
      NMAX=NMAX1
      NMIN=NMIN1
      X2=4.0D0*XX*XX - 2.0D0
      GO TO 3
    2 LARGE=.TRUE.
      IF(XABS.LT.6.0D0) GO TO 8
      DERFF=1.0D0
      RETURN
    8 XX=18.0D0/(X*X)-1.0D0
      NMAX=NMAX2
      NMIN=NMIN2
      X2=2.0D0*XX
    3 CONTINUE
C
C**** CALCULATION OF ERF FUNCTION
      N=NMAX+1
    4 N=N-1
      IF(N-NMIN) 6,5,5
    5 B=A(N)-B2+B1*X2
      B2=B1
      B1=B
      GO TO 4
C
C**** TERMINATION OF CALCULATION
    6 IF(LARGE) GO TO 7
      DERFF=XX*(B1-B2)*CONSTT
      RETURN
    7 FAC=DEXP(-X*X)*0.5D0/X
      CERF=FAC*(A(NMIN-1)-B2+B1*XX)
      DERFF=1.0D0 - CERF*CONSTT
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE RANGED
C
C***********************************************************************
C*
C*  THIS SUBROUTINE COMPUTES THE LOWER AND UPPER LIMITS OF THE RANGE OF
C*  FAILIER PROBABILITY P.
C*  IT PASSES THE LIMITS THROUGH THE ARRAY RNG IN COMMON BLOCK PRCT.
C*
C*  SUBROUTINES NEEDED: DXP
C*
C*  COMMON BLOCK /PRIR/ PARM(4),RNG(2),IPRIR
C*     PARM(1) - ALPHA, PARM(2) - BETA, PARM(3) - FIRST RANGE PARAMETER
C*               PARM(4) - SECOND RANGE PARAMETER,
C*     IPRIR   - DENSITY CODE,
C*     RNG(1)  - LOWER LIMIT OF FAILIER PROBABILITY,
C*     RNG(2)  - UPPER LIMIT OF FAILIER PROBABILITY.
C*
C*  WRITTEN BY V.A.SAMARANAYAKE.
C*
C***********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      EQUIVALENCE   (RNG(1),PMIN),(RNG(2),PMAX)
      II = -1*IPRIR
      GO TO (21,22,23,24,25,26),II
   21 CONTINUE
      PMIN=PARM(3)
      PMAX=PARM(4)
      GO TO 30
   22 CONTINUE
      PMIN = DXP(-1.D0*PARM(4))
      PMAX = DXP(-1.D0*PARM(3))
      GO TO 30
   23 CONTINUE
      PMIN = 1.D0/(DXP(PARM(3))+1.D0)
      PMAX = 1.D0/(DXP(PARM(4))+1.D0)
      GO TO 30
   24 CONTINUE
      PMIN = DXP(-1.D0*PARM(3))
      PMAX = DXP(-1.D0*PARM(4))
      GO TO 30
   25 CONTINUE
      PMIN =0
      PMAX=DXP(-1.D0*PARM(3))
      GO TO 30
   26 CONTINUE
      PMIN =1.D0 - ( DXP(-1.D0*PARM(3)))
      PMAX=1.D0
   30 CONTINUE
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION MFUNCD(X)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /SMPL/ PHAT,PHATSQ,SPSQ,RNSTAR,RN,N,IDID,ISPRNT
      COMMON /POPL/ W(4),IPOWER,IDIST,IMPRNT
C
C
C
      EQUIVALENCE (PARM(1),ALPHA),(PARM(2),BETA),(PARM(3),THETA)
      PARM2 = PARM(2)
      PARM(2) = X
C
      IF (IPRIR .EQ. -6) GO TO 60
      IF (IDIST .EQ. 2) GO TO 20
      MFUNCD = (2.D0*THETA+DLG(SPSQ+PHATSQ))*DLG(BETA+1.D0)
     > -DLG(2.D0*BETA +1.D0)*(THETA+DLG(PHAT))
      GO TO 999
C
C     NEED FUNCTION FOR MARGINAL CASE
C
   20 CONTINUE
      MFUNCD = (2.D0*THETA + DLG(SPSQ+PHATSQ-PHAT/RNSTAR))
     > * (DLG(BETA+1.D0)) - DLG((RNSTAR-1.D0)/RNSTAR)*DLG(BETA+1.D0)
     > - (THETA+DLG(PHAT))*DLG(2.D0*BETA+1.D0)
      GO TO 999
C
C      COMPUTE MFUNCD FOR LOGGAMMA 2 DISTRIBUTION
C
   60 CONTINUE
      IF (IDIST .EQ. 2) GO TO 80
      EXPON = -(THETA + DLG(1.D0-PHAT))/DLG(BETA+1.D0)
      MFUNCD = 2.D0*THETA + DLG(SPSQ) - DLG(1.D0/((2.D0*BETA+1.D0)**
     > EXPON) - 1.D0/((BETA+1.D0)**(2.D0*EXPON)))
C
      GO TO 999
C     COMPUTE MFUNCD FOR THE MARGINAL CASE
C
   80 CONTINUE
      MFUNCD = 2.D0*THETA + DLG(SPSQ) - DLG(DXP(THETA)/(RNSTAR*(BETA
     > +1.D0)**ALPHA) - 1.D0/((BETA+1.D0)**(2.D0*ALPHA))
     > + (RNSTAR-1.D0)/(RNSTAR*(2.D0*BETA+1.D0)**ALPHA))
C
  999 CONTINUE
      PARM(2) = PARM2
C
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION SUMSQD(XX)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION RX(2)
      COMMON /ASMIN/ FVECTT(2),EPS,ANS(2),S(2),Y(15),K,NEV,ICONT
      DIMENSION XX(2)
      EQUIVALENCE (FVECTT(1),FVECT1),(FVECTT(2),FVECT2)
      RX(1) = XX(1)
      RX(2) = XX(2)
      CALL MMTEQD (2,2,RX,FVECTT,NEV)
      SUMSQD = (FVECT1*FVECT1) + (FVECT2*FVECT2)
      RETURN
      END
C+** MLSIMD ************************************************************
C
      SUBROUTINE MLSIMD (EPSI,ANS,S,NEV,Y,IER,IPRT)
C
C*** MLSIMD ************************************************************
C*  PURPOSE : COMPUTE MAX.LIKELIHOOD ESTIMATORS (2 PARAMETERS)
C*            FOR FAILURE-ON-DEMAND CASE
C*            BY USING SUBROUTINE SIMIN
C*  COMMON STATEMENTS NEEDED :
C*            COMMON  /PRIR/ PARM(4),RNG(2),IPRIR
C*            COMMON /DATA1D/ NN
C*            COMMON /DATA2D/ KD(500),ND(500)
C*            COMMON /ACPRMD/ EPS
C*  SUBROUTINE REQUIRED : FSIMD SIMIN MRGCLD
C*  PARAMETERS DESCRIPTION :
C*   EPSI,ANS,NEV,Y,S SEE SUBROUTINE SIMIN
C*    IF EPSI=0.,EPSI=1.0D-6 IS USED
C*    IF NEV=0,NEV=100 IS USED
C*    IF S(1)=-,S(1)&S(2) ARE USED (SEE SIMIN)
C*       S(1)=0.,10% OF INITIAL GUESS VALUES ARE USED FOR S(1)&S(2)
C*       S(1)=+,S(1)% OF INITIAL GUESS VALUES ARE USED FOR S(1)&S(2)
C*   PARM,RNG,IPRIR,EPS SEE SUBROUTINE HINTD
C*   NN,KD,ND FAILURE DATA
C*    IER     ERROR CODE
C*     IER=0  NORMAL RETURN
C*     IER=1  FAILS TO CONVERGE WITHIN SPECIFIED NO.OF SEARCH
C*     IER=2  SEE MRGCLD
C*    IPRT    PRINTING CONTROL
C*     IPRT=0 NO INTERMEDIAT PRINTOUT
C*     IPRT=1 PRINT INTERMEDIATE RESULTS
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ANS(2),Y(15),S(2)
      COMMON /ERROR/ IERR,IPT
      EXTERNAL FSIMD
      IPT=IPRT
      IF(EPSI.EQ.0.0D0) EPSI=1.0D-6
      IF(NEV.EQ.0) NEV=100
      IF(S(1)) 400,200,300
200   S(1)=-0.1D0*ANS(1)
      S(2)=0.1D0*ANS(2)
      GO TO 400
300   S(2)=S(1)*ANS(2)/100.D0
      S(1)=-S(1)*ANS(1)/100.D0
400   NEVX=NEV
      CALL SIMIN(FSIMD,2,EPSI,ANS,S,NEV,0,Y)
      IF(IERR.GT.1) GO TO 410
      GO TO 420
410   IER=IERR
      RETURN
420   IF(NEV.GE.NEVX) GO TO 500
      IER=0
      RETURN
500   IER=1
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION FSIMD(X)
C
C***********************************************************************
C   CALCULATE NEG.OF ML FUNCTION
C   (SUPPLY FUNCTION VALUE TO SUBROUTINE SIMIN)
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(2)
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /DATA1D/ NN
      COMMON /DATA2D/ KD(500),ND(500)
      COMMON /DATAD/ K,N
      COMMON /ERROR/ IERR,IPT
      COMMON /GENCOM/ IGOUT
      Z=1.0D-10
      IND=-IPRIR
      GO TO (50,50,60,60,50,50,60),IND
50    IF((X(1).GT.Z).AND.(X(2).GT.Z)) GO TO 99
      FSIMD=1.0D20
      RETURN
60    IF(X(2).GT.Z) GO TO 99
      FSIMD=1.0D20
      RETURN
99    PARM(1)=X(1)
      PARM(2)=X(2)
      FSIMD=0.0D00
      DO 100 I=1,NN
      K=KD(I)
      N=ND(I)
      CALL MRGCLD(ANS,IERR)
      IF(IERR.EQ.1) GO TO 110
      WRITE(IGOUT,600)IERR
600   FORMAT(/T2,'*** ERROR *** RETURN CODE FROM MRGCLD IS ',I6/)
      RETURN
110   IF(ANS.NE.0.D0) GO TO 115
      Y=-1.D75
      GO TO 100
115   Y=DLG(ANS)
100   FSIMD=FSIMD+Y
      FSIMD=-FSIMD
      IF(IPT.EQ.1) WRITE(IGOUT,610) X,FSIMD
610   FORMAT(T2,'X(1),X(2),NEG.LF. : ',3(1X,G15.8))
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE MMVARD (SIGMA,VAR,N,IIDIST,IVERR)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /POPL/ W(4),IPOWER,IDIST,IMPRNT
      COMMON /TRNS/ T(4),TIN(4),RRNST,IPARA,ISNGLR
      COMMON /FDDATA/ FI(500),XNI(500)
      DIMENSION SIGMA(3),VAR(3),STOREW(4)
C
C
      EQUIVALENCE (T(1),T1),(T(2),T2),(T(3),T3),(T(4),T4)
C
C     INITIALIZE
C
      IDIST = IIDIST
      ISNGLR = 0
      IVERR = 0
      RN = DBLE(N)
C
C
C     COMPUTE RNSTAR VALUES
C
      RNSTAR = 0.D0
      RNSTR2 = 0.D0
      RNSTR3 = 0.D0
      DO 11 I=1,N
      RNI = XNI(I)
      RNSTAR = RNSTAR + 1.D0/RNI
      RNSTR2 = RNSTR2 + 1.D0/(RNI*RNI)
      RNSTR3 = RNSTR3 + 1.D0/(RNI*RNI*RNI)
   11 CONTINUE
      RNSTAR = 10.D0/RNSTAR
      RNSTR2 = 10.D0/RNSTR2
      RNSTR3 = 10.D0/RNSTR3
C
C
C     COMPUTE THE FIRST 4 MOMENTS (ABOUT 0) OF PRIR
C     DISTRIBUTION OF INTEREST AT THE GIVEN POINT
C     (ALPHA,BETA).
C
      IPOWER = 4
      CALL POPMMD
      DO 10 I=1,4
      STOREW(I) = W(I)
   10 CONTINUE
C
C     COMPUTE THE VARIANCE COVARIANCE MATIX OF
C     THE VECTOR (LHAT,SLSQ)' .
C
      CALL SMVARD (SIGMA,RN,RNSTAR,RNSTR2,RNSTR3)
C
C     GET THE TRANSFORMATION MATRIX
C
      CALL TRANSD (RNSTAR)
C
C
C     LOAD ORIGINAL MOMENTSINTO W ARRAY
C
      DO 20 I = 1,4
      W(I) = STOREW(I)
   20 CONTINUE
C
C
C     COMPUTE THE VARIANCE COVARIANCE MATRIX OF
C     THE VECTOR (ALPHAHAT,BETAHAT)' .
C
C
      S1 = SIGMA(1)
      S2 = SIGMA(2)
      S3 = SIGMA(3)
      IF (ISNGLR .EQ. 1)  GO TO 99
      VAR(1) = T1*T1*S1 + 2.D0*T1*T2*S3 + T2*T2*S2
      VAR(2) = T3*T3*S1 + 2.D0*T3*T4*S3 + T4*T4*S2
      VAR(3) = T1*T3*S1 + T2*T3*S3 + T1*T4*S3 + T2*T4*S2
      RETURN
C
C     TINVERSE IS SINGULAR. CANNOT DETERMINE VARIANCE
C     COVARIANCE MATRIX OF (ALPHAHAT,BETAHAT).
C
   99 CONTINUE
      IVERR = 1
      DO 98 I=1,3
      VAR(I) = 0.D0
   98 CONTINUE
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE SMVARD (SIGMA,RN,RNSTAR,RNSTR2,RNSTR3)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /POPL/ W(4),IPOWER,IDIST,IMPRNT
      DIMENSION SIGMA(3)
C
C
      EQUIVALENCE (W(1),W1),(W(2),W2),(W(3),W3),(W(4),W4)
C
C
C     COMPUTE CENTRAL MOMENTS
C
      W1W1 = W1*W1
      CW2 = W2 - W1W1
      CW3 = W3 - 3.D0*W2*W1 + 2.D0*W1W1*W1
      CW4 = W4 - 4.D0*W3*W1 + 6.D0*W2*W1W1 -3.D0*W1W1*W1W1
C
C
C     COMPUTE OMEGA'S
C
      IF (IDIST .EQ. 1) GO TO 10
      CALL OMEGAD(W1,CW2,CW3,CW4,OMEGA0,OMEGA1,OMEGA2,OMEGA3,
     >OMEGA4,OMEGA5)
C
C     COMPUTE SIGMA
C
   10 CONTINUE
      SIGMA(1) = CW2/RN
      SIGMA(2) = (CW4 - ((RN-3.D0)/(RN-1.D0))*CW2*CW2)/RN
      SIGMA(3) = CW3/RN
      IF (IDIST .EQ. 1) RETURN
C
C     COMPUTE SIGMA FOR THE ESTIMATES BASED ON MARGINAL
C
      ADD1 = (OMEGA0/RNSTAR)/RN
      ADD21 = (OMEGA3 - 2.D0*(RN-3.D0)*OMEGA0*CW2/(RN-1.D0))/RNSTAR
      ADD22 = (OMEGA4 - (RN*RN - 2.D0*RN +3.D0)*OMEGA0*OMEGA0/
     > ((RN-1.D0)*(RN-1.D0)))/RNSTR2
      ADD23 = OMEGA5/RNSTR3
      ADD24 = (2.D0*OMEGA0*OMEGA0*RN/((RN-1.D0)*(RN-1.D0)))/(RNSTAR*
     > RNSTAR)
      ADD2 = (ADD21+ADD22+ADD23+ADD24)/RN
      ADD3 = (OMEGA1/RNSTAR + OMEGA2/RNSTR2)/RN
C
      SIGMA(1) = SIGMA(1) + ADD1
      SIGMA(2) = SIGMA(2) + ADD2
      SIGMA(3) = SIGMA(3) + ADD3
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE OMEGAD (W1,CW2,CW3,CW4,OMEGA0,OMEGA1,
     >OMEGA2,OMEGA3,OMEGA4,OMEGA5)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     COMPUTE OMEGA'S
C
      W1W1 = W1*W1
      OMEGA0 = W1-CW2-W1W1
      OMEGA1 = 3.D0*(CW2-CW3-2.D0*W1*CW2)
      OMEGA2 = 2.D0*CW3 +6.D0*W1*CW2 + W1 + 2.D0*W1W1*W1 - 3.D0*CW2
     > - 3.D0*W1W1
      OMEGA3 = 6.D0*(CW3 -2.D0*W1*CW3 + W1*CW2 - W1W1*CW2 - CW4)
      OMEGA4 = 11.D0*CW4 + 36.D0*W1*CW3 +42.D0*W1W1*CW2 + 3.D0*W1W1*W1W1
     > - 18.D0*CW3 -42.D0*W1*CW2 -6.D0*W1W1*W1 +3.D0*W1W1 +7.D0*CW2
      OMEGA5 = W1 -7.D0*W1W1 -7.D0*CW2 +12.D0*CW3 +36.D0*W1*CW2
     > + 12.D0 *W1W1*W1 -6.D0*CW4 -24.D0*W1*CW3 -36.D0*W1W1*CW2
     > -6.D0*W1W1*W1W1
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE TRANSD (RNSTAR)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER OUTPUT
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /POPL/ W(4),IPOWER,IDIST,INPRNT
      COMMON /TRNS/ T(4),TIN(4),RRNST,IPARA,ISNGLR
      COMMON /IOMM/ INPARM,INDATA,OUTPUT
      EXTERNAL F1D
      EXTERNAL F2D
C
C
C
C     SET VALUES FOR DIFFERENTIATION ROUTINE
C
      IH = 1
C
C     STORE ORIGINAL PARM VALUES
C
      PARM1 = PARM(1)
      PARM2 = PARM(2)
C
C     PASS VALUE OF RNSTAR TO F1 AND F2D THROUGH RRNST
C
      RRNST = RNSTAR
C
C     GET VALUES OF TRANSFORMATION MATRIX
C
      ALPHA = PARM(1)
      BETA = PARM(2)
C
      IPARA = 1
      H = ALPHA*1.0D-01
      CALL DCAR(ALPHA,H,IH,F1D,Z)
      TIN(1) = Z
C
      PARM(1) = PARM1
      CALL DCAR (ALPHA,H,IH,F2D,Z)
      TIN(3) = Z
C
      PARM(1) = PARM1
      IPARA = 2
      H = BETA *1.0D-01
      CALL DCAR ( BETA,H,IH,F1D,Z)
      TIN(2) = Z
C
      PARM(2) = PARM2
      CALL DCAR (BETA,H,IH,F2D,Z)
      TIN(4) = Z
      PARM(2) = PARM2
C
C     FIND INVERSE OF TINVERSE
C
      DET = TIN(1)*TIN(4) - TIN(3)*TIN(2)
      ABSDET = DABS(DET)
      IF (ABSDET .LT. 1.0D-77) GO TO 10
      ABSDET = DXP(DLG(ABSDET))
      IF (ABSDET .LT. 1.0D-77) GO TO 10
      DLADET = DLG(ABSDET)
      IDTSGN = 1
      IF (DET .LT. 0.D0) IDTSGN = -1
      ABST = DABS(TIN(4))
      ISIGN = 1
      IF (TIN(4) .LT. 0.D0) ISIGN = -1
      T(1) = DXP(DLG(ABST) -DLADET)*ISIGN*IDTSGN
      ISIGN = 1
      IF ( TIN(1) .LT. 0.D0) ISIGN = -1
      ABST = DABS(TIN(1))
      T(4) = DXP(DLG(ABST) - DLADET)*ISIGN*IDTSGN
      ISIGN = 1
      IF (TIN(2) .LT. 0.D0) ISIGN = -1
      ABST = DABS(TIN(2))
      T(2) = -DXP(DLG(ABST) - DLADET)*ISIGN*IDTSGN
      ISIGN = 1
      IF (TIN(3) .LT. 0.D0) ISIGN = -1
      ABST = DABS(TIN(3))
      ISIGN = ABST/TIN(3)
      T(3) = -DXP(DLG(ABST) - DLADET)*ISIGN*IDTSGN
      RETURN
C
C     DETERMINENT TOO SMALL
C
   10 CONTINUE
      WRITE(OUTPUT,100) DET
  100 FORMAT (' ','******* DETERMINANT OF INVERSE TRANSFORMATION
     >MATRIX TOO SMALL. VALUE OF DET IS =',D20.14)
      ISNGLR = 1
      DO 11 I=1,4
      T(I) = 0.D0
   11 CONTINUE
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION F1D(X)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /POPL/ W(4),IPOWER,IDIST,IMPRNT
      COMMON /TRNS/ T(4),TIN(4),RRNST,IPARA,ISNGLR
C
C     GET NEW PARM VALUE PASSED BY DCAR
C
      PARM(IPARA) = X
      IPOWER = 1
      CALL POPMMD
      F1D = W(1)
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION F2D(X)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /POPL/ W(4),IPOWER,IDIST,IMPRNT
      COMMON /TRNS/ T(4),TIN(4),RRNST,IPARA,ISNGLR
C
      EQUIVALENCE (W(1),W1),(W(2),W2)
C     GET NEW PARAMETER VALUE PASSED BY DCAR
C
      PARM(IPARA) = X
      IPOWER = 2
      CALL POPMMD
      F2D = W2 - W1*W1 + (IDIST - 1)*(W1-W2)/RRNST
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE DCAR(X,H8,IH,FCT,Z)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION AUX(5)
C
C
C     ASSIGN VALUE OF H8 TO SINGLE PRECISION H
C
      H = H8
C NO ACTION IF ZERO INTERVAL WIDTH
      IF(H) 1,17,1
C GENERATE STEP SIZE HH FOR DIVIDED DIFFERENCES
    1 C=DABS(H)
      IF(H) 2,9,2
    2 HH=5.0D-3
      IF(C-HH) 3,4,4
    3 HH=C
    4 A=FCT(X+HH)
      B=FCT(X-HH)
      Z=DABS((A-B)/(HH+HH))
      A=.5D0*DABS(A+B)
      HH=5.0D-3
      IF(A-1.D0)6,6,5
    5 HH=HH*A
    6 IF(Z-1.D0) 8,8,7
    7 HH=HH/Z
    8 IF (HH-C) 10,10,9
    9 HH=C
C INITIALIZE DIFFERENTIATION LOOP
   10 Z=(FCT(X+HH)-FCT(X-HH))/(HH+HH)
      J=5
      JJ=J-1
      AUX(J)=Z
      DH=HH/DBLE(J)
      DZ=1.D75
C START DIFFERENTIATION LOOP
   11 J=J-1
      C=J
      HH=C*DH
      AUX(J)=(FCT(X+HH)-FCT(X-HH))/(HH+HH)
C INITIALIZA EXTRAPOLATION LOOP
      D2=1.D75
      B=0.D0
      A=1.D0/C
C START EXTRAPOLATION LOOP
      DO 12 I=J,JJ
      D1=D2
      B=B+A
      HH=(AUX(I)-AUX(I+1))/(B*(2.D0+B))
      AUX(I+1)=AUX(I)+HH
C TEST ON OSCILLATING INCREMENTS
      D2=DABS(HH)
      IF(D2-D1) 12,13,13
   12 CONTINUE
C END OF EXTRAPOLATION LOOP
C UPDATE RESULT FOR Z
      I=JJ+1
      GO TO 14
   13 D2=D1
      JJ=1
   14 IF(D2-DZ) 15,16,16
   15 DZ=D2
      Z=AUX(I)
   16 IF(J-1) 17,17,11
C END OF DIFFERENTIATION LOOP
   17 RETURN
      END
C+**********************************************************************
C
      SUBROUTINE VARMLD(AP,BT,DALPH,DBETA,V,NSIG,IER,IOPT,IPRT)
C
C***********************************************************************
C*  PURPOSE : COMPUTE COVARIANCE MATRIX OF MAX.LIKELIHOOD ESTIMATORS
C*            FOR FAILURE-ON-DEMAND CASE
C*            BY USING NUMERICAL DIFFERENTIATION
C*            WITH ERROR AT THE ORDER OF H**4
C*            AND APPROXIMATING EXPECTATION OF DERIVATIVES BY VALUES OF
C*            DERIVATIVES AT ESTIMATED PARAMETERS
C*            OR USING EXACT EXPECTATION OF DERIVATIVES
C*  COMMON STATEMENTS NEEDED :
C*            COMMON /PRIR/ PARM(4),RNG(2),IPRIR
C*            COMMON /DATA1D/ NN
C*            COMMON /DATA2D/ KD(500),ND(500)
C*            COMMON /ACPRMD/ EPS
C*  SUBROUTINE REQUIRED : VAML1D VAML2D MRGCLD
C*  PARAMETERS DESCRIPTION :
C*   IOPT     =1 USE APPROXIMATED VALUES OF EXPECTATIONS
C*            =2 USE EXACT VALUES OF EXPECTATIONS
C*   DALPH,DBETA
C*            IF DALPH=0,10% OF AP WILL BE USED
C*            IF DBETA=0,10% OF BT WILL BE USED
C*            IF DALPH IS NEG.,DALPH=DALPH% OF AP
C*            AND DBETA=DALPH% OF BT WILL BE USED
C*   EPS      IF EPS=0.,EPS=1.0D-6 IS USED
C*   NSIG     IF NSIG=0,NSIG=3 IS USED
C*   SEE DETAILS OF THE OTHER PARAMETERS IN SUBROUTINE VAML1D OR VAML2D
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION V(2,2)
      COMMON /ACPRMD/ EPS
      IF(EPS.EQ.0.0D0) EPS=1.0D-6
      IF(NSIG.EQ.0) NSIG=3
      IF(DALPH.GE.0.D0) GO TO 4
      DBETA=-DALPH*BT/100.D0
      DALPH=-DALPH*AP/100.D0
      GO TO 5
4     IF(DALPH.EQ.0.D0) DALPH=0.1D0*AP
      IF(DBETA.EQ.0.D0) DBETA=0.1D0*BT
5     GO TO (10,20),IOPT
10    CALL VAML1D(AP,BT,DALPH,DBETA,V,NSIG,IER,IPRT)
      RETURN
20    CALL VAML2D(AP,BT,DALPH,DBETA,V,NSIG,IER,IPRT)
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE VAML1D(AP,BT,DALPH,DBETA,V,NSIG,IER,IPRT)
C
C***********************************************************************
C*  PURPOSE : COMPUTE COVARIANCE MATRIX OF MAX.LIKELIHOOD ESTIMATORS
C*            FOR FAILURE-ON-DEMAND CASE
C*            BY USING NUMERICAL DIFFERENTIATION
C*            WITH ERROR AT THE ORDER OF H**2
C*            AND APPROXIMATING EXPECTATION OF DERIVATIVES BY VALUES OF
C*            DERIVATIVES AT ESTIMATED PARAMETERS
C*  COMMON STATEMENTS NEEDED :
C*            COMMON /PRIR/ PARM(4),RNG(2),IPRIR
C*            COMMON /DATA1D/ NN
C*            COMMON /DATA2D/ KD(500),ND(500)
C*            COMMON /ACPRMD/ EPS
C*  SUBROUTINE REQUIRED : MRGCLD
C*  PARAMETERS DESCRIPTION :
C*   INPUT :
C*   AP,BT    PARM(1),PARM(2) OF THE DISTRIBUTION
C*            SEE DETAILS IN SUBROUTINE HINTD
C*   PARM(3),PARM(4) SEE DETAILS IN SUBROUTINE HINTD
C*   IPRIR    SEE DETAILS IN SUBROUTINE HINTD
C*   DALPH    INITIAL INTERVAL SIZE OF AP,RETURN AS FINAL INTERVAL SIZE
C*   DBETA    INITIAL INTERVAL SIZE OF BT,RETURN AS FINAL INTERVAL SIZE
C*   NSIG     # OF SIGNIFICANT DIGITS REQUIRED FOR COVARIANCE MATRIX
C*   NN,KD(I),ND(I)  FAILURE DATA
C*    NN      SAMPLE SIZE
C*    KD      # OF FAILURES
C*    ND      # OF ATTEMPTS
C*   OUTPUT :
C*   V(1,1)   VARIANCE OF AP
C*   V(2,2)   VARIANCE OF BT
C*   V(1,2)   COVARIANCE OF AP AND BT
C*   IER      ERROR CODE
C*    IER=0   NORMAL RETURN
C*    IER=1   # OF SIGNIFICANT DIGITS REQUIRED CAN NOT BE ACCOMPLISHED
C*            AND NSIG WILL BE RETURNED AS CURRENT # OF SIGNIFICANT
C*            DIGITS
C*            THE COVARIANCE MATRIX COMPUTED BY USING INITIAL INTERVAL
C*            SIZE IS RETURNED
C*    IER=2   SEE SUBROUTINE MRGCAL FOR DETAILS
C*   IPRT
C*    IPRT=0  NO INTERMEDIAT PRINTOUT
C*    IPRT=1  PRINT INTERMEDIAT RESULTS
C*    IPRT=2  ONLY ONE ITERATION WILL BE MADE
C*   EPS      SEE SUBROUTINE HINTD FOR DETAIL
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION DA(3),DB(3),F(3),G(2)
      DIMENSION A(3),VX(3),VY(3),DIF(3),V(2,2),VZ(3)
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /DATAD/ K,N
      COMMON /DATA1D/ NN
      COMMON /DATA2D/ KD(500),ND(500)
      COMMON /GENCOM/ IGOUT
      SIG=DBLE(NSIG)
      ICT=1
80    DAP=DALPH
      DBT=DBETA
      DO 90 I=1,3
90    A(I)=0.0D00
      DA(1)=AP+DAP
      DA(3)=AP
      DA(2)=AP-DAP
      DB(1)=BT+DBT
      DB(3)=BT
      DB(2)=BT-DBT
      DO 100 I=1,NN
      N=ND(I)
      K=KD(I)
      PARM(2)=BT
      DO 120 J=1,3
      PARM(1)=DA(J)
      CALL MRGCLD(F(J),IER)
120   F(J)=DLG(F(J))
      D2LLAP=(F(1)-2.D0*F(3)+F(2))/(DAP*DAP)
      PARM(1)=AP
      DO 130 J=1,3
      PARM(2)=DB(J)
      CALL MRGCLD(F(J),IER)
130   F(J)=DLG(F(J))
      D2LLBT=(F(1)-2.D0*F(3)+F(2))/(DBT*DBT)
      DO 310 L=1,2
      PARM(1)=DA(L)
      DO 300 J=1,2
      PARM(2)=DB(J)
      CALL MRGCLD(F(J),IER)
300   F(J)=DLG(F(J))
310   G(L)=(F(1)-F(2))/(2.D0*DBT)
      D2LLAB=(G(1)-G(2))/(2.D0*DAP)
      A(1)=A(1)-D2LLAP
      A(2)=A(2)-D2LLBT
      A(3)=A(3)-D2LLAB
100   CONTINUE
      DM1=A(1)*A(2)-A(3)*A(3)
      VX(1)=A(2)/DM1
      VX(2)=A(1)/DM1
      VX(3)=-A(3)/DM1
      IF(IPRT-1) 139,210,205
205   V(1,1)=VX(1)
      V(2,2)=VX(2)
      V(1,2)=VX(3)
      V(2,1)=VX(3)
      WRITE(IGOUT,605) A(1),A(3),A(3),A(2)
605   FORMAT(/T2,'INFORMATION MATRIX : ',(T23,2(1X,G15.8)))
      IER=0
      RETURN
210   WRITE(IGOUT,610)DAP,DBT,A(1),A(3),A(3),A(2),VX(1),VX(3),VX(3),
     >VX(2)
610   FORMAT(/T2,'INTERVAL SIZE : ',2(1X,G15.8)
     >/T2,'INFORMATION MATRIX : ',T23,2(1X,G15.8)/T23,2(1X,G15.8)
     >/T2,'COVARIANCE MATRIX : ',(T23,2(1X,G15.8)))
139   IF(ICT.EQ.1) GO TO 150
      DO 140 I=1,3
140   DIF(I)=DABS((VX(I)-VY(I))/VY(I))
      SIGX=-DLOG10(DMAX1(DIF(1),DIF(2),DIF(3)))
      IF(SIGX.LT.SIG) GO TO 145
      NSIG=SIGX
      V(1,1)=VX(1)
      V(2,2)=VX(2)
      V(1,2)=VX(3)
      V(2,1)=VX(3)
      IER=0
      RETURN
145   IF(ICT.EQ.2) GO TO 147
      IF(SIGY.LE.SIGX) GO TO 147
      IER=1
      NSIG=SIGX
      V(1,1)=VZ(1)
      V(2,2)=VZ(2)
      V(1,2)=VZ(3)
      V(2,1)=VZ(3)
      RETURN
147   SIGY=SIGX
      GO TO 152
150   DO 151 I=1,3
151   VZ(I)=VX(I)
152   DO 160 I=1,3
160   VY(I)=VX(I)
      ICT=ICT+1
      DALPH=DALPH*0.1D00
      DBETA=DBETA*0.1D00
      GO TO 80
      END
C+**********************************************************************
C
      SUBROUTINE VAML2D(AP,BT,DALPH,DBETA,V,NSIG,IER,IPRT)
C
C***********************************************************************
C*  PURPOSE : COMPUTE COVARIANCE MATRIX OF MAX.LIKELIHOOD ESTIMATORS
C*            FOR FAILURE-ON-DEMAND CASE
C*            BY USING NUMERICAL DIFFERENTIATION
C*            WITH ERROR AT THE ORDER OF H**4
C*            AND EXACT EXPECTATION OF DERIVATIVES
C*            (PRODUCT OF 1ST ORDER DERIVATIVES)
C*  COMMON STATEMENTS NEEDED :
C*            COMMON /PRIR/ PARM(4),RNG(2),IPRIR
C*            COMMON /DATA1D/ NN
C*            COMMON /DATA2D/ KD(500),ND(500)
C*            COMMON /ACPRMD/ EPS
C*  SUBROUTINE REQUIRED : MRGCLD
C*  PARAMETERS DESCRIPTION :
C*   INPUT :
C*   AP,BT    PARM(1),PARM(2) OF THE DISTRIBUTION
C*            SEE DETAILS IN SUBROUTINE HINTD
C*   PARM(3),PARM(4) SEE DETAILS IN SUBROUTINE HINTD
C*   IPRIR    SEE DETAILS IN SUBROUTINE HINTD
C*   DALPH    INITIAL INTERVAL SIZE OF AP,RETURN AS FINAL INTERVAL SIZE
C*   DBETA    INITIAL INTERVAL SIZE OF BT,RETURN AS FINAL INTERVAL SIZE
C*   NSIG     # OF SIGNIFICANT DIGITS REQUIRED FOR COVARIANCE MATRIX
C*   NN,KD(I),ND(I)  FAILURE DATA
C*    NN      SAMPLE SIZE
C*    KD      # OF FAILURES
C*    ND      # OF ATTEMPTS
C*   OUTPUT :
C*   V(1,1)   VARIANCE OF AP
C*   V(2,2)   VARIANCE OF BT
C*   V(1,2)   COVARIANCE OF AP AND BT
C*   IER      ERROR CODE
C*    IER=0   NORMAL RETURN
C*    IER=1   # OF SIGNIFICANT DIGITS REQUIRED CAN NOT BE ACCOMPLISHED
C*            AND NSIG WILL BE RETURNED AS CURRENT # OF SIGNIFICANT
C*            DIGITS
C*            THE COVARIANCE MATRIX COMPUTED BY USING INITIAL INTERVAL
C*            SIZE IS RETURNED
C*    IER=2   SEE SUBROUTINE MRGCAL FOR DETAILS
C*   IPRT
C*    IPRT=0  NO INTERMEDIAT PRINTOUT
C*    IPRT=1  PRINT INTERMEDIAT RESULTS
C*    IPRT=2  ONLY ONE ITERATION WILL BE MADE
C*   EPS      SEE SUBROUTINE HINTD FOR DETAIL
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION DA(4),DB(4),F(4)
      DIMENSION A(3),VX(3),VY(3),DIF(3),V(2,2),VZ(3)
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /DATAD/ K,N
      COMMON /DATA1D/ NN
      COMMON /DATA2D/ KD(500),ND(500)
      COMMON /GENCOM/ IGOUT
      SIG=DBLE(NSIG)
      ICT=1
80    DAP=DALPH
      DBT=DBETA
      DO 90 I=1,3
90    A(I)=0.0D00
      DAP2=2.D0*DAP
      DA(1)=AP-DAP2
      DA(2)=AP+DAP
      DA(3)=AP-DAP
      DA(4)=AP+DAP2
      DBT2=2.D0*DBT
      DB(1)=BT-DBT2
      DB(2)=BT+DBT
      DB(3)=BT-DBT
      DB(4)=BT+DBT2
      DO 100 I=1,NN
      NDM=ND(I)+1
      N=ND(I)
      DO 100 KDM=1,NDM
      K=KDM-1
      PARM(1)=AP
      PARM(2)=BT
      CALL MRGCLD(FAB,IER)
96    DO 120 J=1,4
      PARM(1)=DA(J)
      CALL MRGCLD(F(J),IER)
120   F(J)=DLG(F(J))
      DLLAP=((F(1)+8.D0*F(2))-(8.D0*F(3)+F(4)))/(12.D0*DAP)
      PARM(1)=AP
      DO 130 J=1,4
      PARM(2)=DB(J)
      CALL MRGCLD(F(J),IER)
130   F(J)=DLG(F(J))
      DLLBT=((F(1)+8.D0*F(2))-(8.D0*F(3)+F(4)))/(12.D0*DBT)
      A(1)=A(1)+DLLAP*DLLAP*FAB
      A(2)=A(2)+DLLBT*DLLBT*FAB
      A(3)=A(3)+DLLAP*DLLBT*FAB
100   CONTINUE
      DM1=A(1)*A(2)-A(3)*A(3)
      VX(1)=A(2)/DM1
      VX(2)=A(1)/DM1
      VX(3)=-A(3)/DM1
      IF(IPRT-1) 139,210,205
205   V(1,1)=VX(1)
      V(2,2)=VX(2)
      V(1,2)=VX(3)
      V(2,1)=VX(3)
      WRITE(IGOUT,605) A(1),A(3),A(3),A(2)
605   FORMAT(/T2,'INFORMATION MATRIX : ',(T23,2(1X,G15.8)))
      IER=0
      RETURN
210   WRITE(IGOUT,610)DAP,DBT,A(1),A(3),A(3),A(2),VX(1),VX(3),VX(3),
     >VX(2)
610   FORMAT(/T2,'INTERVAL SIZE : ',2(1X,G15.8)
     >/T2,'INFORMATION MATRIX : ',T23,2(1X,G15.8)/T23,2(1X,G15.8)
     >/T2,'COVARIANCE MATRIX : ',(T23,2(1X,G15.8)))
139   IF(ICT.EQ.1) GO TO 150
      DO 140 I=1,3
140   DIF(I)=DABS((VX(I)-VY(I))/VY(I))
      SIGX=-DLOG10(DMAX1(DIF(1),DIF(2),DIF(3)))
      IF(SIGX.LT.SIG) GO TO 145
      NSIG=SIGX
      V(1,1)=VX(1)
      V(2,2)=VX(2)
      V(1,2)=VX(3)
      V(2,1)=VX(3)
      IER=0
      RETURN
145   IF(ICT.EQ.2) GO TO 147
      IF(SIGY.LE.SIGX) GO TO 147
      IER=1
      NSIG=SIGX
      V(1,1)=VZ(1)
      V(2,2)=VZ(2)
      V(1,2)=VZ(3)
      V(2,1)=VZ(3)
      RETURN
147   SIGY=SIGX
      GO TO 152
150   DO 151 I=1,3
151   VZ(I)=VX(I)
152   DO 160 I=1,3
160   VY(I)=VX(I)
      ICT=ICT+1
      DALPH=DALPH*0.1D00
      DBETA=DBETA*0.1D00
      GO TO 80
      END
C+**********************************************************************
C
      SUBROUTINE PRMOMD(PW)
C
C***********************************************************************
C* COMPUTE FIRST FOUR CENTRAL MOMENTS,SKEWNESS AND
C* KURTOSIS FROM FIRST FOUR MOMENTS ABOUT ZERO OF THE PRIOR
C* FOR FAILURE RATE CASE
C*  COMMON STATEMENTS NEEDED :
C*   COMMON /PRIR/ PARM(4),RNG(2),IPRIOR
C*   COMMON /AQNC7/ ERR,IERR2
C*  SUBROUTINE REQUIRED : POPMM
C*  INPUT PARAMETERS : PARM(4),IPRIOR,ERR
C*    ERR ACCURACY PARAMETER IN SUBROUTINE QNC7
C*  OUTPUT PARAMETERS : IERR2, PW(10)
C*   IERR2 ERROR CODE RETURN FROM QNC7
C*   (FIRST FOUR MOMENTS ABOUT ZERO,
C*   FIRST FOUR CENTRAL MOMENTS,SKEWNESS & KURTOSIS)
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION PW(10)
      COMMON /AQNC7/ ERR,IERR2
      COMMON /POPL/ W(4),IPOWER,IDIST,IMPRNT
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      IPOWER=4
      ERR=1.0D-6
      IDIST=1
      IMPRNT=0
      CALL POPMMD
      DO 100 I=1,4
100   PW(I)=W(I)
      PW(5)=0.D0
      PW(6)=W(2)-W(1)*W(1)
      PW(7)=W(3)-3.D0*W(2)*W(1)+2.D0*W(1)**3
      PW(8)=W(4)-4.D0*W(3)*W(1)+6.D0*W(2)*W(1)*W(1)-3.D0*W(1)**4
      PW(9)=PW(7)/(PW(6)**1.5D0)
      PW(10)=PW(8)/(PW(6)*PW(6))
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE PUPJKS (XL,XR,AL,IDIST,FV,IERR)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C*                                                                     *
C*    THIS ROUTINE LOCATES AN UPPER AND  LOWER DENSITY VALUE SUCH THAT *
C* THE  TWO DENSITY  VALUES ARE  EQUAL WHILE  THE PROBABILITY  OUTSIDE *
C* THESE VALUES IS EQUAL TO SOME GIVEN PROBABILITY.                    *
C*                                                                     *
C*    THE  METHOD   INVOLVES  FINDING   THE  ROOT   OF  THE   EQUATION *
C* CUMM(P-UPPER)-CUMM(P-LOWER)-(1-ALPHA)=0, WHERE ALPHA=AL.   THE DEN- *
C* SITY VALUE OF P-UPPER IS CALCULATED  USING THE ROOT FINDING ROUTINE *
C* DRTMI.  THE EQUATION  SOLVED  IS  DENS(X)-DENS(P-LOWER)=0. FOR  EACH*
C* EVALUATION  OF  THE FIRST  EQUATION  THE  SECOND EQUATION  MUST  BE *
C* SOLVED.                                                             *
C*                                                                     *
C*    THIS ROUTINE IS DESIGNED TO  MINIMIZE CUMMULATIVE FUNCTION CALLS *
C* BY USING THE DENSITY FUNCTION  FOR MOST EVALUATIONS.  FEWER DENSITY *
C* FUNCTION EVALUATIONS MAY BE OBTAINED USING BETTER ESTIMATES FOR THE *
C* USER SUPPLIED DENSITY FUNCTION ENDPOINTS.                           *
C*                                                                     *
C*                                                                     *
C*    REQUIRED SUBROUTINES:  PCOM, CMFX, DENS, CUM, DRTMI, MAXFN, IMTRD*
C*                                                                     *
C*    NOTE:   DRTMI AND IMTRD ARE THE SAME ROUTINES UNDER DIFFERENT    *
C*            NAMES.                                                   *
C*                                                                     *
C*                                                                     *
C*                                                                     *
C*      PARAMETERS:                                                    *
C*                                                                     *
C*         XL = CLOSEST ZERO OF LOWER DENSITY FUNCTION                 *
C*                                                                     *
C*         XR = CLOSEST ZERO OF UPPER DENSITY FUNCTION                 *
C*                                                                     *
C*         AL = PROBABILITY DESIRED OUTSIDE P-UPPER AND P-LOWER        *
C*                                                                     *
C*      FV(1) = THE LOWER P-VALUE OF THE SHORTEST INTERVAL ENDPOINT.   *
C*                                                                     *
C*      FV(2) = THE LOWER ALPHA SOLUTION VALUE. (AREA BELOW P LOWER)   *
C*                                                                     *
C*      FV(3) = THE P-VALUE OF THE UPPER SHORTEST INTERVAL ENDPOINT.   *
C*                                                                     *
C*      FV(4) = THE UPPER ALPHA SOLUTION VALUE. (AREA ABOVE P-UPPER)   *
C*                                                                     *
C*         PANS = SOLUTION DENSITY VALUE FOR LOWER                     *
C*                                                                     *
C*         IERR = ERROR RETURN CODE FROM SEARCH ROUTINES.              *
C*                IERR IS NEGATIVE  IF ERROR OCCURS IN  MAXFN. IERR IS *
C*                POSITIVE IF ERROR OCCURS IN  DRTMI. REFER TO DOCUMEN-*
C*                TATION ON SEARCH ROUTINES FOR IERR VALUES.           *
C*         IDIST = 1 PRIOR
C*         IDIST = 2 POSTERIOR
C*                                                                     *
C*                                                                     *
C*    COMMON BLOCK /PASS/ VARIABLES:                                   *
C*                                                                     *
C*       PMAX = MODE OF THE DENSITY FUNCTION                           *
C*                                                                     *
C*       XLZ = XL FROM ABOVE.                                          *
C*                                                                     *
C*       XRZ = XR FROM ABOVE.                                          *
C*                                                                     *
C*       EPS = ACCURACY DESIRED IN ROOT FINDING ROUTINES               *
C*                                                                     *
C*       PVPASS = HOLDING VARIABLE FOR PASSING VALUES TO FUNCTIONS     *
C*                                                                     *
C*       ALPHA = TOTAL PROBABILITY DESIRED OUTSIDE THE UPPER           *
C*               AND LOWER DENSITY VALUE.                              *
C*                                                                     *
C*                           SCOTT HANSEN                              *
C*                      KANSAS STATE UNIVERSITY                        *
C*                         MANHATTAN, KANSAS                           *
C*                                                                     *
C*                                           DATE WRITTEN:  10/4/79.   *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION FV(6)
      COMMON /DIS/ IDIS
      COMMON /DIST/ VMARG,IMRGCL
      COMMON /PASS/ PMAX,XLZ,XRZ,EPS,PVPASS,ALPHA
      COMMON /GENCOM/ IGOUT
      EXTERNAL DENS,CUM,CMFX,PCOM,PUPV
      DO 100 I=1,6
100   FV(I)=0.D0
      IDIS=IDIST
      IMRGCL=0
      XLZ=XL
      XRZ=XR
      ALPHA=AL
      NITER=100
      EPS=1.0D-6
      CALL MAXFN (DENS,XLZ,XRZ,PMAX,FMAX,NITER,EPS,IERR)
      IF (IERR.GT.0) GOTO 900
      CALL DRTMI (PANS,F,CMFX,XLZ,PMAX,EPS,NITER,IERR)
      IF (IERR.GT.0) IERR=-IERR
      IF (IERR.NE.0) GOTO  900
      FV(1)=PANS
      FV(3)=CUM(PANS)
      PVPASS=PANS
      CALL DRTMI (FV(2),F,PCOM,PMAX,XRZ,EPS,NITER,IR)
      FV(4)=1-CUM(FV(2))
      FV(5)=DENS(FV(1))
      FV(6)=DENS(FV(2))
      RETURN
 900  WRITE (IGOUT,1000) IERR
 1000 FORMAT ('  FAILURE TO CONVERGE IN PUPJKS. IERR= ',I5)
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      DOUBLE PRECISION FUNCTION PUPV (PV)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C*                                                                     *
C*    THIS  ROUTINE USES  THE IMTRD  ROUTINE TO  CALCULATE THE  P-UPPER*
C* VALUE THAT MATCHES THE  P-LOWER VALUE PASSED TO IT  FROM THE PREDE- *
C* CESSOR ROUTINE  DRTMI. THE ROUTINE SETS  UP THE FUNCTION PCOM  TO BE*
C* EVALUATED BY PASSING IT THE DESIRED CONVERGANCE VALUE PV. THE VALUE *
C* IS PASSED THROUGH  THE COMMON BLOCK /PASS/  VARIABLE PVPASS.  NOTE: *
C* REFER TO PUPJKS DOCUMENTATION FOR INFO ON OTHER /PASS/ VARIABLES.   *
C*                                                                     *
C*                                                                     *
C*                                                                     *
C*                                                                     *
C*    REQUIRED SUBROUTINES :  IMTRD, PCOM                              *
C*                                                                     *
C*                                                                     *
C* NOTE:  IMTRD IS THE SAME ROUTINE AS DRTMI ONLY UNDER A DIFFERENT    *
C*        NAME.                                                        *
C*                                                                     *
C*                           S. D. HANSEN                              *
C*                      KANSAS STATE UNIVERSITY                        *
C*                         MANHATTAN, KANSAS                           *
C*                                                                     *
C*                                           DATE WRITTEN:  10/4/79.   *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PASS/ PMAX,XLZ,XRZ,EPS,PVPASS,ALPHA
      COMMON /GENCOM/ IGOUT
      EXTERNAL PCOM
      PVPASS=PV
      NITER=100
      CALL IMTRD (ANS,DUMMY,PCOM,PMAX,XRZ,EPS,NITER,IERR)
      IF (IERR.GE.1) GOTO 100
      PUPV=ANS
      RETURN
 100  WRITE (IGOUT,1000) IERR
 1000 FORMAT (' FAILURE TO CONVERGE IN IMTRD ROUTINE.  IERR= ',I5)
      PUPV=0.D0
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      DOUBLE PRECISION FUNCTION PCOM (PV)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C*                                                                     *
C*    THIS ROUTINE IS THE FUNCTION EVALUATED  BY IMTRD IN SEARCHING FOR*
C* THE  P-UPPER VALUE  THAT EQUALS  THE  P-LOWER VALUE.  THE VALUE  OF *
C* PVPASS IS SET IN  THE PUPV ROUTINE AND IS CONSTANT  DURING FOR EACH *
C* CALL OF IMTRD.                                                      *
C*                                                                     *
C*    REQUIRED SUBROUTINES: DENS                                       *
C*                                                                     *
C*                                                                     *
C*                                                                     *
C*    DENS(P) CALCULATES THE DENSITY VALUE AT P.                       *
C*                                                                     *
C*                                                                     *
C* NOTE:  PVPASS = P-LOWER VALUE.                                      *
C*                                                                     *
C*                           S. D. HANSEN                              *
C*                      KANSAS STATE UNIVERSITY                        *
C*                         MANHATTAN, KANSAS                           *
C*                                                                     *
C*                                           DATE WRITTEN:  10/4/79.   *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PASS/ PMAX,XLZ,XRZ,EPS,PVPASS,ALPHA
      PCOM=DABS(DENS(PV))-DABS(DENS(PVPASS))
      IF (DABS(PCOM).LT.1.0D-6*EPS) PCOM=0.0D0
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      DOUBLE PRECISION FUNCTION CMFX (PV)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C*                                                                     *
C*    THIS SUBROUTINE IS USED IN THE DRTMI SEARCH ROUTINE FOR CALCULAT-*
C* ING THE P-UPPER  AND P-LOWER CUMMULATIVE VALUES.  IT IMPLEMENTS THE *
C* PUPV ROUTINE TO CALCULATE THE P-UPPER VALUE GIVEN A P-LOWER VALUE.  *
C*                                                                     *
C*                                                                     *
C*                                                                     *
C*    REQUIRED SUBROUTINES:  PUPV, CUM                                 *
C*                                                                     *
C*                                                                     *
C* NOTE: CUM(P) CALCULATES THE CUMMULATIVE VALUE FOF P.                *
C*                                                                     *
C*                           S. D. HANSEN                              *
C*                      KANSAS STATE UNIVERSITY                        *
C*                         MANHATTAN, KANSAS                           *
C*                                                                     *
C*                                           DATE WRITTEN:  10/4/79.   *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PASS/ PMAX,XLZ,XRZ,EPS,PVPASS,ALPHA
      PVU=PUPV(PV)
      CMFX=CUM(PVU)-CUM(PV)-1+ALPHA
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      DOUBLE PRECISION FUNCTION DENS (XT)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE CALCULATES THE DENSITY VALUE AT XT FOR THE PRIOR OR *
C* THE POSTERIOR DISTRIBUTION.  FOR IDIS=1: PRIOR,  IDIS=2: POSTERIOR. *
C* SUBROUTINES REQUIRED: PSTERD, PRIORD.                               *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /DIS/ IDIS
      COMMON /GENCOM/ IGOUT
      X=XT
      GO TO (10,20),IDIS
10    DENS=PRIORD(X)
      RETURN
20    CALL PSTERD(X,ANS,IER)
      IF(IER.EQ.1) GO TO 30
      WRITE(IGOUT,600) IER
600   FORMAT(/T2,'***ERROR*** ERROR CODE IS ',I2/
     >T2,'SEE SUBROUTINE PSTERD FOR DETAILS'/)
30    DENS=ANS
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      DOUBLE PRECISION FUNCTION CUM (X)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE CALCULATES THE CUMULATIVE VALUE AT X FOR THE PRIOR  *
C* OR POSTERIOR DISTRIBUTION. FOR IDIS=1: PRIOR,   IDIS=2: POSTERIOR.  *
C* SUBROUTINES REQUIRED: PRCUMD, PSTCMD                                *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /DIS/ IDIS
      COMMON /GENCOM/ IGOUT
      XD=X
      GO TO (10,20),IDIS
10    CUM=PRCUMD(XD)
      RETURN
20    CALL PSTCMD(XD,ANS,IERR)
      IF(IERR.EQ.1) GO TO 30
      WRITE(IGOUT,600) IERR
600   FORMAT(/T2,'***ERROR*** ERROR CODE IS ',I2/
     >T2,'SEE SUBROUTINE PSTCMD FOR DETAILS'/)
30    CUM=ANS
      RETURN
      END
C+****************************** IMTRD *********************************
C
      SUBROUTINE IMTRD (X,F,FCT,XLI,XRI,EPS,IEND,IER)
C
C******************************* IMTRD *********************************
C*                                                                     *
C* THIS SUBROUTINE SOLVES A NONLINEAR EQUATION OF THE FORM FCT(X)=0    *
C* BY THE MUELLER'S ITERATION METHOD.                                  *
C* THIS PROGRAM IS TAKEN FROM THE IBM SSP (RTMI) AND MODIFIED TO WORK  *
C* IN DOUBLE PRECISION.                                                *
C*                                                                     *
C*    INPUT VARIABLES:                                                 *
C*        X    = RETURNED VALUE OF THE ZERO                            *
C*        F    = VALUE OF THE FUNCTION FCT AT THE ZERO                 *
C*        FCT  = EXTERNAL FUNCTION WHOSE ZERO IS TO BE FOUND           *
C*        XLI  = LEFT BOUNDARY OF THE X-AXIS TO BE SEARCHED FOR ROOT   *
C*        XRI  = RIGHT BOUNDARY OF X-AXIS TO BE SEARCHED FOR ROOT      *
C*        EPS  = ACCURACY OF DESIRED RESULT                            *
C*        IEND = MAXIMUM NUMBER OF ITERATIONS TO BE USED               *
C*        IER  = ERROR RETURN CODE; =0 IF DESIRED ACCURACY ACHIEVED    *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C
C     PREPARE ITERATION
      IER = 0
      XL = XLI
      XR = XRI
      X = XL
      TOL = X
      F = FCT(TOL)
      IF (F) 1,16,1
    1 FL = F
      X = XR
      TOL = X
      F = FCT(TOL)
      IF (F) 2,16,2
    2 FR = F
      IF (DSIGN(1.D0,FL)+DSIGN(1.D0,FR)) 25,3,25
C
C     BASIC ASSUMPTION FL*FR LESS THAN 0 IS SATISFIED.
C     GENERATE TOLERANCE FOR FUNCTION VALUES.
    3 I = 0
      TOLF = 100.*EPS
C
C
C     START ITERATION LOOP
    4 I = I+1
C
C     START BISECTION LOOP
      DO 13 K=1,IEND
      X = .5*(XL+XR)
      TOL = X
      F = FCT(TOL)
      IF (F) 5,16,5
    5 IF (DSIGN(1.D0,F)+DSIGN(1.D0,FR)) 7,6,7
C
C     INTERCHANGE XL AND XR IN ORDER TO GET THE SAME SIGN IN F AND FR
    6 TOL = XL
      XL = XR
      XR = TOL
      TOL = FL
      FL = FR
      FR = TOL
    7 TOL = F-FL
      A = F*TOL
      A = A+A
      IF (A-FR*(FR-FL)) 8,9,9
    8 IF (I-IEND) 17,17,9
    9 XR = X
      FR = F
C
C     TEST ON SATISFACTORY ACCURACY IN BISECTION LOOP
      TOL = EPS
      A = DABS(XR)
      IF (A-1.) 11,11,10
   10 TOL = TOL*A
   11 IF (DABS(XR-XL)-TOL) 12,12,13
   12 IF (DABS(FR-FL)-TOLF) 14,14,13
   13 CONTINUE
C     END OF BISECTION LOOP
C
C     NO CONVERGENCE AFTER IEND ITERATION STEPS FOLLOWED BY IEND
C     SUCCESSIVE STEPS OF BISECTION OR STEADILY INCREASING FUNCTION
C     VALUES AT RIGHT BOUNDS. ERROR RETURN.
      IER = 1
   14 IF (DABS(FR)-DABS(FL)) 16,16,15
   15 X = XL
      F = FL
   16 RETURN
C
C     COMPUTATION OF ITERATED X-VALUE BY INVERSE PARABOLIC INTERPOLATION
   17 A = FR-F
      DX = (X-XL)*FL*(1.+F*(A-TOL)/(A*(FR-FL)))/TOL
      XM = X
      FM = F
      X = XL-DX
      TOL = X
      F = FCT(TOL)
      IF (F) 18,16,18
C
C     TEST ON SATISFACTORY ACCURACY IN ITERATION LOOP
   18 TOL = EPS
      A = DABS(X)
      IF (A-1.) 20,20,19
   19 TOL = TOL*A
   20 IF (DABS(DX)-TOL) 21,21,22
   21 IF (DABS(F)-TOLF) 16,16,22
C
C     PREPARATION OF NEXT BISECTION LOOP
   22 IF (DSIGN(1.D0,F)+DSIGN(1.D0,FL)) 24,23,24
   23 XR = X
      FR = F
      GO TO 4
   24 XL = X
      FL = F
      XR = XM
      FR = FM
      GO TO 4
C     END OF ITERATION LOOP
C
C
C     ERROR RETURN IN CASE OF WRONG INPUT DATA
   25 IER = 2
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE MAXFN(F,XL,XR,XMAX,FMAX,NITER,EPS,IERR)
C
C***********************************************************************
C* THIS SUBROUTINE FINDS THE MAXIMUM OF A SPECIFIED FUNCTION,F, IN
C* AN INTERVAL (XL,XR) TO AN ABSOLUTE ACCURACY OF EPS BY THE METHOD
C* OF GOLDEN SECTION BISECTION.  THE MAXIUMUM VALUE IS AT 'XMAX'
C* AND IS DENOTED BY 'FMAX'.
C* A RETURN CODE WITH IERR=0 INDICATES THE DESIRED ACCURACY WAS OBTAINED
C* WHILE A RETURN OF IERR=1 INDICATES THE DESIRED ACCURACY WAS NOT
C* OBTAINED WITHIN NITER ITERATIONS.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C INITIALIZE THE BISECTION
      IERR=0
      TAU=0.6180339885
      TAUM=1.E0-TAU
      X2=XL+0.6180339885*(XR-XL)
      X1=XL+XR-X2
      F2=F(X2)
      F1=F(X1)
      A=XL
      B=XR
C BEGIN BISECTION ITERATION
      DO 10 I=1,NITER
      IF (F2.GT.F1) GO TO 20
      B=X2
      X2=X1
      F2=F1
      X1=A+TAUM*(B-A)
      F1=F(X1)
      GO TO 30
   20 A=X1
      X1=X2
      F1=F2
      X2=B-TAUM*(B-A)
      F2=F(X2)
   30 ERROR=DABS((B-A)/B)
      IF(ERROR-EPS) 40,40,10
C
CTERMINATE DUE TO ITERATION COUNT
   10 CONTINUE
      IERR=1
C
C AVERAGE FINAL INTERVAL TO OBTAIN XMAX
   40 XMAX=0.5E0*(B+A)
      FMAX=F(XMAX)
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE TABLED(TPTS,NPTS,IMETH,IDIST,IOUT,OUTLN,PCMP)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C*                                                                     *
C*    THIS ROUTINE CALCULATES AND TABULATES THE DENSITY AND CUMULATIVE *
C* VALUES OF THE  IPRIR DISTRIBUTION. THE USER MAY  SPECIFY EITHER THE *
C* PRIOR OR  THE POSTERIOR  DISTRIBUTION FOR  TABULATION. THE  ROUTINE *
C* CREATES A TABLE BY ANY ONE  OF FIVE DIFFERENT METHODS OF SPECIFYING *
C* THE RANGE OF 'P' OVER WHICH VALUES ARE TO BE TABULATED.             *
C*                                                                     *
C*                                                                     *
C* REQUIRED ROUTINES:  RANGED, PRIORD, PRCMUD, PSTERD, PSTCMD,         *
C*                     PRCNTD,PSCNTD.                                  *
C*                                                                     *
C*                                                                     *
C* PARAMETERS:                                                         *
C*                                                                     *
C*      TPTS: DATA VECTOR FOR PASSING SPECIFIC OR ENDPOINTS OF 'P'     *
C*           OR PERCENTILES.                                           *
C*                                                                     *
C*      NPTS: NUMBER OF POINTS TO BE TABULATED. MAXIMUM=40.            *
C*                                                                     *
C*      IDIST: INDICATES WHETHER THE PRIOR OR THE POSTERIOR            *
C*             DISTRIBUTION  IS  TO  BE  TABULATED.   IDIST=1:  PRIOR. *
C*             IDIST=2: POSTERIOR.                                     *
C*                                                                     *
C*     PCMP: ID OF COMPONENT FOR WHICH POSTERIOR ANALYSIS IS PERFORMED *
C*                                                                     *
C*       EXPLANATION OF METHODS SPECIFIED BY IMETH PARAMETER:          *
C*                                                                     *
C*         =1:  USER SPECIFIES SPECIFIC 'P' VALUES IN VECTOR PTS.      *
C*              NPTS MUST BE SET TO THE NUMBER OF 'P' POINTS PASSED TO *
C*              THE ROUTINE IN PTS.                                    *
C*                                                                     *
C*         =2:  USER SPECIFIES LOWER AND UPPER LIMITS OF 'P' IN        *
C*              PTS(1) AND  PTS(2) RESPECTIVELY. NPTS  MUST BE  SET TO *
C*              THE NUMBER OF  EQUIDISTANT 'P' POINTS  (INCLUDING END- *
C*              POINTS) THAT ARE TO BE  TABULATED OVER THE GIVEN RANGE *
C*              OF 'P'.                                                *
C*                                                                     *
C*         =3:  USER SPECIFIES LOWER AND UPPER PERCENTILE POINTS IN    *
C*              PTS(1) AND PTS(2) RESPECTIVELY FOR WHICH CORRESPONDING *
C*              'P' POINTS  ARE OBTAINED.  NPTS  SHOULD BE SET  TO THE *
C*              NUMBER OF EQUIDISTANT 'P' POINTS (INCLUDING ENDPOINTS) *
C*              THAT ARE TO BE TABULATED OVER THE RANGE OF THE CORRES- *
C*              PONDING PERCENTILE 'P' POINTS.                         *
C*                                                                     *
C*         =4:  USER SPECIFIES SPECIFIC INDIVIDUAL PERCENTILE LEVELS   *
C*              IN PTS VECTOR AT WHICH 'P' VALUES ARE TO BE TABULATED. *
C*              NPTS SHOULD BE  SET TO THE NUMBER  OF GIVEN PERCENTILE *
C*              POINTS.                                                *
C*                                                                     *
C*         =5:  USER SPECIFIC LOWER AND UPPER PERCENTILE POINTS IN     *
C*              PTS(1) AND PTS(2) RESPECTIVELY,  OVER WHICH PERCENTILE *
C*              POINTS ARE TO BE TABULATED.  NPTS SHOULD BE SET TO THE *
C*              NUMBER  OF  EQUIDISTANT PERCENTILE  POINTS  (INCLUDING *
C*              ENDPOINTS) THAT ARE TO BE TABULATED.                   *
C*                                                                     *
C* COMMON BLOCK /PRIR/ VARIABLES:                                      *
C*                                                                     *
C*    IPRIR = DISTRIBUTION INDEX: =-1 BETA, =-2 LOGBETA,               *
C*            =-3 LOGNORMAL-1, =-4 LOGNORMAL-2,  =-5 LOGGAMMA-1,       *
C*            =-6 LOG-GAMMA-2, =-7 LOGLOGNORMAL.                       *
C*                                                                     *
C*    PARM  = DISTRIBUTION PARAMETER VECTOR: PARM(1)=ALPHA, PARM(2)=   *
C*            BETA,  PARM(3)=FIRST  RANGE   PARAMETER,  PARM(4)=SECOND *
C*            RANGE PARAMETER.                                         *
C*                                                                     *
C*    RNG   = RNG(1): LOWER LIMIT OF 'P'. RNG(2): UPPER LIMIT OF 'P'.  *
C*                                                                     *
C*                                                                     *
C*    COMMON BLOCK  /RTPASS/:  USED  IN PASSING  PARAMETERS TO  PRCNT, *
C*    PSCNT                                                            *
C*                            ROUTINES. USER NEED NOT MANIPULATE.      *
C*                                                                     *
C*                           S. D. HANSEN                              *
C*                      KANSAS STATE UNIVERSITY                        *
C*                         MANHATTAN, KANSAS                           *
C*                                                                     *
C*                                           DATE WRITTEN:  5/80.      *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION    PTS(40),OUTLN(40,3),TPTS(40)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      COMMON /RTPASS/  XLI,XRI,EPS,IEND,IERR
      COMMON /DATAD/K,N
      COMMON /GENCOM/ IGOUT
      EPS=1.0D-6
      IEND=250
      DO 987 I=1,40
      OUTLN(I,1)=0.0D0
      OUTLN(I,2)=0.0D0
      OUTLN(I,3)=0.0D0
 987  CONTINUE
      KTP=2
      IF (IMETH.EQ.1.OR.IMETH.EQ.4) KTP=NPTS
      DO 700 I=1,KTP
      PTS(I)=TPTS(I)
 700  CONTINUE
      IF (IMETH.LE.2) GOTO 800
      CALL RANGED
      XLI=RNG(1)
      XRI=RNG(2)
 800  GOTO (100,200,300,400,500),IMETH
 100  IF (IDIST.EQ.2) GOTO 150
      DO 110 I=1,NPTS
      OUTLN(I,1)=PTS(I)
      OUTLN(I,2)=PRIORD(OUTLN(I,1))
      OUTLN(I,3)=PRCUMD(OUTLN(I,1))
 110  CONTINUE
      GOTO 600
 150  DO 160 I=1,NPTS
      OUTLN(I,1)=PTS(I)
      CALL PSTERD(OUTLN(I,1),ANS,IERRQ)
      IF (IERRQ.NE.1) WRITE(IGOUT,1100)
      OUTLN(I,2)=ANS
      CALL PSTCMD(OUTLN(I,1),ANS,IERRQ)
      IF (IERRQ.NE.1) WRITE(IGOUT,1100)
      OUTLN(I,3)=ANS
 160  CONTINUE
      GOTO 600
 200  DELTA=(PTS(2)-PTS(1))/(NPTS-1)
      OUTLN(1,1)=PTS(1)
      IF (IDIST.EQ.2) GOTO 250
      OUTLN(1,2)=PRIORD(OUTLN(1,1))
      OUTLN(1,3)=PRCUMD(OUTLN(1,1))
      DO 210 I=2,NPTS
      OUTLN(I,1)=OUTLN(I-1,1)+DELTA
      OUTLN(I,2)=PRIORD(OUTLN(I,1))
      OUTLN(I,3)=PRCUMD(OUTLN(I,1))
 210  CONTINUE
      GOTO 600
 250  CALL PSTERD(OUTLN(1,1),ANS,IERRQ)
      IF (IERRQ.NE.1) WRITE(IGOUT,1100)
      OUTLN(1,2)=ANS
      CALL PSTCMD (OUTLN(1,1),ANS,IERRQ)
      IF (IERRQ.NE.1) WRITE(IGOUT,1100)
      OUTLN(1,3)=ANS
      DO 260 I=2,NPTS
      OUTLN(I,1)=OUTLN(I-1,1)+DELTA
      CALL PSTERD(OUTLN(I,1),ANS,IERRQ)
      IF (IERRQ.NE.1) WRITE(IGOUT,1100)
      OUTLN(I,2)=ANS
      CALL PSTCMD(OUTLN(I,1),ANS,IERRQ)
      IF (IERRQ.NE.1) WRITE(IGOUT,1100)
      OUTLN(I,3)=ANS
 260  CONTINUE
      GOTO 600
 300  IF (IDIST.EQ.2) GOTO 350
      PTS(1)=PRCNTD(PTS(1))
      PTS(2)=PRCNTD(PTS(2))
      GOTO 200
 350  PTS(1)=PSCNTD(PTS(1))
      PTS(2)=PSCNTD(PTS(2))
      GOTO 200
 400  IF (IDIST.EQ.2) GOTO 450
      DO 410 I=1,NPTS
      PTS(I)=PRCNTD(PTS(I))
 410  CONTINUE
      GOTO 100
 450  DO 460 I=1,NPTS
      PTS(I)=PSCNTD(PTS(I))
 460  CONTINUE
      GOTO 100
 500  DELTA=(PTS(2)-PTS(1))/(NPTS-1)
      DO 510 I=2,NPTS
      PTS(I)=PTS(I-1)+DELTA
 510  CONTINUE
      GOTO 400
 600  IF (IOUT.EQ.0) RETURN
      WRITE(IOUT,1234)
 1234 FORMAT('1',7('**********'))
 1000 FORMAT(' ',7('**********'))
      WRITE(IOUT,1001)
 1001 FORMAT(' *',T71,'*')
      II = -IPRIR
      GO TO (1,2,3,4,5,6),II
    1 CONTINUE
      WRITE (IOUT,2001)
 2001 FORMAT (' *',T34,'BETA',T71,'*')
      GO TO 7
    2 CONTINUE
      WRITE (IOUT,2002)
 2002 FORMAT (' *',T32,'LOG BETA',T71,'*')
      GO TO 7
    3 CONTINUE
      WRITE (IOUT,2003)
 2003 FORMAT (' *',T31,'LOGNORMAL 1',T71,'*')
      GO TO 7
    4 CONTINUE
      WRITE (IOUT,2004)
 2004 FORMAT (' *',T31,'LOGNORMAL 2',T71,'*')
      GO TO 7
    5 CONTINUE
      WRITE (IOUT,2005)
 2005 FORMAT (' *',T31,'LOGGAMMA 1',T71,'*')
      GO TO 7
    6 CONTINUE
      WRITE (IOUT,2006)
 2006 FORMAT (' *',T31,'LOGGAMMA 2',T71,'*')
    7 CONTINUE
      WRITE(IOUT,1001)
      WRITE(IOUT,1000)
      WRITE(IOUT,1001)
      WRITE(IOUT,1006)
 1006 FORMAT(' *',T28,'FAILURE ON DEMAND',T71,'*')
      IF (IDIST.EQ.2) GOTO 610
      WRITE(IOUT,1002)
 1002 FORMAT(' *',T27,'PRIOR  DISTRIBUTION',T71,'*')
      GOTO 620
 610  WRITE(IOUT,1010)
 1010 FORMAT(' *',T25,'POSTERIOR DISTRIBUTION',T71,'*')
 620  WRITE(IOUT,1001)
      WRITE(IOUT,1007) PARM(1),PARM(2)
 1007 FORMAT(' *',T15,'ALPHA= ',1PG12.5,T41,'BETA= ',G12.5,T71,'*')
      WRITE(IOUT,1008) PARM(3),PARM(4)
 1008 FORMAT(' *',T15,'A(OR THETA)= ',1PG12.5,T44,'B= ',G12.5,T71,'*')
      IF (IDIST.EQ.2) WRITE(IOUT,1011) PCMP, K,N
 1011 FORMAT(' *',T15,'FOR COMPONENT:  ',A8,                  T71,'*'/
     >       ' *',T15,'FAILURES: ',I8,T37,'ATTEMPTS: ',I8,T71,'*')
      WRITE(IOUT,1001)
      WRITE(IOUT,1000)
      WRITE(IOUT,1001)
      WRITE(IOUT,1003)
 1003 FORMAT(' *',T13,'P',T36,'DENSITY',T58,'CUMULATIVE',T71,'*',/,
     >       ' *',T13,'-',T36,'-------',T58,'----------',T71,'*')
      DO 650 I=1,NPTS
      WRITE(IOUT,1005) OUTLN(I,1),OUTLN(I,2),OUTLN(I,3)
 1005 FORMAT(' *',T8,1PG11.4,T27,0PD16.8,T58,F10.8,T71,'*')
 650  CONTINUE
      WRITE(IOUT,1001)
      WRITE(IOUT,1000)
 1100 FORMAT (' INACCURATE RESULTS FROM HINTD ROUTINE IN TABLED')
      RETURN
      END
C+**********************************************************************
C*
      DOUBLE PRECISION FUNCTION PRCNTD (X)
C*
C***********************************************************************
C*                                                                     *
C*                               PRCNTD                                *
C*                                                                     *
C***********************************************************************
C*  THIS FUNCTION RETURNS THE CORRESPONDING 'P' VALUE AT THE           *
C*  X*(100) PERCENTILE FOR THE PRIOR DISTRIBUTION. THE FUNCTION        *
C*  USES THE DRTMI ROUTINE TO FIND THE ROOT OF F(P)=CUM(P)-X=0         *
C*  /SUBPRM/ COMMON BLOCK IS USED TO PASS PARAMETERS TO THE FUNCTION   *
C*  PRFCTD. PARAMETERS IN COMMON BLOCK /RTPASS/ ARE SET BY THE CALLING *
C*  ROUTINE AND ARE EXPLAINED IN THE DRTMI DOCUMENTATION.              *
C*                                                                     *
C*                                                                     *
C*  WRITTEN BY: S. D. HANSEN         5/80.                             *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /RTPASS/  XLI,XRI,EPS,IEND,IERR
      COMMON /SUBPRM/ ALPHA
      COMMON /GENCOM/ IGOUT
      EXTERNAL PRFCTD
      ALPHA=X
      CALL DRTMI (ANS,F,PRFCTD,XLI,XRI,EPS,IEND,IERR)
      IF (IERR.NE.0) WRITE(IGOUT,100) IERR
 100  FORMAT(' ERROR IN PRCNTD ROUTINE, IERR= ',I4)
      PRCNTD=ANS
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION PRFCTD(P)
C
C***********************************************************************
C*                                                                     *
C*  THIS FUNCTION EVALUATES F(P)=CUM(P)-ALPHA FOR THE PRIOR            *
C*  DISTRIBUTION.                                                      *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /SUBPRM/ ALPHA
      PRFCTD=PRCUMD(P)-ALPHA
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION PSCNTD (X)
C
C***********************************************************************
C*                                                                     *
C*                               PSCNTD                                *
C*                                                                     *
C***********************************************************************
C*  THIS FUNCTION RETURNS THE CORRESPONDING 'P' VALUE AT THE           *
C*  X*(100) PERCENTILE FOR THE POSTERIOR DISTRIBUTION. THE FUNCTION    *
C*  USES THE DRTMI ROUTINE TO FIND THE ROOT OF F(P)=CUM(P)-X=0         *
C*  /SUBPRM/ COMMON BLOCK IS USED TO PASS PARAMETERS TO THE FUNCTION   *
C*  PSFCTD. PARAMETERS IN COMMON BLOCK /RTPASS/ ARE SET BY THE CALLING *
C*  ROUTINE AND ARE EXPLAINED IN THE DRTMI DOCUMENTATION.              *
C*                                                                     *
C*                                                                     *
C*  WRITTEN BY: S. D. HANSEN         5/80.                             *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /RTPASS/  XLI,XRI,EPS,IEND,IERR
      COMMON /SUBPRM/ ALPHA
      COMMON /GENCOM/ IGOUT
      EXTERNAL PSFCTD
      ALPHA=X
      CALL DRTMI (ANS,F,PSFCTD,XLI,XRI,EPS,IEND,IERR)
      IF (IERR.NE.0) WRITE(IGOUT,100) IERR
 100  FORMAT(' ERROR IN PSCNTD ROUTINE, IERR= ',I4)
      PSCNTD=ANS
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION PSFCTD(P)
C
C***********************************************************************
C*                                                                     *
C*  THIS FUNCTION EVALUATES F(P)=CUM(P)-ALPHA FOR THE POSTERIOR        *
C*  DISTRIBUTION.                                                      *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /SUBPRM/ ALPHA
      COMMON /GENCOM/ IGOUT
      CALL PSTCMD(P,ANS,IERR)
      PSFCTD=ANS-ALPHA
      IF (IERR.NE.1) WRITE (IGOUT,5)
 5    FORMAT (' INACCURATE RESULTS IN PSFCTD')
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE PLOT(NO,A,N,M,NL,NS,CDT,XAX,YAX,IXOUT,AXMX,AXMN,IOUT)
C*                                                                     *
C***********************************************************************
C
C        PURPOSE
C           PLOT SEVERAL CROSS-VARIABLES VERSUS A BASE VARIABLE
C
C        DESCRIPTION OF PARAMETERS
C           NO - CHART NUMBER (3 DIGITS MAXIMUM)
C           A  - MATRIX OF DATA TO BE PLOTTED. FIRST COLUMN REPRESENTS
C                BASE VARIABLE AND SUCCESSIVE COLUMNS ARE THE CROSS-
C                VARIABLES (MAXIMUM IS 9).
C           N  - NUMBER OF ROWS IN MATRIX A
C           M  - NUMBER OF COLUMNS IN MATRIX A (EQUAL TO THE TOTAL
C                NUMBER OF VARIABLES). MAXIMUM IS 10.
C           NL - NUMBER OF LINES IN THE PLOT. IF 0 IS SPECIFIED, 51
C                LINES ARE USED.
C           NS - CODE FOR SORTING THE BASE VARIABLE DATA IN ASCENDING
C                ORDER
C                  0  SORTING IS NOT NECESSARY (ALREADY IN ASCENDING
C                     ORDER).
C                  1  SORTING IS NECESSARY.
C           CDT- CHART DESCRIPTION (56 CHARACTERS,DIMENSION 7)
C           XAX- BASE VARIABLE-AXIS DESCRIPTION (6 CHARACTERS)
C           YAX- CROSS VARIABLE-AXIS DESCRIPTION (6 CHARACTERS)
C           IXOUT - MARKS ON BASE VARIABLE-AXIS WILL BE PRINTED
C                   EVERY IXOUT DATA POINTS
C                   IXOUT=0 PRINT MARK ON EVERY DATA POINT
C           AXMX  - MAXIMUM VALUE ON THE CROSS VARIABLE AXIS
C           AXMN  - MINIMUM VALUE ON THE CROSS VARIABLE AXIS
C                   IF AXMX & AXMN = 0.0D00,MAX.& MIN. VALUES
C                   IN THE MATRIX A WILL BE USED
C           IOUT  - NO.OF PRINTING UNIT
C
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C           NONE
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION YPR(11),A(40,2)
      CHARACTER*56 CDT
      CHARACTER*4 YAX, XAX*6
      CHARACTER*1 BLANK,ANG(9),OUT(101)
      DATA BLANK/' '/,ANG/'1','2','3','4','5','6','7','8','9'/
C
      IF(IXOUT.EQ.0) IXOUT=1
      NLL=NL
C
C        SORT BASE VARIABLE DATA IN ASCENDING ORDER
C
      IF(NS.GT.0) THEN
      DO 15 I=1,N-1
         DO 14 J=I+1,N
            IF(A(I,1).GT.A(J,1)) THEN
            DO 12 K=1,M
            F=A(I,K)
            A(I,K)=A(J,K)
   12       A(J,K)=F
            ENDIF
   14       CONTINUE
   15    CONTINUE
      ENDIF
C
C        TEST NLL
C
      IF(NLL.EQ.0) NLL=51
C
C        PRINT TITLE
C
      WRITE(IOUT,215)NO,CDT
  215 FORMAT('1',37X,'CHART ',I3,4X,A56/)
C
C        FIND SCALE FOR BASE VARIABLE
C
      XSCAL=(A(N,1)-A(1,1))/(NLL-1)
C
C        FIND SCALE FOR CROSS-VARIABLES
C
      IF(AXMX.GT.AXMN) THEN
        YMIN=AXMN
        YMAX=AXMX
      ELSE
        YMIN=A(1,2)
        YMAX=YMIN
        DO 20 J=1,N
        DO 20 K=2,M
          IF(A(J,K).LT.YMIN) YMIN=A(J,K)
          IF(A(J,K).GT.YMAX) YMAX=A(J,K)
   20   CONTINUE
      ENDIF
      YSCAL=(YMAX-YMIN)/100.0D00
C
C        PRINT CROSS-VARIABLES NUMBERS
C
      YPR(1)=YMIN
      DO 30 KN=1,9
   30 YPR(KN+1)=YPR(KN)+YSCAL*10.0D00
      YPR(11)=YMAX
      WRITE(IOUT,230)(YPR(IP),IP=1,11)
  230 FORMAT(' ',9X,11F10.4)
      WRITE(IOUT,231)YAX
  231 FORMAT(' ',16X,'+',10('---------+'),5X,A6)
C
C        FIND BASE VARIABLE PRINT POSITION
C
      XB = A(1,1)
      L = 1
C
C        LOOP OVER (NLL) OUTPUT LINES
C
      DO 90 I=1,NLL
         F = I-1
         XPR = XB+F*XSCAL
         DO 40 IX=1,101
   40       OUT(IX)=BLANK
   50    IF(L.GT.N) GOTO 60
         IF(A(L,1).LT.XPR+XSCAL) THEN
         DO 55 K=2,M    ! LOOP OVER CROSS-VARIABLES
            JP=((A(L,K)-YMIN)/YSCAL)+1.0D00
   55       OUT(JP)=ANG(K-1)
         L = L+1
         ENDIF
   60    IF((I-1).EQ.(I-1)/IXOUT*IXOUT) THEN
           WRITE(IOUT,260) XPR,(OUT(IZ),IZ=1,101)
  260      FORMAT(' ',F11.4,4X,'+',101A1)
         ELSE
           WRITE(IOUT,261) (OUT(IZ),IZ=1,101)
  261      FORMAT(' ',15X,'|',101A1)
         ENDIF
   90 CONTINUE
      WRITE(IOUT,290) XAX
  290 FORMAT(/' ',15X,A6/)
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE MAXLD (DMAXL,DLMAXL,P,IMODL,IERR,N)
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /FDDATA/ FI(500),XNI(500)
      COMMON /DATAD/ K,NN
C
C
C     INITIALIZE
C
      DLMAXL = 0.D0
      DMAXL = 0.D0
C
C
C
C     CHECK ON MODEL (HOMOGENEOUS OR COMPOUND).
C
C
      IF (IMODL .EQ. 2) GO TO 50
C
C
C     COMPUTE MAXIMUM LIKELIHOOD FOR EACH DATA POINT
C
C
C     DO MAXIMUM LIKELIHOOD COMPUTATIONS FOR THE COMPOUND MODEL
C
C
      DO 1 I=1,N
      K = FI(I)
      NN = XNI(I)
      CALL MRGCLD (ANS,IERR)
      DLMAXL = DLMAXL + DLG(ANS)
    1 CONTINUE
C
C
C
C     COMPUTE  LIKELIHOOD FORM LOG LIKELIHOOD.
C
C
   20 CONTINUE
      DMAXL = DXP(DLMAXL)
C
      RETURN
C
C
C     COMPUTE MAXIMUM LIKELIHOOD FOR THE HOMOGENEOUS MODEL
C
   50 CONTINUE
C
C
      DO 60 I =1,N
      K = FI(I)
      NN = XNI(I)
      DLMAXL = DLMAXL + DLGAMA(NN+1.D0) - DLGAMA(NN-K+1.D0)
     > - DLGAMA(K+1.D0)  + K*DLG(P) + (NN-K)*DLG(1.D0-P)
   60 CONTINUE
      GO TO 20
C
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE PSTCPD (PCT,EPS)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE CALCULATES PROPERTIES OF THE POSTERIOR DISTRI-      *
C* BUTION FOR EACH COMPONENT AND COMPARES THE RESULTS TO THE CORRE-    *
C* SPONDING HOMOGENEOUS ANALYSIS. THIS SUBROUTINE PRODUCES A TABLE     *
C* GIVING THE MEAN, MEDIAN, VARIANCE, SKEWNESS, AND UPPER AND LOWER    *
C* PERCENTILES FOR THE POSTERIOR AS WELL AS THE ESTIMATED FAILURE RATE *
C* AND ITS CONFIDENCE INTERVAL FOR THE HOMOGENEOUS MODEL.              *
C*                                                                     *
C* INPUT VARIABLES:                                                    *
C*    PCT = PERCENTILE VALUE FOR TAIL AREA IN CALCULATION OF CONFI-    *
C*          DENCE AND PROBABILITY INTERVALS                            *
C*    EPS = RELATIVE ACCURACY OF THE PROBABILITY AND CONFIDENCE LIMITS *
C*                                                                     *
C* FAILURE DATA IS PASSED THROUGH COMMON BLOCK /DTA/                   *
C* SUBPROGRAMS NEEDED: FBETA, MDBETA                                   *
C*                                                                     *
C***********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION PW(10)
      DOUBLE PRECISION LAM(100),LAML,LAMU,MEAN(100),MEDIAN
      CHARACTER*8 CID(500)
      COMMON /DTA/ CID,NF(500),NT(500),N,IPRT,NITER
      COMMON /FBET/ARG1,ARG2,PCTL,IPRTT
      COMMON /DATAD/ KK,NN
      COMMON /DIST/ VMARG,IMRGCL
      COMMON /RTPASS/ XLI,XRI,EPSRT,IEND,IERR
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      EXTERNAL FBETA
C
      IPRTT=IPRT
C     PRINT TABLE HEADINGS
      PCCT = 1.0-2.0*PCT
      CALL RANGED
      XRI=RNG(2)
      XLI=RNG(1)
      EPSRT=EPS
      IEND=NITER
      WRITE (IPRT,8)
    8 FORMAT (/////' ANALYSIS OF EACH COMPONENT WITH A HOMOGENEOUS',
     >' MODEL AND WITH A COMPOUND MODEL '/)
      WRITE (IPRT,9)
    9 FORMAT (' ---COMPONENT DATA----  ',8('-'),'HOMOGENEOUS MODEL',
     >9('-'),2X,22('-'),'COMPOUND MODEL (POSTERIOR)',22('-'))
      WRITE (IPRT,10) PCCT,PCCT
   10 FORMAT (4X,'ID',4X,'NO.    NO.',6X,'   P  ',4X,F5.3,
     >' CONFIDENCE INT.',6X,'MEAN',7X,'MEDIAN',5X,'VARIANCE',4X,
     >'SKEWNESS',3X,F5.3,' PROBABILITY INT.')
      WRITE (IPRT,11)
   11 FORMAT (9X,'FAIL',3X,'TRIES',6X,'(F/T)',6X,'LOWER',7X,'UPPER',7X,
     >'   P  ',6X,'  P   ',30X,'LOWER',7X,'UPPER'/1X,129('-'))
C
C     BEGIN POSTERIOR ANALYSIS OF EACH COMPONENT
      DO 1 I=1,N
      T = NT(I)
      F = NF(I)
      KK=NF(I)
      NN=NT(I)
C     CALCULATE MOMENTS ABOUT ZERO
      CALL PSMOMD (PW,IERR)
      IF (IERR.NE.0) WRITE (IPRT,444)
 444  FORMAT(' ***WARNING: INACCURATE RESULTS IN POSTERIOR MOMENTS IN',
     >' ROUTINE PSTCPD. RETURN CODE=',I6)
      ZM1 = PW(7)
      ZM2=PW(8)
      ZM3=PW(9)
      VAR = PW(2)
      CM3 = PW(3)
      SKW = PW(5)
      MEAN(I) = ZM1
C
C     CALCULATE POSTERIOR PERCENTILES AND MEDIAN
      IMRGCL=0
      RTLOW = PSCNTD(PCT)
      IF (IERR.NE.0) WRITE (IPRT,445)
 445  FORMAT(' *** WARNING: POSSIBLE ERROR IN CALCULATION OF THE',
     >' UPPER CONFIDENCE LIMIT IN ROUTINE PSTCPD')
      RTUP = PSCNTD(PCCT)
      IF (IERR.NE.0) WRITE (IPRT,446)
 446  FORMAT(' *** WARNING: POSSIBLE ERROR IN CALCULATION OF THE',
     >' LOWER CONFIDENCE LIMIT IN ROUTINE PSTCPD')
      MEDIAN = PSCNTD(.50D0)
      IF (IERR.NE.0) WRITE (IPRT,447)
 447  FORMAT(' *** WARNING: POSSIBLE ERROR IN CALCULATION OF THE',
     >' MEDIAN VALUE IN ROUTINE PSTCPD')
C
C CALCULATE HOMOGENEOUS MEAN AND PERCENTILES FOR EACH COMPONENT
      LAM(I)=F/T
C FIND UPPER SEARCH LIMIT FROM TCHBYCHEFF'S INEQUALITY
      IF(NF(I).EQ.0) GO TO 30
      IF (NF(I).EQ.NT(I)) GO TO 40
      PCTL=DSQRT(F)
      UPP=1.D0/DSQRT(PCT)
      IF(PCTL.GT.UPP) UPP=PCTL
      UPP=(F+PCTL*UPP)/T
      IF(UPP.GT.1.D0) UPP=1.D0
C  CALCULATE LOWER CONFIDENCE LIMIT
      PCTL=PCT
      ARG1=F
      ARG2=T-F+1.D0
      CALL DRTMI(LAML,FNVAL,FBETA,0.0D0,UPP,EPS,NITER,IER)
      IF(IER.EQ.0) GO TO 20
      WRITE(IPRT,31)
   31 FORMAT(' *** WARNING: POSSIBLE ERROR IN THE CALCULATION OF THE',
     >' LOWER HOMOGENEOUS CONFIDENCE LIMIT IN SUBROUTINE PSTCPD')
C  CALCULATE UPPER CONFIDENCE LIMIT
   20 ARG1=F+1.D0
      ARG2=T-F
      PCTL=1.D0-PCT
      CALL DRTMI(LAMU,FNVAL,FBETA,0.0D0,UPP,EPS,NITER,IERR)
      IF(IERR.EQ.0) GO TO 1
      WRITE(IPRT,32)
   32 FORMAT(' *** WARNING: POSSIBLE ERROR IN THE CALULATION OF THE',
     >' UPPER HOMOGENEOUS CONFIDENCE LIMIT IN SUBROUTINE PSTCPD')
      GO TO 1
C SPECIAL CASE FOR F=0
   30 LAML=0.0
      LAMU=1.D0-(PCT)**(1.D0/T)
      GO TO 1
C SPECIAL CASE FOR F = T
   40 LAMU=1.D0
      LAML=(1.D0-PCT)**(1.D0/T)
C
C     PRINT OUT VALUES FOR EACH COMPONENT
    1 WRITE (IPRT,12) CID(I),NF(I),NT(I),LAM(I),LAML,LAMU,ZM1,MEDIAN,
     >VAR,SKW,RTLOW,RTUP
   12 FORMAT (' ',A8,I3,I8,2X,9(1X,G11.4))
      WRITE (IPRT,13)
   13 FORMAT (' ',129('-'))
C
C  CALCULATE MINIMUM AND MAXIMUM FAILURE RATES FOR HOMO. NODEL
C  (HRTMAX & HRTMIN) AND FOR COMPOUND MODEL (CRTMAX & CRTMIN)
      HRTMAX = LAM(1)
      HRTMIN = LAM(1)
      CRTMAX = MEAN(1)
      CRTMIN = CRTMAX
      IHMAX = 1
      IHMIN = 1
      ICMAX = 1
      ICMIN = 1
C      SEARCH FOR MIN AND MAX AMOUNG REST OF THE COMPONENTS
      DO 5 I=1,N
      IF (HRTMAX.GE.LAM(I)) GO TO 2
      HRTMAX = LAM(I)
      IHMAX = I
    2 IF (HRTMIN.LE.LAM(I)) GO TO 3
      HRTMIN = LAM(I)
      IHMIN = I
    3 IF (CRTMAX.GE.MEAN(I)) GO TO 4
      CRTMAX = MEAN(I)
      ICMAX = I
    4 IF (CRTMIN.LE.MEAN(I)) GO TO 5
      CRTMIN = MEAN(I)
      ICMIN = I
    5 CONTINUE
C     PRINT OUT MIN AND MAX VALUES OF FAILURE RATES
      WRITE (IPRT,6)
    6 FORMAT (/' MINIMUM AND MAXIMUM FAILURE PROBABILITIES:',T53,
     > 'MINIMUM  P --- COMP.   ID',T97,'MAXIMUM  P --- COMP.   ID')
      WRITE (IPRT,7) HRTMIN,IHMIN,CID(IHMIN),HRTMAX,IHMAX,CID(IHMAX),
     > CRTMIN,ICMIN,CID(ICMIN),CRTMAX,ICMAX,CID(ICMAX)
    7 FORMAT (4X,'HOMOGENEOUS MODEL:',15X,2(15X,G11.4,I6,4X,A8)/4X,
     >'COMPOUND MODEL (POSTERIOR MEAN):',2X,2(14X,G11.4,I6,4X,A8,1X))
      RETURN
C
      END
C+**********************************************************************
C
      SUBROUTINE PSMOMD (DMOM,IERR)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION DMOM(10),IRV(5)
      COMMON /DATAD/ KK,NN
      IERR=0
      KTMP=KK
      NTMP=NN
      PN=1.0D0
      PK=1.0D0
      CALL MRGCLD (CONS,IR)
      IRV(5)=IR
 75   DO 200  I=1,4
      KK=KTMP+I
      NN=NTMP+I
      PK=PK*KK
      PN=PN*NN
      CALL MRGCLD (RES,IR)
      IRV(I)=IR
      DMOM(I+6)=(PK/PN)*(RES/CONS)
 200  CONTINUE
      DMOM(1)=0.0D0
      DMOM(2)=DMOM(8)-DMOM(7)**2
      DMOM(3)=DMOM(9)-3*DMOM(8)*DMOM(7)+2*DMOM(7)**3
      DMOM(4)=DMOM(10)-4*DMOM(9)*DMOM(7)+6*DMOM(8)*DMOM(7)**2 -
     >  3*DMOM(7)**4
      DMOM(5)=DMOM(3)/DMOM(2)**1.5
      DMOM(6)=DMOM(4)/DMOM(2)**2
      DO 300 I=1,5
      J=6-I
      IF (IRV(J).NE.1) IERR=J
 300  CONTINUE
      IF (IERR.EQ.5) IERR=1
      KK=KTMP
      NN=NTMP
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE QNC7(FUN,A,B,ERR,ANS,IERR)
C
C***********************************************************************
C****
C**** PURPOSE
C****    QNC7 INTEGRATES DOUBLE FUNCTIONS OF ONE VARIABLE OVER FINITE
C****    INTERVALS, USING AN ADAPTIVE 7-POINT NEWTON-COTES ALGORITHM.
C****    QNC7 IS EFFICIENT OVER A WIDE RANGE OF ACCURACIES, BUT QNC3
C****    MAY BE MORE EFFICIENT ON DIFFICULT LOW ACCURACY PROBLEMS,
C****    AND GAUS8 MAY BE MORE EFFICIENT ON HIGH ACCURACY PROBLEMS
C****    (ERR LESS THAN 1.0D-08, SAT) OR ON PROBLEMS INVOLVING VERY
C**** SMOOTH FUNCTIONS.
C****
C**** USAGE
C****    CALL QNC7(FUN,A,B,ERR,ANS,IERR)
C****
C****    FUN  - NAME OF EXTERNAL FUNCTION TO BE INTEGRATED.  THIS NAME
C****           MUST BE IN AN EXTERNAL STATEMENT IN THE CALLING PROGRAM.
C****           FUN MUST BE A FUNCTION OF ONE DOUBLE ARGUMENT (THE
C****           VARIABLE OF INTEGRATION).
C****    A    - LOWER LIMIT OF INTEGRAL.
C****    B    - UPPER LIMIT OF INTEGRAL (MAY BE LESS THAN A).
C****    ERR  - USER-SUPPLIED ERROR PARAMETER.  ANS WILL NORMALLY HAVE
C****           NO MORE ERROR THAN ERR TIMES THE INTEGRAL OF THE
C****          ABSOLUTE VALUE OF FUN(X).
C****    ANS  - COMPUTED VALUE OF INTEGRAL.
C****    IERR - ERROR PARAMETER SET BY QNC7:
C****           IERR = 1 IS NORMAL.
C****           IERR = 2 MEANS ANS IS PROBABLY INSUFFICIENTLY ACCURATE.
C****
C**** SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C****    THE EXTERNAL FUNCTION FUN(X) MUST BE SUPPLIED BY THE USER.
C****
C**** METHOD
C****    AN ADAPTIVE 7-POINT NEWTON-COTES ALGOR2THM WITH INTERVAL
C****    BISECTION, COMBINED RELATIVE/ABSOLUTE ERROR CONTROL,
C****    ESTIMATION OF TOTAL QUADRATURE ERROR, AND COMPUTED MAXIMUM
C****    REFINEMENT LEVEL WHEN A IS CLOSE TO B.
C****
C**** REMARKS
C****    THIS ROUTINE HAS BEEN ADAPTED FOR DOUBLE PRECISION USE ON THE
C****    IBM 370/158 BY JEFFREY C. RYMAN, DEPT. OF NUCLEAR ENGINEERING,
C****    KANSAS STATE UNIVERSITY, MANHATTAN, KANSAS 66506.  THIS IS AN
C****    ADAPTATION OF THE SUBROUTINE QNC7 BY RANDALL E. JONES, FROM
C****    THE SANDIA MATHEMATICAL PROGRAM LIBRARY, MATHEMATICAL
C****    COMPUTING SERVICES DIVISION 5422, SANDIA LABORATORIES,
C****    P.O. BOX 5800, ALBUQUERQUE, N.M. 87115, CDC 6600 VERSION 4.7,
C****    18 JULY 1972.  A COPY OF THIS ROUTINE MAY BE FOUND ON PAGE 60
C****    OF:  ADAMS, K.G., BIGGS, F., AND RENKEN, J.H., 'DINT:  A
C****    COMPUTER PROGRAM WHICH PREPARES MULTIGROUP COHERENT-INCOHERENT
C****    CROSS SECTIONS FOR PHOTON TRANSPORT CALCULATIONS', AEC REPORT
C****    SC-RR-72 0684, DEC. 1972.  THE ORIGINAL ROUTINE WAS BY DAVID
C****    L. KAHANER, LASL.
C****
C****    IT SHOULD BE NOTED THAT THE DEFAULT AND MAXIMUM ALLOWABLE
C****    REFINEMENT LEVELS HAVE BEEN LEFT THE SAME AS IN THE CDC
C****    6600 VERSION.
C****
C****
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION AA(40),HH(40),LR(40),VL(40),Q7R(40),F(13),F1(40),F2(40),
     >F3(40),F4(40),F5(40),F6(40),F7(40)
C**** MISCELLANEOUS PARAMETERS.
      DATA SQ2/1.414213562373095D0/
      DATA LMN,NLMX,KMX,KML,NBITS/2,40,5000,7,48/
C**** INITIALIZE.
      ANS=0.D0
      IERR=1
      IF(A.EQ.B) RETURN
      W1=41.D0/140.D0
      W2=216.D0/140.D0
      W3=27.D0/140.D0
      W4=272.D0/140.D0
      LMX=NLMX
      IF(B.EQ.0.D0) GO TO 3
      IF(DSIGN(1.D0,B)*A.LE.0.D0) GO TO 3
      C=DABS(1.D0-A/B)
      IF(C.GT.0.1D0) GO TO 3
      NIB=-IDINT(DLG(C)/DLG(2.D0))
      LMX=MIN0(NLMX,NBITS-NIB-6)
      LMX=MAX0(LMX,LMN)
    3 TOL=DMAX1(ERR,2.D0**(5-NBITS))/2.D0
      IF(ERR.LT.0.D0) TOL=0.5D-6
      EPS=TOL
      HH(1)=(B-A)/12.D0
      AA(1)=A
      LR(1)=1
      DO 4 I=1,13,2
    4 F(I)=FUN(A+DBLE(I-1)*HH(1))
      K=7
      L=1
      AREA=0.D0
      Q7=0.D0
      EF=128.D0/255.D0
      CE=0.D0
C**** COMPUTE REFINED ESTIMATES, ESTIMATE THE ERROR, ETC.
    5 DO 6 I=2,12,2
    6 F(I)=FUN(AA(L)+DBLE(I-1)*HH(L))
      K=K+6
      Q7L=HH(L)*((W1*(F(1)+F(7))+W2*(F(2)+F(6)))+(W3*(F(3)+F(5))+
     >W4*F(4)))
      Q7R(L)=HH(L)*((W1*(F(7)+F(13))+W2*(F(8)+F(12)))+(W3*(F(9)+F(11))+
     >W4*F(10)))
      AREA=AREA+(DABS(Q7L)+DABS(Q7R(L))-DABS(Q7))
      IF(L.LT.LMN) GO TO 11
      Q13=Q7L+Q7R(L)
      IF ( DABS(Q7-Q13).LT. 1.D-74 .OR. DABS(EF).LT. 1.D-74) GOTO 678
      IS=SGN(EF)
      TMP1=DLG(DABS(Q7-Q13))+DLG(DABS(EF))
      EE=DXP(TMP1)*IS
      GOTO 679
 678  EE=0.0D0
 679  IF (EPS.NE.0.AND.AREA.NE.0) GOTO 551
      TMP1=0.0D0
      GOTO 552
551   IS=SGN(AREA)*SGN(EPS)
      TMP1=DXP(DLG(DABS(EPS))+DLG(DABS(AREA)))*IS
552   IF (Q13.NE.0) GOTO 555
      TMP2=0.0D0
      GOTO 558
555   IS=SGN(TOL)
      TMP2=DXP(DLG(DABS(TOL))+DLG(DABS(Q13)))*IS
558   AE=DMAX1(TMP1,TMP2)
      IF(EE-AE) 8,8,10
    7 CE=CE+(Q7-Q13)
      GO TO 9
 8    IF (Q7-Q13.EQ.0) GOTO 9
      IS1=SGN(Q7-Q13)
      TMP=DXP(DLG(DABS(Q7-Q13))-DLG(255.0D0))
      CE=CE+IS1*TMP
    9 IF(LR(L)) 12,12,14
C**** CONSIDER THE LEFT HALF OF NEXT LEVEL.
   10 IF(K.GT.KMX) LMX=KML
      IF(L.GE.LMX) GO TO 7
   11 L=L+1
      EPS=EPS/2.D0
      IF(L.LE.15) EF=EF/SQ2
      HH(L)=HH(L-1)/2.D0
      LR(L)=-1
      AA(L)=AA(L-1)
      Q7=Q7L
      F1(L)=F(7)
      F2(L)=F(8)
      F3(L)=F(9)
      F4(L)=F(10)
      F5(L)=F(11)
      F6(L)=F(12)
      F7(L)=F(13)
      F(13)=F(7)
      F(11)=F(6)
      F(9)=F(5)
      F(7)=F(4)
      F(5)=F(3)
      F(3)=F(2)
      GO TO 5
C**** PROCEED TO RIGHT HALF AT THIS LEVEL.
   12 VL(L)=Q13
   13 Q7=Q7R(L-1)
      LR(L)=1
      AA(L)=AA(L)+12.D0*HH(L)
      F(1)=F1(L)
      F(3)=F2(L)
      F(5)=F3(L)
      F(7)=F4(L)
      F(9)=F5(L)
      F(11)=F6(L)
      F(13)=F7(L)
      GO TO 5
C**** RETURN ONE LEVEL.
   14 VR=Q13
   15 IF(L.LE.1) GO TO 18
      IF(L.LE.15) EF=EF*SQ2
      EPS=EPS*2.D0
      L=L-1
      IF(LR(L).GT.0) GO TO 17
      VL(L)=VL(L+1)+VR
      GO TO 13
   17 VR=VL(L+1)+VR
      GO TO 15
C**** EXIT.
   18 ANS=VR
      IF (TOL.EQ.0.OR.AREA.EQ.0) GOTO 789
      IS=SGN(TOL)*SGN(AREA)
      TMP1=DLG(DABS(TOL))+DLG(DABS(AREA))
      TMP1=DXP(TMP1)
      GOTO 790
 789  TMP1=0.0D0
 790  IF(DABS(CE).LE.2.D0*TMP1)  RETURN
      IERR=2
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE SIMIN (F,K,EPS,ANS,S,NEV,ICONT,Y)
C
C***********************************************************************
C
C     ORIGINAL ROUTINE BY L F SHAMPINE, AS DESCRIBED IN REF.1 BELOW.
C     PREPARATION FOR MATH LIBRARY BY R E JONES.
C
C     ABSTRACT
C        SIMIN FINDS AN APPROXIMATE MINIMUM OF A DOUBLE FUNCTION OF K
C        VARIABLES, GIVEN AN INITIAL ESTIMATE OF THE POSITION OF THE
C        MINIMUM. THE SIMPLEX METHOD IS USED. SEE REFERENCE 1 BELOW
C        FOR A FULL EXPLANATION OF THIS METHOD.  BRIEFLY, A SET OF
C        K+1 POINTS IN K-DIMENSIONAL SPACE IS CALLED A SIMPLEX.
C        THE MINIMIZATION PROCESS ITERATES BY REPLACING THE POINT
C        WITH THE LARGEST FUNCTION VALUE BY A NEW POINT WITH A
C        SMALLER FUNCTION VALUE.  ITERATION CONTINUES UNTIL ALL THE
C        POINTS CLUSTER SUFFICIENTLY CLOSE TO A MINIMUM.
C
C     REFERENCES
C      1. L F SHAMPINE, A ROUTINE FOR UNCONSTRAINED OPTIMIZATION,
C         SC-TM-72130  OR  SC-RR-720657
C      2. J A NELDER AND R MEAD, A SIMPLEX METHOD FOR FUNCTION
C         MINIMIZATION, COMPUTER JOURNAL, 7(1965) 308-313
C
C     DESCRIPTION OF PARAMETERS
C      --INPUT--
C        F  - NAME OF FUNCTION OF K VARIABLES TO BE MINIMIZED.
C             (THIS NAME MUST APPEAR IN AN EXTERNAL STATEMENT.)
C             FORM OF THE CALLING SEQUENCE MUST BE FUNCTION F(X),
C             WHERE X IS AN ARRAY OF K VARIABLES.
C        K  - THE NUMBER OF VARIABLES.  K MUST BE AT LEAST 2.
C             NORMALLY K SHOULD BE LESS THAN ABOUT 10, AS SIMIN
C             BECOMES LESS EFFECTIVE FOR LARGER VALUES OF K.
C        EPS- THE CONVERGENCE CRITERION.  LET YAVG BE THE AVERAGE
C             VALUE OF THE FUNCTION F AT THE K+1 POINTS OF THE
C             SIMPLEX, AND LET R BE THEIR STANDARD ERROR.  (THAT IS,
C             THE ROOT-MEAN-SQUARE OF THE SET OF VALUES (Y(I)-YAVG),
C             WHERE Y(I) IS THE FUNCTION VALUE AT THE I-TH POINT OF
C             THE SIMPLEX.)  THEN--
C             IF EPS.GT.0, CONVERGENCE IS OBTAINED IF  R.LE.EPS.
C             IF EPS.LT.0, CONVERGENCE IS IF  R.LE.ABS(EPS*YAVG).
C             IF EPS=0, THE PROCESS WILL NOT CONVERGE BUT INSTEAD WILL
C             QUIT WHEN NEV FUNCTION EVALUATIONS HAVE BEEN USED.
C        ANS- AN ARRAY OF LENGTH K CONTAINING A GUESS FOR THE LOCATION
C             OF A MINIMUM OF F.
C        S  - A SCALE PARAMETER, WHICH MAY BE A SIMPLE VARIABLE OR AN
C             ARRAY OF LENGTH K.  USE OF AN ARRAY IS SIGNALLED BY
C             SETTING S(1) NEGATIVE.
C             -SIMPLE VARIABLE CASE.  HERE S IS THE LENGTH OF EACH
C             SIDE OF THE INITIAL SIMPLEX.  THUS, THE INITIAL SEARCH
C             RANGE IS THE SAME FOR ALL THE VARIABLES.
C             -ARRAY CASE.  HERE THE LENGTH OF SIDE I OF THE INITIAL
C             SIMPLEX IS ABS(S(I)).  THUS, THE INITIAL SEARCH RANGE
C             MAY BE DIFFERENT FOR DIFFERENT VARIABLES.
C             NOTE-- THE VALUE(S) USED FOR S ARE NOT VERY CRITICAL.
C             ANY REASONABLE GUESS SHOULD DO O.K.
C        NEV- THE MAXIMUM NUMBER OF FUNCTION EVALUATIONS TO BE USED.
C             (THE ACTUAL NUMBER USED MAY EXCEED THIS SLIGHTLY SO THE
C             LAST SEARCH ITERATION MAY BE COMPLETED.)
C        ICONT - ICONT SHOULD BE ZERO ON ANY CALL TO SIMIN WHICH
C             IS NOT A CONTINUATION OF A PREVIOUS CALL.
C             IF ICONT=1 THE PROBLEM WILL BE CONTINUED.  IN THIS
C             CASE THE WORK ARRAY Y MUST BE THE SAME ARRAY THAT WAS
C             USED IN THE CALL THAT IS BEING CONTINUED (AND THE VALUES
C             IN IT MUST BE UNCHANGED).  THE REASON FOR THIS IS THAT
C             IF ICONT=1 THEN THE ARGUMENT S IS IGNORED AND THE SIMPLEX
C             AND RELATED FUNCTION VALUES THAT WERE STORED IN ARRAY Y
C             DURING A PREVIOUS EXECUTION ARE USED TO CONTINUE THAT
C             PREVIOUS PROBLEM.
C        Y  - A WORK ARRAY CONTAINING AT LEAST K*K + 5*K + 1 WORDS.
C             IF ICONT=1 THIS MUST BE THE SAME ARRAY USED IN THE CALL
C             THAT IS BEING CONTINUED.
C      --OUTPUT--
C        ANS- ANS WILL CONTAIN THE LOCATION OF THE POINT WITH THE
C             SMALLEST VALUE OF THE FUNCTION THAT WAS FOUND.
C        S  - IN THE SIMPLE VARIABLE CASE S WILL BE RETURNED AS THE
C             AVERAGE DISTANCE FROM THE VERTICES TO THE CENTROID OF
C             THE SIMPLEX.
C             IN THE ARRAY CASE S(I) WILL BE RETURNED AS THE AVERAGE
C             DISTANCE IN THE I-TH DIMENSION OF VERTICES FROM
C             THE CENTROID.  (S(1) WILL BE NEGATED.)
C             NOTE-- THE VALUE(S) RETURNED IN S ARE USEFUL FOR
C             ASSESSING THE FLATNESS OF THE FUNCTION NEAR THE
C             MINIMUM.  THE LARGER THE VALUE OF S (FOR A GIVEN
C             VALUE OF EPS), THE FLATTER THE FUNCTION.
C        NEV- NEV WILL BE THE COUNT OF THE ACTUAL NUMBER OF FUNCTION
C             EVALUATIONS USED.
C        Y  - WILL CONTAIN ALL DATA NEEDED TO CONTINUE THE MINIMIZATION
C             SEARCH EFFICIENTLY IN A SUBSEQUENT CALL.
C             NOTE -- THE FIRST K+1 ELEMENTS OF Y WILL CONTAIN THE
C             FUNCTION VALUES AT THE K+1 POINTS OF THE LATEST SIMPLEX.
C             THE NEXT K*(K+1) ELEMENTS OF Y WILL BE THE K+1 POINTS
C             OF THE SIMPLEX (IN EXACT CORRESPONDENSE TO THE ARRAY
C             P DISCUSSED IN REFERENCE 1 ABOVE).  THE REMAINING 3*K
C             WORDS ARE TEMPORARY WORKING STORAGE ONLY.
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ANS(K),S(K),Y(15)
      EXTERNAL F
C
      IF(K.GE.2 .AND. S(1).NE.0.) GO TO 10
      PRINT 95
   95 FORMAT(' IN SIMIN,  S(1)=0. OR K IS LESS THAN 2.')
      RETURN
   10 IF(K.GT.50) PRINT 96
   96 FORMAT(' IN SIMIN,  K IS LARGER THAN 50')
C
      IP = K+2
      IC = IP+K*(K+1)
      IR = IC+K
      IRR = IR+K
      CALL SIMINA(F,K,EPS,ANS,S,NEV,ICONT,Y,Y(IP),Y(IC),Y(IR),Y(IRR))
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE SIMINA(F,K,EPS,ANS,S,NEV,ICONT,Y,P,PC,PR,PRR)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ANS(K),S(K),Y(3),P(K,3),PC(K),PR(K),PRR(K)
      DATA ALPHA,BETA,GAMMA/1.0D0,0.5D0,2.0D0/
C
C     SIMINA IS A CORE MINIMIZATION ROUTINE CALLED ONLY BY SIMIN.
C
C     INITIALIZATION
C
      KOUNT = 0
      KK = K
      IF (KK.LE.1) GO TO 99
      ONEK = 1.0/DBLE(KK)
      KP1 = KK+1
      ONEKP1 = 1.0/DBLE(KP1)
      TOL = DBLE(KP1)*EPS**2
C
C     INITIAL SIMPLEX
C
      IF (ICONT.GE.1) GO TO 10
      IF (S(1)) 4,99,1
    1 SKP1 = S(1)*ONEKP1
      DO 2 I=1,KP1
      DO 2 J=1,KK
    2 P(J,I) = ANS(J) - SKP1
      DO 3 J=1,KK
    3 P(J,J+1) = P(J,J+1) + S(1)
      GO TO 7
C
    4 DO 5 I=1,KP1
      DO 5 J=1,KK
    5 P(J,I) = ANS(J) - DABS(S(J))*ONEKP1
      DO 6 J=1,KK
    6 P(J,J+1) = P(J,J+1) + DABS(S(J))
C
C     FUNCTION VALUES FOR INITIAL SIMPLEX
C
    7 I1 = 1
      DO 8 I=1,KP1
      Y(I) = F(P(1,I))
      IF (Y(I).GT.Y(I1)) I1 = I
    8 CONTINUE
      YANS = F(ANS)
      KOUNT = KP1+1
      IF (YANS.GE.Y(I1)) GO TO 10
      Y(I1) = YANS
      DO 9 J=1,KK
    9 P(J,I1) = ANS(J)
C
C     RE-START / NEXT ITERATION
C     IF K.LT.0 VALUES IN THE P AND Y ARRAYS (AND ONLY THESE VALUES)
C     WILL NOT HAVE BEEN DEFINED IN THIS CALL.  THIS IS NON-ANSI USAGE.
C
C     FIRST FIND LARGEST, SECOND LARGEST, AND SMALLEST FUNCTION VALUES.
C
   10 I1 = 1
      IL = 1
      DO 12 I=2,KP1
      IF (Y(I).LT.Y(IL)) IL = I
      IF (Y(I).GT.Y(I1)) I1 = I
   12 CONTINUE
      I2 = IL
      DO 13 I=1,KP1
      IF (I.EQ.I1) GO TO 13
      IF (Y(I).GT.Y(I2)) I2 = I
   13 CONTINUE
C
C     COMPUTE CENTROID, LEAVING OUT P(*,I1)
C
      DO 15 J=1,KK
      SUM = 0.0
      DO 14 I=1,KP1
      IF (I.EQ.I1) GO TO 14
      SUM = SUM + P(J,I)
   14 CONTINUE
   15 PC(J) = SUM*ONEK
C
C     FORM REFLECTED POINT AND TEST
C
      DO 20 J=1,KK
   20 PR(J) = PC(J) + ALPHA*(PC(J)-P(J,I1))
      YR = F(PR)
      KOUNT = KOUNT+1
      IF (YR.LT.Y(IL)) GO TO 30
      IF (YR.GE.Y(I2)) GO TO 40
C
C     ACCEPT REFLECTED POINT
C
   21 Y(I1) = YR
      DO 22 J=1,KK
   22 P(J,I1) = PR(J)
      GO TO 60
C
C     EXPAND IN FAVORABLE DIRECTION AND TEST
C
   30 DO 31 J=1,KK
   31 PRR(J) = PR(J) + GAMMA*(PR(J)-PC(J))
      YRR = F(PRR)
      KOUNT = KOUNT+1
      IF (YRR.GE.YR) GO TO 21
C
C     ACCEPT EXPANDED POINT
C
      Y(I1) = YRR
      DO 32 J=1,KK
   32 P(J,I1) = PRR(J)
      GO TO 60
C
C     DECIDE WHETHER TO ACCEPT REFLECTED POINT.
C
   40 IF (YR.GE.Y(I1)) GO TO 42
      Y(I1) = YR
      DO 41 J=1,KK
   41 P(J,I1) = PR(J)
C
C     TRY CONTRACTION.
C
   42 DO 43 J=1,KK
   43 PR(J) = PC(J) + BETA*(P(J,I1)-PC(J))
      YCT = F(PR)
      KOUNT = KOUNT+1
      IF (YCT.GT.Y(I1)) GO TO 50
      Y(I1) = YCT
      DO 44 J=1,KK
   44 P(J,I1) = PR(J)
      GO TO 60
C
C     ALL EFFORTS FAILED.  SHRINK THE SIMPLEX ABOUT BEST POINT.
C
   50 DO 52 I=1,KP1
      IF (I.EQ.IL) GO TO 52
      DO 51 J=1,KK
   51 P(J,I) = 0.5*(P(J,I)+P(J,IL))
      Y(I) = F(P(1,I))
   52 CONTINUE
      KOUNT = KOUNT+KP1
C
C     CHECK FOR CONVERGENCE
C
   60 IF (KOUNT.GE.NEV) GO TO 65
      IF (EPS.EQ.0.0) GO TO 10
      SUM = 0.0
      DO 61 I=1,KP1
   61 SUM = SUM + Y(I)
      YAVG = SUM*ONEKP1
      SUM = 0.0
      DO 62 I=1,KP1
   62 SUM = SUM + (Y(I)-YAVG)**2
      IF (EPS) 64,63,63
   63 IF (SUM-TOL) 65,65,10
   64 IF (SUM-TOL*DABS(YAVG)) 65,65,10
C
C     CONVERGENCE OBTAINED.
C
C     COMPUTE CENTROID
C
   65 DO 68 J=1,KK
      SUM = 0.0
      DO 67 I=1,KP1
   67 SUM = SUM+P(J,I)
   68 PC(J) = SUM*ONEKP1
      IF (S(1)) 73,69,69
C
C     COMPUTE S(1) AS AVERAGE DISTANCE OF VERTICES FROM CENTROID.
C
   69 DIST = 0.0
      DO 71 I=1,KP1
      SUM = 0.0
      DO 70 J=1,KK
   70 SUM = SUM + (P(J,I)-PC(J))**2
   71 DIST = DIST + DSQRT(SUM)
      S(1) = DIST*ONEKP1
      GO TO 80
C
C     COMPUTE S(J) AS AVERAGE DISTANCE IN J-TH DIMENSION OF
C     VERTICES FROM THE CENTROID.
C
   73 DO 75 J=1,KK
      SUM = 0.0
      DO 74 I=1,KP1
   74 SUM = SUM + DABS(P(J,I)-PC(J))
   75 S(J) = SUM*ONEKP1
      S(1) = -S(1)
C
C     RETURN P(*,IL) AS ANSWER
C
   80 IL = 1
      DO 82 I=2,KP1
      IF (Y(I).LT.Y(IL)) IL = I
   82 CONTINUE
      DO 84 J=1,KK
   84 ANS(J) = P(J,IL)
      NEV = KOUNT
      RETURN
C
C     ERROR MESSAGE
C
   99 PRINT 97
   97 FORMAT(' IN SIMINA,  S(1)=0. OR K IS LESS THAN 2.')
      RETURN
      END
C+****************************** DRTMI *********************************
C
      SUBROUTINE DRTMI (X,F,FCT,XLI,XRI,EPS,IEND,IER)
C
C******************************* DRTMI *********************************
C*                                                                     *
C* THIS SUBROUTINE SOLVES A NONLINEAR EQUATION OF THE FORM FCT(X)=0    *
C* BY THE MUELLER'S ITERATION METHOD.                                  *
C* THIS PROGRAM IS TAKEN FROM THE IBM SSP (RTMI) AND MODIFIED TO WORK  *
C* IN DOUBLE PRECISION.                                                *
C*                                                                     *
C*    INPUT VARIABLES:                                                 *
C*        X    = RETURNED VALUE OF THE ZERO                            *
C*        F    = VALUE OF THE FUNCTION FCT AT THE ZERO                 *
C*        FCT  = EXTERNAL FUNCTION WHOSE ZERO IS TO BE FOUND           *
C*        XLI  = LEFT BOUNDARY OF THE X-AXIS TO BE SEARCHED FOR ROOT   *
C*        XRI  = RIGHT BOUNDARY OF X-AXIS TO BE SEARCHED FOR ROOT      *
C*        EPS  = ACCURACY OF DESIRED RESULT                            *
C*        IEND = MAXIMUM NUMBER OF ITERATIONS TO BE USED               *
C*        IER  = ERROR RETURN CODE; =0 IF DESIRED ACCURACY ACHIEVED    *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C
C     PREPARE ITERATION
      IER = 0
      XL = XLI
      XR = XRI
      X = XL
      TOL = X
      F = FCT(TOL)
      IF (F) 1,16,1
    1 FL = F
      X = XR
      TOL = X
      F = FCT(TOL)
      IF (F) 2,16,2
    2 FR = F
      IF (DSIGN(1.D0,FL)+DSIGN(1.D0,FR)) 25,3,25
C
C     BASIC ASSUMPTION FL*FR LESS THAN 0 IS SATISFIED.
C     GENERATE TOLERANCE FOR FUNCTION VALUES.
    3 I = 0
      TOLF = 100.*EPS
C
C
C     START ITERATION LOOP
    4 I = I+1
C
C     START BISECTION LOOP
      DO 13 K=1,IEND
      X = .5*(XL+XR)
      TOL = X
      F = FCT(TOL)
      IF (F) 5,16,5
    5 IF (DSIGN(1.D0,F)+DSIGN(1.D0,FR)) 7,6,7
C
C     INTERCHANGE XL AND XR IN ORDER TO GET THE SAME SIGN IN F AND FR
    6 TOL = XL
      XL = XR
      XR = TOL
      TOL = FL
      FL = FR
      FR = TOL
    7 TOL = F-FL
      A = F*TOL
      A = A+A
      IF (A-FR*(FR-FL)) 8,9,9
    8 IF (I-IEND) 17,17,9
    9 XR = X
      FR = F
C
C     TEST ON SATISFACTORY ACCURACY IN BISECTION LOOP
      TOL = EPS
      A = DABS(XR)
      IF (A-1.) 11,11,10
   10 TOL = TOL*A
   11 IF (DABS(XR-XL)-TOL) 12,12,13
   12 IF (DABS(FR-FL)-TOLF) 14,14,13
   13 CONTINUE
C     END OF BISECTION LOOP
C
C     NO CONVERGENCE AFTER IEND ITERATION STEPS FOLLOWED BY IEND
C     SUCCESSIVE STEPS OF BISECTION OR STEADILY INCREASING FUNCTION
C     VALUES AT RIGHT BOUNDS. ERROR RETURN.
      IER = 1
   14 IF (DABS(FR)-DABS(FL)) 16,16,15
   15 X = XL
      F = FL
   16 RETURN
C
C     COMPUTATION OF ITERATED X-VALUE BY INVERSE PARABOLIC INTERPOLATION
   17 A = FR-F
      DX = (X-XL)*FL*(1.+F*(A-TOL)/(A*(FR-FL)))/TOL
      XM = X
      FM = F
      X = XL-DX
      TOL = X
      F = FCT(TOL)
      IF (F) 18,16,18
C
C     TEST ON SATISFACTORY ACCURACY IN ITERATION LOOP
   18 TOL = EPS
      A = DABS(X)
      IF (A-1.) 20,20,19
   19 TOL = TOL*A
   20 IF (DABS(DX)-TOL) 21,21,22
   21 IF (DABS(F)-TOLF) 16,16,22
C
C     PREPARATION OF NEXT BISECTION LOOP
   22 IF (DSIGN(1.D0,F)+DSIGN(1.D0,FL)) 24,23,24
   23 XR = X
      FR = F
      GO TO 4
   24 XL = X
      FL = F
      XR = XM
      FR = FM
      GO TO 4
C     END OF ITERATION LOOP
C
C
C     ERROR RETURN IN CASE OF WRONG INPUT DATA
   25 IER = 2
      RETURN
      END
C+**********************************************************************
C
      SUBROUTINE MDBETA(X, P, Q, PROB, IER)
C
C***********************************************************************
C*
C*  FUNCTION:      EVALUATE THE INCOMPLETE BETA DISTRIBUTION FUNCTION
C*
C*  PARAMETERS:
C*  X    - VALUE TO WHICH FUNCTION IS TO BE INTERGRATED. X MUST BE IN TH
C*         RANGE (0,1) INCLUSIVE.
C*  P    - INPUT (1ST) PARAMETER (MUST BE GREATER THAN 0)
C*  Q    - INPUT (2ND) PARAMETER (MUST BE GREATER THAN 0)
C*  PROB - OUTPUT PROBABILITY THAT A RANDOM VAPIABLE FROM A BETA DISTRIB
C*           HAVING PARAMETERS P AND Q WILL BE LESS THAN OR EQUAL TO 0.
C*  IER  - ERROR PARAMETER.
C*           IER = 0 INDICATES A NORMAL EXIT
C*           IER = 1 INDICATES THAT X IS NOT IN THE RANGE (0,1) INCLUSIV
C*           IER = 2 INDICATES THAT P AND/OR Q IS LESS THAN OR EQUAL TO
C*
C* CODE BASED ON SIMILAR CODE BY N. BOSTEN AND E.BATTISTE AS MODIFIED BY
C* M. PIKE AND J. HOO.
C*
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C DOUBLE PRECISION FUNCTION DLGAMA
C MACHINE PRECISION
      DATA EPS/1.0D-6/
C SMALLEST POSITIVE NUMBER REPRESENTABLE
      DATA EPS1/1.0D-78/
C NATURAL LOG OF EPS1
      DATA ALEPS/-179.6016D0/
C CHECK RANGES OF THE ARGUMENTS
      Y = X
      IF ((X.LE.1.0) .AND. (X.GE.0.0)) GO TO 10
      IER = 1
      PROB = 0.0
      GO TO 140
   10 IF ((P.GT.0.0) .AND. (Q.GT.0.0)) GO TO 20
      IER = 2
      PROB  = 0.0
      GO TO 140
   20 IER =0
      IF (X.GT.0.5) GO TO 30
      INT = 0
      GO TO 40
C SWITCH ARGUMENTS FOR MORE EFFICIENT USE OF THE POWER
C SERIES
   30 INT = 1
      TEMP = P
      P = Q
      Q = TEMP
      Y = 1.D0 - Y
   40 IF (X.NE.0. .AND. X.NE.1.) GO TO 60
C SPECIAL CASE - X IS 0. OR 1.
   50 PROB = 0.
      GO TO 130
   60 IB = Q
      TEMP = IB
      PS = Q - FLOAT(IB)
      IF (Q.EQ.TEMP) PS = 1.D0
      DP = P
      DQ = Q
      PX = DP*DLG(Y)
      PQ = DLGAMA(DP+DQ)
      P1 = DLGAMA(DP)
      C = DLGAMA(DQ)
      D4 = DLG(DP)
C DLGAMA IS A FUNCTION WHICH CALCULATES THE DOUBLE
C PRECISION LOG GAMMA FUNCTION
      XB = PX + DLGAMA(PS+DP) - DLGAMA(PS) - D4 - P1
C SCALING
      IB = XB/ALEPS
      INFSUM = 0.D0
C FIRST TERM OF A DECREASING SERIES WILL UNDERFLOW
      IF (IB.NE.0) GO TO 90
      INFSUM = DEXP(XB)
      CNT = INFSUM*DP
C CNT WILL EQUAL DEXP(TEMP)*(1.D0-PS)I*P*Y**I/FACTORIAL(I)
      WH = 0.0D0
   80 WH = WH + 1.D0
      CNT = CNT*(WH-PS)*Y/WH
      IF (CNT.LT.EPS1/EPS) GOTO 90
      XB = CNT/(DP+WH)
      INFSUM = INFSUM + XB
      IF (XB/EPS.GT.INFSUM) GO TO 80
C DLGAMA IS A FUNCTION WHICH CALCULATES THE DOUBLE
C PRECISION LOG GAMMA FUNCTION
   90 FINSUM = 0.D0
      IF (DQ.LE.1.D0) GO TO 120
      XB = PX+ DQ*DLG(1.D0-Y) + PQ - P1 - DLG(DQ) - C
C SCALING
      IB = XB/ALEPS
      IF (IB.LT.0) IB = 0
      C = 1.D0/(1.D0-Y)
      CNT = DEXP(XB-FLOAT(IB)*ALEPS)
      PS = DQ
      WH = DQ
  100 WH =WH -1.D0
      IF (WH.LE.0.0D0) GO TO 120
      PX = (PS*C)/ (DP+WH)
      IF (PX.GT.1.0D0) GO TO 105
      IF (CNT/EPS.LE.FINSUM.OR.CNT.LE.EPS1/PX) GO TO 120
  105 CNT =CNT*PX
      IF (CNT.LE.1.D0) GO TO 110
C RESCALE
      IB = IB - 1
      CNT = CNT*EPS1
  110 PS =WH
      IF (IB.EQ.0) FINSUM = FINSUM + CNT
      GO TO 100
  120 PROB =FINSUM + INFSUM
  130 IF (INT.EQ.0) GO TO 140
      PROB = 1.0 - PROB
      TEMP = P
      P = Q
      Q = TEMP
  140 RETURN
      END
C+**********************************************************************
C
      SUBROUTINE GAMIC(X,ALPHA,REL,N,Y,NZ)
C
C***********************************************************************
C
C     WRITTEN BY D.E. AMOS AND S.L. DANIEL, NOVEMBER, 1974.
C     EXTRACTED FROM THE SANDIA LABORATORY'S MATHEMATICAL
C     LIBRARY AND ADAPTED FOR THE IBM 370 SYSTEM BY J. K. SHULTIS
C     JULY, 1979.
C
C     REFERENCE SC-DR-72 0303
C
C     ABSTRACT
C         GAMIC COMPUTES AN N MEMBER SEQUENCE OF INCOMPLETE GAMMA
C         FUNCTIONS NORMALIZED SO THAT AT X=INFINITY, THE INCOMPLETE
C         GAMMA FUNCTION HAS THE VALUE 1. THE SEQUENCE IS DENOTED BY
C
C         Y(K)=INCGAMMA(ALPHA+K-1,X)/GAMMA(ALPHA+K-1),  K=1,2,...,N
C
C         AND IS COMPUTED TO A RELATIVE ERROR REL OR BETTER WHERE ALPHA
C         .GT.0. IF ALPHA+N-1.GE.X, THE LAST MEMBER IS COMPUTED BY THE
C         CONFLUENT HYPERGEOMETRIC SERIES WITH THE OTHER MEMBERS
C         COMPUTED BY BACKWARD RECURSION ON A TWO-TERM FORMULA,
C
C           Y(K-1)=Y(K)+DEXP((ALPHA+K-1)*DLG(X)-X-DLGAMA(ALPHA+K)).
C
C         IF ALPHA+N-1.LT.X, AN INTEGER M IS ADDED SO THAT
C         ALPHA+N-1+M.GE.X AND THE FIRST PROCEDURE IS APPLIED. SPECIAL
C         PROCEDURES APPLY FOR ALPHA.EQ.1 OR AN UNDERFLOW OCCURS OR
C         X EXCEEDS A CRITICAL VALUE, APTEST, WHERE ALL MEMBERS ARE 1.
C         TO THE WORD LENGTH OF THE CDC 6600. GAMIC USES DLGAMA.
C
C     DESCRIPTION OF ARGUMENTS
C
C         INPUT
C           X      - ARGUMENT, X.GE.0.0
C           ALPHA  - PARAMETER, ALPHA.GT.0.0
C           REL    - RELATIVE ERROR TOLERANCE, REL=1.E-S FOR S
C                    SIGNIFICANT DIGITS
C           N      - NUMBER OF GAMMA FUNCTIONS IN THE SEQUENCE
C                    BEGINNING AT PARAMETER ALPHA, N.GE.1
C
C         OUTPUT
C           Y      - A VECTOR CONTAINING AN N MEMBER SEQUENCE
C                    Y(K)= INCGAMMA(ALPHA+K-1,X)/GAMMA(ALPHA+K-1),
C                    K=1,...,N TO A RELATIVE ERROR REL.
C           NZ     - UNDERFLOW FLAG
C                    NZ.EQ.0, A NORMAL RETURN
C                    NZ.NE.0, UNDERFLOW, Y(K)=0.0, K=N-NZ+1,N RETURNED
C
C     ERROR CONDITIONS
C         IMPROPER INPUT PARAMETERS - A FATAL ERROR
C         UNDERFLOW - A NON-FATAL ERROR.
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /GENCOM/ IGOUT
      DIMENSION Y(1)
      DIMENSION AA(6),BB(6),CC(5)
C
      DATA   AA            / 1.18399941922176D+00, 3.30888136276861D+02,
     > 1.04930832947926D+04, 3.78420325596908D+04, 1.57586618187374D+02,
     > 1.30569632410551D+03/
      DATA   BB            / 1.02652821626751D+00, 9.29753107520368D+03,
     > 6.53848923630220D+06, 2.89543295992889D+08, 8.16836456953161D+03,
     > 4.12237656364399D+06/
      DATA   CC            / 4.30952856710482D+05, 8.27988256743362D+09,
     > 2.41944468684445D+12, 4.21722873236008D+05, 7.56593802747116D+09/
      DATA SLOG/-160.0D0/
      DATA  CON14          / 3.32361913019165D+1/
      DATA  SCALE          /1.0D-60/
      DATA  ASCL           /1.0D-16/
C
      IF(REL.LE.0.) GO TO 91
      IF(N.LT.1) GO TO 92
      IF(ALPHA.LE.0.0) GO TO 93
      NZ=0
      IF(X) 94,10,20
   10 DO 11 I=1,N
   11 Y(I)=0.
      RETURN
C
C     IF X.GE.XLIM(ALPHA+N-1), THEN Y(K)=1., K=1,2,...,N
C
   20 NN=N
      RX=1./X
      NBAR=0
      APN=ALPHA+DBLE(N)-1.
      APTEST=APN
      AM1=ALPHA-1.
      IF(APN.LE.1.) GO TO 22
   21 IF(X.LE.APTEST) GO TO 40
      IF(APTEST.GT.200.) GO TO 25
      S1=((AA(1)*APTEST+AA(2))*APTEST+AA(3))*APTEST+AA(4)
      S2=(APTEST+AA(5))*APTEST+AA(6)
      GO TO 226
   25 IF(APTEST.GT.10000.) GO TO 36
      S1=((BB(1)*APTEST+BB(2))*APTEST+BB(3))*APTEST+BB(4)
      S2=(APTEST+BB(5))*APTEST+BB(6)
  226 XLIM=S1/S2
C
      IF(X.GE.XLIM) GO TO 26
      GO TO 35
   36 S1=((APTEST+CC(1))*APTEST+CC(2))*APTEST+CC(3)
      S2=(APTEST+CC(4))*APTEST+CC(5)
      GO TO 226
   26 DO 27 I=1,N
   27 Y(I)=1.
      RETURN
C
   22 IF(ALPHA.NE.1.) GO TO 32
      IF(X.GT.CON14) GO TO 26
      IF(X.LT.0.1) GO TO 40
      Y(NN)=1.-DEXP(-X)
      RETURN
   32 NBAR=1
      APTEST=APN+1.
      GO TO 21
   35 NBAR=X-APTEST+5.+DBLE(NBAR)
   40 ABAR=APN+DBLE(NBAR)
      XLOG=DLG(X)
      A1=1.
      SUM=1.
      ABK=ABAR+1.
 80   A1 = A1*X/ABK
      SUM = SUM + A1
      IF(A1.LT.REL) GO TO 100
      ABK=ABK+1.
      GO TO 80
  100 YY=SUM*SCALE
      D=ABAR
      IF(NBAR.EQ.0) GO TO 110
  105 CONTINUE
      DO 106 K=1,NBAR
      XOD=X/D
      IF(XOD.LT.ASCL) GO TO 106
      YY=XOD*YY+SCALE
  106 D=D-1.0D0
C
      IF(NZ.NE.0) GO TO 114
 110  E= -X+D*XLOG-DLGAMA(D+1.)
  114 IF(E.GE.SLOG) GO TO 120
      Y(NN)=0.
      NZ=NZ+1
      NN=NN-1
      IF(NN.EQ.0) RETURN
      NBAR=1
      APN=APN-1.
      E=E+DLG(D*RX)
      GO TO 105
 120  EXE = DEXP(E)
      Y(NN)=(EXE/SCALE)*YY
      NM1=NN-1
      IF(NM1.EQ.0) RETURN
      F=EXE*APN*RX
      KK = NN
      AK=DBLE(NN)+AM1
      DO 125 K=1,NM1
      Y(KK-1)=Y(KK)+F
      KK=KK-1
      AK=AK-1.
      F=F*AK*RX
  125 CONTINUE
      RETURN
C
   91 WRITE(IGOUT,191)
  191 FORMAT('IN GAMIC, IMPROPER INPUT FOR REL.')
      RETURN
   92 WRITE(IGOUT,192)
  192 FORMAT('IN GAMIC, IMPROPER INPUT FOR N.')
      RETURN
   93 WRITE(IGOUT,193)
  193 FORMAT('IN GAMIC, IMPROPER INPUT FOR ALPHA.')
      RETURN
   94 WRITE(IGOUT,194)
  194 FORMAT('IN GAMIC, IMPROPER INPUT FOR X.')
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION PRCUMD(X)
C
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE EVALUATES THE PRIOR CUMULATIVE FUNCTION FOR THE     *
C* FAILURE-ON-DEMAND CASES. DISTRIBUTION PARAMETER VALUES AND FUNCTION *
C* DEFINING INDEX ARE PASSED TO THE SUBROUTINE THROUGH THE COMMON BLOCK*
C* /PRIR/.                                                             *
C*                                                                     *
C* SUBROUTINES NEEDED: DXP, MDBETA                                     *
C*                                                                     *
C* COMMON BLOCK /PRIR/ VARIABLES:                                      *
C*    IPRIR = DISTRIBUTION INDEX: =-1 BETA, =-2 LOGBETA,               *
C*            =-3 LOGNORMAL-1, =-4 LOGNORMAL-2, =-5 LOGGAMMA-1,        *
C*            =-6 LOGGAMMA-2,  =-7 LOGLOGNORMAL.                       *
C*    PARM  = DISTRIBUTION PARAMETER VECTOR: PARM(1)=ALPHA, PARM(2)=   *
C*            BETA, PARM(3) FIRST RANGE PARAMETER, PARM(4) SECOND      *
C*            RANGE PARAMETER.                                         *
C*                                                                     *
C* WRITTEN BY J. K. SHULTIS 7/79                                       *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      DOUBLE PRECISION Y(1)
      DATA RT2 / 1.4142135623731D0 /
      PRCUMD=0.0D0
      II=-IPRIR
      GO TO (10,20,30,40,50,60,70),II
C*****
C***** BETA CUMULATIVE FUNCTION:   P=BETA(ALPHA,BETA,A,B)
C***** PARAMETERS: ALPHA,BETA,A,B>0;  0<A<P<B<1
   10 A=PARM(3)
      B=PARM(4)
      IF((B-X)*(X-A)) 11,11,12
   11 IF(X.LE.A) RETURN
      PRCUMD=1.D0
      RETURN
   12 AAA=PARM(1)
      BBB=PARM(2)
      XXX=(X-A)/(B-A)
      CALL MDBETA(XXX,AAA,BBB,PROB,IER)
      PRCUMD=PROB
      RETURN
C*****
C***** LOGBETA CUMULATIVE FUNCTION: -LOG(P)=BETA(ALPHA,BETA,A,B)
C***** PARAMETERS: ALPHA,BETA,A,B>0:  0<EXP(-B)<P<EXP(-A)<1
   20 A=PARM(3)
      B=PARM(4)
      PMIN=DXP(-B)
      PMAX=DXP(-A)
      IF((PMAX-X)*(X-PMIN)) 21,21,22
   21 IF(X.LE.PMIN) RETURN
      PRCUMD=1.D0
      RETURN
   22 AAA=PARM(1)
      BBB=PARM(2)
      XXX=-(DLG(X)+A)/(B-A)
      CALL MDBETA(XXX,AAA,BBB,PROB,IER)
      PRCUMD=1.D0-PROB
      RETURN
C*****
C***** TRUNCATED LOGNORMAL -1: LOG(1/P-1)=TRUN.NORM(ALPHA,BETA,A,B)
C***** PARAMETERS: -INF<ALPHA<INF,  A,B,BETA>0,  0<PMIN<P<PMAX<1
C*****              PMIN=1/(1+EXP(A))   PMAX=1/(1+EXP(B))
C      CHECK THAT ARGUMENT X FALLS IN ACCEPTIBLE RANGE
   30 A=PARM(3)
      PMIN=1.D0/(1.D0+DXP(A))
      IF (X.LE.PMIN) RETURN
      B=PARM(4)
      PMAX=1.D0/(1.D0+DXP(B))
      PRCUMD=1.D0
      IF(X.GE.PMAX) RETURN
C            CALCULATE THE TRUNCATION NORMALIZATION CONSTANT
      AA=PARM(1)
      BB=PARM(2)
      XA=(A-AA)/(RT2*BB)
      ABSXA=DABS(XA)
      XB=(B-AA)/(RT2*BB)
      ABSXB=DABS(XB)
      XX=XA*DERFF(ABSXA)/ABSXA
      TRUNC=0.5D0*(XX - XB*DERFF(ABSXB)/ABSXB)
C            CALCULATE CUMULATIVE FUNCTION AT GIVEN ARGUMENT
      XA=(DLG((1.D0/X)-1.D0)-AA)/(BB*RT2)
      ABSXA=DABS(XA)
      PRCUMD=0.5D0*(XX - XA*DERFF(ABSXA)/ABSXA)/TRUNC
      RETURN
C*****
C***** TRUNCATED LOGNORMAL -2: -LOG(P)=TRUN.NORM(ALPHA,BETA,A,B)
C***** PARAMETERS: -INF<ALPHA<INF,  BETA,A,B>0,  0<EXP(-A)<P<EXP(-B)<1
C          CHECK THAT ARGUMENT IS IN PROPER RANGE
   40 A=PARM(3)
      IF(X.LE.DXP(-A)) RETURN
      B=PARM(4)
      PRCUMD=1.D0
      IF(X.GE.DXP(-B)) RETURN
C            CALCULATE THE TRUNCATION NORMALIZATION CONSTANT
      AA=PARM(1)
      BB=PARM(2)
      XA=(A-AA)/(RT2*BB)
      ABSXA=DABS(XA)
      XB=(B-AA)/(RT2*BB)
      ABSXB=DABS(XB)
      XX=XA*DERFF(ABSXA)/ABSXA
      TRUNC=0.5D0*(XX - XB*DERFF(ABSXB)/ABSXB)
C            CALCULATE CUMULATIVE DISTRIBUTION FOR GIVEN ARGUMENT
      XA=(-DLG(X)-AA)/(BB*RT2)
      ABSXA=DABS(XA)
      PRCUMD=0.5D0*(XX-XA*DERFF(ABSXA)/ABSXA)/TRUNC
      RETURN
C*****
C***** LOGGAMMA CUMULATIVE FUNCTION -1: -LOG(P)+GAMMA(ALPHA,BETA,THETA)
C***** PARAMETERS: ALPHA,BETA,THETA>0,  0 < P < EXP(-THETA)
   50 THETA=PARM(3)
      IF(X) 51,51,52
   51 RETURN
   52 PRCUMD=1.D0
      IF (X.GE.DXP(-THETA)) RETURN
      PSTAR=-(DLG(X)+THETA)/PARM(2)
      AA=PARM(1)
      CALL GAMIC(PSTAR,AA,1.0D-8,1,Y,NZ)
      PRCUMD=1.D0 - Y(1)
      RETURN
C*****
C***** LOGGAMMA CUM. FUNCTION -2:  -LOG(1-P)=GAMMA(ALPHA,BETA,THETA)
C***** PARAMETERS:  ALPHA,BETA,THETA>0,  1-EXP(-THETA) < P < 1
   60 THETA=PARM(3)
      PMIN=1.D0-DXP(-THETA)
      IF (X.LE.PMIN) RETURN
      PRCUMD=1.D0
      IF(X.GE.1.D0) RETURN
      PSTAR=-(DLG(1.D0-X)+THETA)/PARM(2)
      AA=PARM(1)
      CALL GAMIC(PSTAR,AA,1.0D-8,1,Y,NZ)
      PRCUMD=Y(1)
      RETURN
C*****
C***** LOGLOGNORMAL CUM. FUNCTION: -LOG(P)=LOGNORMAL(ALPHA,BETA,THETA)
C***** PARAMETERS: -INF<ALPHA,BETA<INF;  1-EXP(-THETA)<P<1
   70 TH=PARM(3)
      PMIN=1.D0-DXP(-TH)
      IF((1.D0-X)*(X-PMIN)) 71,71,72
   71 IF(X.LE.PMIN) RETURN
      PRCUMD=1.D0
      RETURN
   72 AA=PARM(1)
      BB=PARM(2)
      D1P=DLG(1.D0-X)
      XX=DLG(-D1P-TH)
      PSTAR=(XX-AA)/BB
      XX=PSTAR/1.414213562373
      PSTAR=DABS(XX)
      PRCUMD=0.5D0+0.5D0*XX*DERFF(PSTAR)/PSTAR
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE MRGCLD (ANS,IERR)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C*  THIS SUBROUTINE CALCULATES THE MARGINAL PROBABILITY FOR THE        *
C*  FAILURE ON DEMAND CASE USING THE HINTD SUBROUTINE PACKAGE.         *
C*                                                                     *
C*  REQUIRED SUBROUTINES: HINTD                                        *
C*                                                                     *
C*  WRITTEN BY S. D. HANSEN 9/79.                                      *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      THETA=PARM(3)
      A=PARM(3)
      B=PARM(4)
      II=-IPRIR
      GOTO (100,200,300,400,500,500), II
 100  BV=B
      CALL HINTD(BV,ANS,IERR)
      RETURN
 200  BD=DXP(-A)
      CALL HINTD(BD,ANS,IERR)
      RETURN
 300  BP=1/(1+DXP(B))
      CALL HINTD(BP,ANS,IERR)
      RETURN
 400  DB=DXP(-B)
      CALL HINTD(DB,ANS,IERR)
      RETURN
 500  DT=DXP(-THETA)
      CALL HINTD(DT,ANS,IERR)
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE PSTERD (PX,ANS,IERR)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C*  THIS SUBROUTINE CALCULATES THE POSTERIOR PROBABILITY               *
C*  FOR THE IPRIR DISTRIBUTION FOR THE FAILURE ON DEMAND CASE.         *
C*                                                                     *
C* COMMON BLOCK /DIST/ VARIABLES:                                      *
C*   VMARG = CONTAINS THE VALUE OF THE MARGINAL FOR THE IPRIR          *
C*           DISTRIBUTION. VALUE OBTAINED FROM SUBROUTINE MRGCLD       *
C*   IMRGCL = CONTAINS THE FLAG INDICATING THE MARGINAL NEEDS TO BE    *
C*            CALCULATED OR HAS BEEN CALCULATED BY ANOTHER SUBROUTINE. *
C*            IMRGCL = 1 INDICATES THE MRGINAL HAS BEEN CALCULATED     *
C*            AND STORED IN VMARG OF THE COMMON BLOCK.                 *
C*                                                                     *
C*  REQUIRED SUBROUTINES:  HINTD PACKAGE                               *
C*                                                                     *
C*    WRITTEN BY: S. D. HANSEN 6/79.                                   *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /DIST/VMARG,IMRGCL
      COMMON /DATAD/ K,N
      IERR=1
      IF (IMRGCL.EQ.1) GOTO 100
      CALL MRGCLD (VMARG,IERR)
      IMRGCL=1
 100  IF (PX.LE.0.0D0.OR.PX.GE.1.0D0) GOTO 200
      GP=PRIORD(PX)
      IF (GP.EQ.0.0D0) GOTO 200
      BIN=DLGAMA(N+1.0D0)-DLGAMA(K+1.0D0)-DLGAMA(N-K+1.0D0)+K*DLG(PX) +
     >(N-K)*DLG(1.0D0-PX) + DLG(GP)
      ANS=DXP(BIN)/VMARG
      RETURN
 200  ANS=0.0D0
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE PSTCMD (PX,ANS,IERR)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C*  THIS SUBROUTINE CALCULATES THE CUMULATIVE POSTERIOR PROBABILITY    *
C*  FOR THE IPRIR DISTRIBUTION FOR THE FAILURE ON DEMAND CASE.         *
C*                                                                     *
C* COMMON BLOCK /DIST/ VARIABLES:                                      *
C*   VMARG = CONTAINS THE VALUE OF THE MARGINAL FOR THE IPRIR          *
C*           DISTRIBUTION. VALUE OBTAINED FROM SUBROUTINE MRGCLD       *
C*   IMRGCL = CONTAINS THE FLAG INDICATING THE MARGINAL NEEDS TO BE    *
C*            CALCULATED OR HAS BEEN CALCULATED BY ANOTHER SUBROUTINE. *
C*            IMRGCL = 1 INDICATES THE MRGINAL HAS BEEN CALCULATED     *
C*            AND STORED IN VMARG OF THE COMMON BLOCK.                 *
C*                                                                     *
C*                                                                     *
C*  REQUIRED SUBROUTINES:   HINTD PACKAGE                              *
C*                                                                     *
C*  WRITTEN BY S. D. HANSEN 6/79.                                      *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /DIST/VMARG,IMRGCL
      IERR1=1
      IF (IMRGCL.EQ.1) GOTO 100
      CALL MRGCLD (VMARG,IERR1)
      IMRGCL=1
  100 CALL HINTD (PX,ANS,IERR2)
      ANS=ANS/VMARG
      IERR=1
      IF (IERR1.GT.1.OR.IERR2.GT.1) IERR=2
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION PRIORD(X)
C
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE EVALUATES THE PRIOR DENSITY FUNCTION FOR THE FAIL-  *
C* URE-ON-DEMAND CASES.  DISTRIBUTION PARAMETER VALUES AND FUNCTION    *
C* DEFINING INDEX ARE PASSED TO THE SUBROUTINE THROUGH THE COMMON BLOCK*
C* /PRIR/.                                                             *
C*                                                                     *
C* SUBROUTINES NEEDED: DXP,DLG,DLGAMA                                  *
C*                                                                     *
C* COMMON BLOCK /PRIR/ VARIABLES:                                      *
C*    IPRIR = DISTRIBUTION INDEX: =-1 BETA, =-2 LOGBETA,               *
C*            =-3 LOGNORMAL-1, =-4 LOGNORMAL-2, =-5 LOGGAMMA-1,        *
C*            =-6 LOGGAMMA-2,  =-7 LOGLOGNORMAL.                       *
C*    PARM  = DISTRIBUTION PARAMETER VECTOR: PARM(1)=ALPHA, PARM(2)=   *
C*            BETA, PARM(3)=FIRST RANGE PARAMETER, PARM(4)=SECOND      *
C*            RANGE PARAMETER.                                         *
C*                                                                     *
C* WRITTEN BY J. K. SHULTIS 7/79                                       *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      DATA RT2, PI2LN / 1.4142135623731D0, 0.91893853320467D0/
      PRIORD=0.0D0
      II=-IPRIR
      GO TO (10,20,30,40,50,60,70),II
C*****
C***** BETA DENSITY FUNCTION:   P=BETA(ALPHA,BETA,A,B)
C***** PARAMETERS: ALPHA,BETA,A,B>0;  0<A<P<B<1
   10 A=PARM(3)
      B=PARM(4)
      IF((B-X)*(X-A)) 11,11,12
   12 AA=PARM(1)
      BB=PARM(2)
      ARG=(DLGAMA(AA+BB)-DLGAMA(AA)-DLGAMA(BB)) +
     >((1.0D0-AA-BB)*DLG(B-A)+(AA-1.D0)*DLG(X-A)+(BB-1.D0)*DLG(B-X))
      PRIORD=DXP(ARG)
   11 RETURN
C*****
C***** LOGBETA DENSITY FUNCTION: -LOG(P)=BETA(ALPHA,BETA,A,B)
C***** PARAMETERS: ALPHA,BETA,A,B>0:  0<EXP(-B)<P<EXP(-A)<1
   20 A=PARM(3)
      B=PARM(4)
      PMIN=DXP(-B)
      PMAX=DXP(-A)
      IF((PMAX-X)*(X-PMIN)) 21,21,22
   22 AA=PARM(1)
      BB=PARM(2)
      DL=DLG(X)
      ARG=((DLGAMA(AA+BB)-DLGAMA(AA)-DLGAMA(BB)) + ((1.D0-AA-BB)*DLG(B-
     >A)+(AA-1.D0)*DLG(-DL-A)+(BB-1.D0)*DLG(B+DL))) - DL
      PRIORD=DXP(ARG)
   21 RETURN
C*****
C***** TRUNCATED LOGNORMAL -1: LOG(1/P-1)=TRUN.NORM(ALPHA,BETA,A,B)
C***** PARAMETERS: -INF<ALPHA<INF,  BETA,A,B>0, 0<PMIN<P<PMAX<1
C*****             PMIN=1/(1+EXP(A))   PMAX=1/(1+EXP(B)
C       CHECK RANGE OF P
   30 A=PARM(3)
      B=PARM(4)
      PMIN=1.D0/(1.D0+DXP(A))
      PMAX=1.D0/(1.D0+DXP(B))
      IF((X.GE.PMAX).OR.(X.LE.PMIN)) RETURN
C         CALCULATED TRUNCATION NORMALIZATION CONSTANT
      AA=PARM(1)
      BB=PARM(2)
      XA=(A-AA)/(RT2*BB)
      ABSXA=DABS(XA)
      XB=(B-AA)/(RT2*BB)
      ABSXB=DABS(XB)
      XX=XA*DERFF(ABSXA)/ABSXA
      TRUNC=0.5D0*(XX - XB*DERFF(ABSXB)/ABSXB)
C          CALCULATE THE DENSITY
      DL=DLG(X)
      DLP=DLG(1.D0-X)
      ARG=-DLG(TRUNC)-DL-DLP-PI2LN-DLG(BB)-0.5D0*((DLP-DL-AA)/BB)**2
      PRIORD=DXP(ARG)
      RETURN
C*****
C***** TRUNCATED LOGNORMAL -2: -LOG(P)=TRUN.NORM(ALPHA,BETA,A,B)
C***** PARMATERS:-INF<ALPHA<INF,  BETA,A,B>0,  0<EXP(-A)<P<EXP(-B)<1
   40 A=PARM(3)
      B=PARM(4)
      PMIN=DXP(-A)
      PMAX=DXP(-B)
      IF((X.GE.PMAX).OR.(X.LE.PMIN)) RETURN
C            EVALUATE TRUNCATION NORMALIZATION CONSTANT
      AA=PARM(1)
      BB=PARM(2)
      XA=(A-AA)/(RT2*BB)
      ABSXA=DABS(XA)
      XB=(B-AA)/(RT2*BB)
      ABSXB=DABS(XB)
      XX=XA*DERFF(ABSXA)/ABSXA
      TRUNC=0.5D0*(XX - XB*DERFF(ABSXB)/ABSXB)
C            CALCULATE DENSITY FOR GIVEN VALUE OF X  (I.E. P)
      DL=DLG(X)
      ARG=-DLG(TRUNC)-DL-PI2LN-DLG(BB)-0.5D0*((DL+AA)/BB)**2
      PRIORD=DXP(ARG)
      RETURN
C*****
C***** LOGGAMMA DENSITY FUNCTION -1: -LOG(P)=GAMMA(ALPHA,BETA,THETA)
C***** PARAMETERS:  ALPHA,BETA,THETA>0,   0<P<EXP(-THETA)
   50 THETA=PARM(3)
      IF((X.LE.0.0D0).OR.(X.GE.DXP(-THETA))) RETURN
      DL=DLG(X)
      AA=PARM(1)
      BB=PARM(2)
      ARG=-DLGAMA(AA)-AA*DLG(BB)-DL+(AA-1.D0)*DLG(-DL-THETA)
     >    +(DL+THETA)/BB
      PRIORD=DXP(ARG)
      RETURN
C*****
C***** LOGGAMMA DENSITY FUNCTION -2: -LOG(1-P)=GAMMA(ALPHA,BETA,THETA)
C***** PARAMETERS:  ALPHA,BETA,THETA>0,    1-EXP(-THETA) < P < 1
   60 THETA=PARM(3)
      PMIN=1.D0 - DXP(-THETA)
      IF((X.LE.PMIN).OR.(X.GE.1.D0)) RETURN
      DL=DLG(1.D0-X)
      AA=PARM(1)
      BB=PARM(2)
      ARG=-DLGAMA(AA)-AA*DLG(BB)-DL+(AA-1.D0)*DLG(-DL-THETA)
     >    +(DL+THETA)/BB
      PRIORD=DXP(ARG)
      RETURN
C*****
C***** LOGLOGNORMAL DENSITY FUNCTION: -LOG(P)=LOGNORMAL(ALPHA,BETA,THETA
C***** PARAMETERS: -INF<ALPHA,BETA<INF;  1-EXP(-THETA)<P<1
   70 TH=PARM(3)
      PMIN=1.D0-DXP(-TH)
      IF((1.D0-X)*(X-PMIN)) 71,71,72
   72 AA=PARM(1)
      BB=PARM(2)
      D1P=DLG(1.D0-X)
      XX=DLG(-D1P-TH)
      ARG=-D1P-XX-DLG(BB)-0.9189385332047-(0.5D0*(XX-AA)**2/BB**2)
      PRIORD=DXP(ARG)
   71 RETURN
      END
C+**********************************************************************
C*                                                                     *
      SUBROUTINE HINTD (PV,ANS,IERR)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C*  THIS SUBROUTINE CALCULATES THE VALUE OF THE I-SUB-P INTEGRAL FOR   *
C* THE IPRIR DISTRIBUTION.                                             *
C*                                                                     *
C*  REQUIRED SUBROUTINES:  BETADF, FXB, GAM1DF, FLG2DF, GAM1DF, SGN,   *
C*                         DXP, GAM2DF, FLLDF                          *
C* COMMON BLOCK /PRIR/ VARIABLES:                                      *
C*    IPRIR = DISTRIBUTION INDEX: =-1 BETA, =-2 LOGBETA,               *
C*            =-3 LOGNORMAL-1, =-4 LOGNORMAL-2, =-5 LOGGAMMA-1,        *
C*            =-6 LOGGAMMA-2,  =-7 LOGLOGNORMAL.                       *
C*    PARM  = DISTRIBUTION PARAMETER VECTOR: PARM(1)=ALPHA, PARM(2)=   *
C*            BETA, PARM(3)=FIRST RANGE PARAMETER, PARM(4)=SECOND      *
C*            RANGE PARAMETER.                                         *
C*                                                                     *
C* COMMON BLOCK /ACPRMD/ VARIABLES:                                    *
C*   EPS = ACCURACY PARAMETER FOR INTEGRATION ROUTINE.                 *
C*         THIS PARAMETER NEEDS TO BE SET BY THE CALLING ROUTINE.      *
C*                                                                     *
C*                                                                     *
C* WRITTEN BY S. D. HANSEN 8/79                                        *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      COMMON /ACPRMD/ EPS
      COMMON /DATAD/K,N
      EXTERNAL BETADF,BET1DF,FLG1DF,FLG2DF,GAM1DF,GAM2DF,FLLDF
      DATA RT2,PI /1.4142135623731D0, 3.14159265358979D0/
      ALPHA=PARM(1)
      BETA=PARM(2)
      THETA=PARM(3)
      A=PARM(3)
      B=PARM(4)
      II=-IPRIR
      IF (II.GE.3) GOTO 50
      BAB=FXB(ALPHA,BETA)
 50   PRMLN=DLGAMA(N+1.0D0)-DLGAMA(K+1.0D0)-DLGAMA(N-K+1.0D0)
      ERR=EPS
      GOTO (100,200,300,400,500,600,700),II
C
C
C   BETA
 100  IF (A.EQ.0.AND.B.EQ.1) GOTO 150
      TLIM=(PV-A)/(B-A)
      IF (PV.GT.B.OR.PV.LT.A) GOTO 175
      IF (PV.EQ.A) GOTO 185
      CALL QNC7(BETADF,0.0D0,TLIM,ERR,ANS,IERR)
      R1=1-(1-TLIM)**BETA
      R2=BFN(1.0D0)-DLG(BETA)+DLG(R1)
      S1=BFN(0.0D0)-DLG(ALPHA)+DLG(TLIM)
      C=DXP(R2)+DXP(S1)
      ANS=DXP(DLG(ANS+C)-BAB)
      RETURN
 150  IF (PV.EQ.0.0D0) GOTO 185
      AK=ALPHA+K
      BNK=N-K+BETA
      PV1=PV
      CALL MDBETA(PV1,AK,BNK,AMDB,IERR)
      DL=(DLGAMA(N+1.0D0)-DLGAMA(ALPHA+BETA+N))+(DLGAMA(N-K+BETA)-DLGAMA
     >(N-K+1.0D0))+(DLGAMA(ALPHA+K)-DLGAMA(K+1.0D0))+(DLGAMA(ALPHA+BETA)
     >-DLGAMA(ALPHA)-DLGAMA(BETA))
      Q1=DLG(AMDB+0.0D0)+DL
      ANS=DXP(Q1)
      IERR=IERR+1
      RETURN
 175  ANS=0.0D0
      IERR=10
      RETURN
 185  ANS=0.0D0
      IERR=1
      RETURN
C
C
C   LOG BETA
 200  AP=DXP(-A)
      BP=DXP(-B)
      IF (BP.GT.AP) GOTO 250
      IF (AP.GT.1.0D0.OR.AP.LT.0.0D0) GOTO 250
      IF (BP.GT.1.0D0.OR.BP.LT.0.0D0) GOTO 250
      IF (PV.GT.AP.OR.PV.LT.BP) GOTO 250
      IF (ALPHA.GT.1.AND.BETA.GT.1) GOTO 275
      BLIM=-(DLG(PV)+A)/(B-A)
      IF (BLIM.LT.0.0D0) BLIM=0.0D0
      CALL QNC7(BETADF,BLIM,1.0D0,ERR,ANS,IERR)
      S1=BETA*DLG(1-BLIM)
      S1=DXP(S1)-1
      S2=DXP(BFN(1.0D0)-DLG(BETA))*S1
      R1=DLG(1-BLIM**ALPHA)
      R2=BFN(0.0D0)+R1-DLG(ALPHA)
      C=S2+DXP(R2)
      ANS=DXP(PRMLN-BAB+DLG(ANS+C))
      RETURN
 250  ANS=0.0D0
      IERR=10
      RETURN
 275  BLIM=BP
      TLIM=PV
      CALL QNC7(BET1DF,BLIM,TLIM,ERR,ANS,IERR)
      S1=(ALPHA+BETA-1)*DLG(B-A)
      ANS=DXP(DLG(ANS)+PRMLN-BAB-S1)
      RETURN
C
C
C   LOGNORMAL  1
 300  BLIM=1/(1+DXP(A))
      TLIM=1/(1+DXP(B))
      IF (PV.GT.TLIM.OR.PV.LT.BLIM) GOTO 350
      IF (TLIM.EQ.BLIM) GOTO 360
      IF (PV.EQ.BLIM) GOTO 360
      CALL QNC7(FLG1DF,BLIM,PV,ERR,ANS,IERR)
C         CALCULATED TRUNCATION NORMALIZATION CONSTANT
      AA=PARM(1)
      BB=PARM(2)
      XA=(A-AA)/(RT2*BB)
      ABSXA=DABS(XA)
      XB=(B-AA)/(RT2*BB)
      ABSXB=DABS(XB)
      XX=XA*DERFF(ABSXA)/ABSXA
      TRUNC=0.5D0*(XX - XB*DERFF(ABSXB)/ABSXB)
      ANS=DXP(PRMLN-DLG(TRUNC)-.5*DLG(2*PI*BETA**2)+DLG(ANS))
      RETURN
 350  ANS=0.0D0
      IERR=10
      RETURN
 360  ANS=0.0D0
      IERR=1
      RETURN
C
C
C   LOGNORMAL  2
 400  BLIM=DXP(-A)
      TLIM=DXP(-B)
      IF (PV.GT.TLIM.OR.PV.LT.BLIM) GOTO 450
      IF (TLIM.EQ.BLIM) GOTO 460
      IF (PV.EQ.BLIM) GOTO 460
      CALL QNC7(FLG2DF,BLIM,PV,ERR,ANS,IERR)
C         CALCULATED TRUNCATION NORMALIZATION CONSTANT
      AA=PARM(1)
      BB=PARM(2)
      XA=(A-AA)/(RT2*BB)
      ABSXA=DABS(XA)
      XB=(B-AA)/(RT2*BB)
      ABSXB=DABS(XB)
      XX=XA*DERFF(ABSXA)/ABSXA
      TRUNC=0.5D0*(XX - XB*DERFF(ABSXB)/ABSXB)
      Q2=.5*DLG(2*PI*BETA**2)
      ANS=DXP(PRMLN-DLG(TRUNC)-Q2+DLG(ANS))
      RETURN
 450  ANS=0.0D0
      IERR=10
      RETURN
 460  ANS=0.0D0
      IERR=1
      RETURN
C
C
C    LOG GAMMA  1
 500  IF (PV.GT.DXP(-THETA).OR.PV.LT.0.0D0) GOTO 550
      IF (PV.EQ.0.0D0) GOTO 560
      CALL QNC7(GAM1DF,0.0D0,PV,ERR,ANS,IERR)
      ANS=DXP(PRMLN-DLGAMA(ALPHA)-ALPHA*DLG(BETA)+DLG(ANS))
      RETURN
 550  ANS=0.0D0
      IERR=10
      RETURN
 560  ANS=0.0D0
      IERR=1
      RETURN
C
C   LOG GAMMA  2
 600  BLIM=1-PV
      TLIM=DXP(-THETA)
      IF (PV.GT.1.OR.PV.LT.1-TLIM) GOTO 650
      IF (BLIM.EQ.TLIM) GOTO 660
      CALL QNC7(GAM2DF,BLIM,TLIM,ERR,ANS,IERR)
      ANS=DXP(PRMLN-DLGAMA(ALPHA)-ALPHA*DLG(BETA)+DLG(ANS))
      RETURN
 650  ANS=0.0D0
      IERR=10
      RETURN
 660  ANS=0.0D0
      IERR=1
      RETURN
C
C
C   LOG LOGNORMAL
 700  BLIM=1-DXP(-THETA)
      CALL QNC7(FLLDF,BLIM,PV,ERR,ANS,IERR)
      ANS=DXP(PRMLN+DLG(ANS))/DSQRT(2*PI*BETA**2)
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION BETADF(X)
C
C**********************************************************************
C
C   THIS FUNCTION EVALUATES THE BETA OR LOG BETA INTEGRAL FOR FAILURE ON
C   DEMAND CASE.
C
C   NOTE:  BFN FUNCTION IS A TWO CASE FUNCTION WHICH RETURNS
C          EITHER F(Y) FOR BETA DISTRIBUTION OR G(X) FOR LOG BETA
C          DISTRIBUTION.
C*
C* COMMON BLOCK /PRIR/ VARIABLES:
C*    IPRIR = DISTRIBUTION INDEX: =-1 BETA, =-2 LOGBETA,
C*            =-3 LOGNORMAL-1, =-4 LOGNORMAL-2, =-5 LOGGAMMA-1,
C*            =-6 LOGGAMMA-2,  =-7 LOGLOGNORMAL.
C*    PARM  = DISTRIBUTION PARAMETER VECTOR: PARM(1)=ALPHA, PARM(2)=
C*            BETA, PARM(3)=FIRST RANGE PARAMETER, PARM(4)=SECOND
C*            RANGE PARAMETER.
C*
C* COMMON BLOCK /DATAD/ VARIABLES
C*    K = THE NUMBER OF FAILURES THAT OCCURRED AFTER N ATTEMPTS.
C*    N = THE TOTAL NUMBER OF ATTEMPTS MADE DURING WHICH K FAILURES
C*        OCCURRED.
C*
C* WRITTEN BY S. D. HANSEN 8/79
C*
C**********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      COMMON /DATAD/K,N
      ALPHA=PARM(1)
      BETA=PARM(2)
      IF (X.EQ.0.0D0) GOTO 100
      IF (X.EQ.1) GOTO 200
      Q1=BFN(0.0D0)+(1-BETA)*DLG(1-X)
      Q2=(DXP(BFN(X))-DXP(Q1))
      IF (Q2.EQ.0.0D0) GOTO 100
      IS=SGN(Q2)
      Q2=DXP(DLG(DABS(Q2))+(ALPHA-1)*DLG(X))*IS
      Q4=Q2-DXP(BFN(1.0D0))
      IF (Q4.EQ.0.0D0) GOTO  300
      IS=SGN(Q4)
      Q4=DLG(DABS(Q4))
      Q4=DXP((BETA-1)*DLG(1-X)+Q4)
      BETADF=Q4*IS
      RETURN
 100  BETADF=-DXP(BFN(1.0D0))
      RETURN
 200  BETADF=-DXP(BFN(0.0D0))
      RETURN
 300  BETADF=0.0
      RETURN
      END
C+*********************************************************************
C
      DOUBLE PRECISION FUNCTION BET1DF (P)
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/ PARM(4),RNG(2),IPRIR
      COMMON /DATAD/ K,N
      IF (P.EQ.0.OR.P.EQ.1) GOTO 100
      IF (P.EQ.DXP(-PARM(4))) GOTO 100
      ALPHA=PARM(1)
      BETA=PARM(2)
      A=PARM(3)
      B=PARM(4)
      Q1=K*DLG(P)+(N-K)*DLG(1-P)+(ALPHA-1)*DLG(-DLG(P)-A)
      TMP=B+DLG(P)
      Q1=Q1+(BETA-1)*DLG(TMP)-DLG(P)
      BET1DF=DXP(Q1)
      RETURN
 100  BET1DF=0.0D0
      RETURN
      END
C+*********************************************************************
C
      DOUBLE PRECISION FUNCTION FLG1DF (P)
C
C**********************************************************************
C
C   THIS FUNCTION RETURNS THE VALUE OF THE INTEGRAND OF THE LOGNORMAL-1
C   DISTRIBUTION FOR THE FAILURE ON DEMAND CASE.
C
C* COMMON BLOCK /PRIR/ VARIABLES:
C*    IPRIR = DISTRIBUTION INDEX: =-1 BETA, =-2 LOGBETA,
C*            =-3 LOGNORMAL-1, =-4 LOGNORMAL-2, =-5 LOGGAMMA-1,
C*            =-6 LOGGAMMA-2,  =-7 LOGLOGNORMAL.
C*    PARM  = DISTRIBUTION PARAMETER VECTOR: PARM(1)=ALPHA, PARM(2)=
C*            BETA, PARM(3)=FIRST RANGE PARAMETER, PARM(4)=SECOND
C*            RANGE PARAMETER.
C*
C* COMMON BLOCK /DATAD/ VARIABLES
C*    K = THE NUMBER OF FAILURES THAT OCCURRED AFTER N ATTEMPTS.
C*    N = THE TOTAL NUMBER OF ATTEMPTS MADE DURING WHICH K FAILURES
C*        OCCURRED.
C*
C* WRITTEN BY S. D. HANSEN 8/79
C*
C**********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      COMMON /DATAD/K,N
      IF (P.EQ.0.0D0.AND.K.EQ.0) GOTO 500
      IF (P.EQ.0.OR.P.GT.0.99999) GOTO 500
      ALPHA=PARM(1)
      BETA=PARM(2)
      Q1=(K-1)*DLG(P)+(N-K-1)*DLG(1-P)
      Q2=-(DLG(1/P-1)-ALPHA )**2/(2*BETA**2)
      FLG1DF=DXP(Q1+Q2)
      RETURN
 500  FLG1DF=0.0D0
      RETURN
      END
C+*********************************************************************
C
      DOUBLE PRECISION FUNCTION FLG2DF (P)
C
C**********************************************************************
C
C   THIS FUNCTION EVALUATES THE LOGNORMAL-2 INTEGRAND FOR THE
C   FAILURE ON DEMAND CASE.
C
C* COMMON BLOCK /PRIR/ VARIABLES:
C*    IPRIR = DISTRIBUTION INDEX: =-1 BETA, =-2 LOGBETA,
C*            =-3 LOGNORMAL-1, =-4 LOGNORMAL-2, =-5 LOGGAMMA-1,
C*            =-6 LOGGAMMA-2,  =-7 LOGLOGNORMAL.
C*    PARM  = DISTRIBUTION PARAMETER VECTOR: PARM(1)=ALPHA, PARM(2)=
C*            BETA, PARM(3)=FIRST RANGE PARAMETER, PARM(4)=SECOND
C*            RANGE PARAMETER.
C*
C* COMMON BLOCK /DATAD/ VARIABLES
C*    K = THE NUMBER OF FAILURES THAT OCCURRED AFTER N ATTEMPTS.
C*    N = THE TOTAL NUMBER OF ATTEMPTS MADE DURING WHICH K FAILURES
C*        OCCURRED.
C*
C* WRITTEN BY S. D. HANSEN 8/79
C*
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      COMMON /DATAD/K,N
      IF (P.EQ.0.OR.P.EQ.1) GOTO 100
      ALPHA=PARM(1)
      BETA=PARM(2)
      Q1=(K-1)*DLG(P)+(N-K)*DLG(1-P)
      Q2=-(-DLG(P)-ALPHA)**2/(2*BETA**2)
      FLG2DF=DXP(Q1+Q2)
      RETURN
 100  FLG2DF=0.0D0
      RETURN
      END
C+*********************************************************************
C
      DOUBLE PRECISION FUNCTION GAM1DF (P)
C
C**********************************************************************
C
C   THIS FUNCTION EVALUATES THE INTEGRAND OF THE LOGNORMAL-1
C   DISRIBUTION FOR THE FAILURE ON DEMAND CASE.
C
C* COMMON BLOCK /PRIR/ VARIABLES:
C*    IPRIR = DISTRIBUTION INDEX: =-1 BETA, =-2 LOGBETA,
C*           =-3 LOGNORMAL-1, =-4 LOGNORMAL-2, =-5 LOGGAMMA-1,
C*           =-6 LOGGAMMA-2,  =-7 LOGLOGNORMAL.
C*    PARM  = DISTRIBUTION PARAMETER VECTOR: PARM(1)=ALPHA, PARM(2)=
C*            BETA, PARM(3)=FIRST RANGE PARAMETER, PARM(4)=SECOND
C*            RANGE PARAMETER.
C*
C* COMMON BLOCK /DATAD/ VARIABLES
C*    K = THE NUMBER OF FAILURES THAT OCCURRED AFTER N ATTEMPTS.
C*    N = THE TOTAL NUMBER OF ATTEMPTS MADE DURING WHICH K FAILURES
C*        OCCURRED.
C*
C* WRITTEN BY S. D. HANSEN 8/79
C*
C**********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      COMMON /DATAD/K,N
      ALPHA=PARM(1)
      BETA=PARM(2)
      THETA=PARM(3)
      IF (P.EQ.0.OR.P.EQ.1) GOTO 500
      TMP=-DLG(P)-THETA
      IF (TMP.EQ.0.0D0) GOTO 500
      IS=SGN(TMP)
      Q1=(K-1)*DLG(P)+(N-K)*DLG(1-P)+(ALPHA-1)*DLG(DABS(TMP))
      Q2=-(-DLG(P)-THETA)/BETA
      GAM1DF=DXP(Q1+Q2)*IS
      RETURN
 500  GAM1DF=0.0D0
      RETURN
      END
C+*********************************************************************
C
      DOUBLE PRECISION FUNCTION GAM2DF (X)
C
C**********************************************************************
C
C   THIS FUNCTION EVALUATES THE LOG GAMMA INTEGRAND FOR THE FAILURE
C   ON DEMAND CASE.
C
C* COMMON BLOCK /PRIR/ VARIABLES:
C*    IPRIR = DISTRIBUTION INDEX: =-1 BETA, =-2 LOGBETA,
C*            =-3 LOGNORMAL-1, =-4 LOGNORMAL-2, =-5 LOGGAMMA-1,
C*            =-6 LOGGAMMA-2,  =-7 LOGLOGNORMAL.
C*    PARM  = DISTRIBUTION PARAMETER VECTOR: PARM(1)=ALPHA, PARM(2)=
C*            BETA, PARM(3)=FIRST RANGE PARAMETER, PARM(4)=SECOND
C*            RANGE PARAMETER.
C*
C* COMMON BLOCK /DATAD/ VARIABLES
C*    K = THE NUMBER OF FAILURES THAT OCCURRED AFTER N ATTEMPTS.
C*    N = THE TOTAL NUMBER OF ATTEMPTS MADE DURING WHICH K FAILURES
C*        OCCURRED.
C*
C* WRITTEN BY S. D. HANSEN 8/79
C*
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      COMMON /DATAD/K,N
      ALPHA=PARM(1)
      BETA=PARM(2)
      THETA=PARM(3)
      IF (X.EQ.1.0D0.OR.X.EQ.0.0D0) GOTO 500
      IF (X.EQ.DXP(-THETA)) GOTO 500
      Q1=K*DLG(1-X)+(N-K)*DLG(X)+(ALPHA-1)*DLG(-DLG(X)-THETA)
      Q2=-(-DLG(X)-THETA)/BETA
      GAM2DF=DXP(Q1+Q2)/X
      RETURN
 500  GAM2DF=0.0D0
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION FLLDF (P)
C
C***********************************************************************
C
C   THIS FUNCTION EVALUATES THE INTEGRAND OF THE LOG LOGNORMAL
C   DISRIBUTION FOR THE FAILURE ON DEMAND CASE.
C
C* COMMON BLOCK /PRIR/ VARIABLES:
C*    IPRIR = DISTRIBUTION INDEX: =-1 BETA, =-2 LOGBETA,
C*            =-3 LOGNORMAL-1, =-4 LOGNORMAL-2, =-5 LOGGAMMA-1,
C*            =-6 LOGGAMMA-2,  =-7 LOGLOGNORMAL.
C*    PARM  = DISTRIBUTION PARAMETER VECTOR: PARM(1)=ALPHA, PARM(2)=
C*            BETA, PARM(3)=FIRST RANGE PARAMETER, PARM(4)=SECOND
C*            RANGE PARAMETER.
C*
C* COMMON BLOCK /DATAD/ VARIABLES
C*    K = THE NUMBER OF FAILURES THAT OCCURRED AFTER N ATTEMPTS.
C*    N = THE TOTAL NUMBER OF ATTEMPTS MADE DURING WHICH K FAILURES
C*        OCCURRED.
C*
C* WRITTEN BY S. D. HANSEN 8/79
C*
C**********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      COMMON /DATAD/K,N
      THETA=PARM(3)
      IF (P.EQ.1.OR.P.EQ.1-DXP(-THETA)) GOTO 100
      ALPHA=PARM(1)
      BETA=PARM(2)
      Q1=K*DLG(P)+(N-K)*DLG(1-P)
      Q2=-(DLG(-DLG(1-P)-THETA)-ALPHA)**2/(2*BETA**2)
      Q3=DLG(1-P)+DLG(-DLG(1-P)-THETA)
      FLLDF=DXP(Q1+Q2-Q3)
      RETURN
 100  FLLDF=0.0D0
      RETURN
      END
C+*********************************************************************
C
      DOUBLE PRECISION FUNCTION BFN (X)
C
C**********************************************************************
C
C   THIS FUNCTION EVALUATES EITHER LN G(X) OF THE BETA DIST. OR
C   THE LN F(Y) OF THE LOG BETA DIST. DEPENDING ON IPRIR.
C
C   NOTE:  IPRIR=1 BETA DISTRIBUTION
C          IPRIR=2 LOG BETA DISTRIBUTION
C
C* COMMON BLOCK /PRIR/ VARIABLES:
C*    IPRIR = DISTRIBUTION INDEX: =-1 BETA, =-2 LOGBETA,
C*            =-3 LOGNORMAL-1, =-4 LOGNORMAL-2, =-5 LOGGAMMA-1,
C*            =-6 LOGGAMMA-2, =-7 LOGLOGNORMAL.
C*    PARM  = DISTRIBUTION PARAMETER VECTOR: PARM(1)=ALPHA, PARM(2)=
C*            BETA, PARM(3)=FIRST RANGE PARAMETER, PARM(4)=SECOND
C*            RANGE PARAMETER.
C*
C* COMMON BLOCK /DATAD/ VARIABLES
C*    K = THE NUMBER OF FAILURES THAT OCCURRED AFTER N ATTEMPTS.
C*    N = THE TOTAL NUMBER OF ATTEMPTS MADE DURING WHICH K FAILURES
C*        OCCURRED.
C*
C* WRITTEN BY S. D. HANSEN 8/79
C*
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PRIR/PARM(4),RNG(2),IPRIR
      COMMON /DATAD/K,N
      ALPHA=PARM(1)
      BETA=PARM(2)
      A=PARM(3)
      B=PARM(4)
      II=-IPRIR
      GOTO (100,500),II
 100  PRMLN=DLGAMA(N+1.0D0)-DLGAMA(K+1.0D0)-DLGAMA(N-K+1.0D0)
      IF (X.EQ.0.0D0) GOTO 200
      IF (X.EQ.1) GOTO 300
      Q1=K*DLG(A+(B-A)*X)+(N-K)*DLG(1-A-(B-A)*X)
      BFN=PRMLN+Q1
      RETURN
 200  BFN=PRMLN+K*DLG(A)+(N-K)*DLG(1-A)
      RETURN
 300  IF (B.EQ.1) GOTO 900
      BFN=PRMLN+K*DLG(B)+(N-K)*DLG(1-B)
      RETURN
 500  IF (X.EQ.0.0D0) GOTO 600
      IF (X.EQ.1) GOTO 700
      Q1=1-DXP(-A-(B-A)*X)
      BFN=-K*(A+(B-A)*X)+(N-K)*DLG(Q1)
      RETURN
 600  IF (DXP(-A).EQ.1) GOTO 900
      BFN=-K*A+(N-K)*DLG(1-DXP(-A))
      RETURN
 700  BFN=-K*B+(N-K)*DLG(1-DXP(-B))
      RETURN
 900  BFN=-1.0D75
      RETURN
      END
C+**********************************************************************
C*
      DOUBLE PRECISION FUNCTION FXB (A,B)
C*
C***********************************************************************
C
C   THIS FUNCTION EVALUATES LN B(ALPHA,BETA).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      FXB=-DLGAMA(A+B)+DLGAMA(A)+DLGAMA(B)
      RETURN
      END
C+**********************************************************************
C*
      DOUBLE PRECISION FUNCTION SGN (X)
C*
C***********************************************************************
C
C   THIS FUNCTION RETURNS A NEGATIVE 1.0D0 FOR NEGATIVE X, OR A POSITIVE
C   1.0D0 FOR VALUES OF X GREATER THAN OR EQUAL TO 0.0D0.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IF (X) 100,200,200
 100  SGN=-1.0D0
      RETURN
 200  SGN=1.0D0
      RETURN
      END
C+**********************************************************************
C
      DOUBLE PRECISION FUNCTION DXP(X)
C
C***********************************************************************
C*
C*  THIS FUNCTION ERROR CHECKS THEN EVALATES DEXP(X).
C*
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IF (DABS(X).GT.170.000) GOTO 10
      DXP=DEXP(X)
      RETURN
   10 IF (X) 20,20,30
   20 DXP=0.0D0
      RETURN
   30 DXP=1.0D70
      RETURN
      END
C+**********************************************************************
C*                                                                     *
      DOUBLE PRECISION FUNCTION DLG(X)
C*                                                                     *
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE CALCULATES THE VALUE OF     DLOG(X)                 *
C* WITH STANDARD CORRECTION AT X = 0                                   *
C*                                                                     *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IF(X.EQ.0.D0) GO TO 10
      DLG=DLOG(X)
      RETURN
   10 DLG=-1.D70
      RETURN
      END
C+**********************************************************************
C
      FUNCTION DLGAMA(XX)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      DOUBLE PRECISION COF(6)
      DATA COF,STP /76.18009173D0, -86.50532033D0, 24.01409822D0,
     > -1.231739516D0, 0.120858003D-2, -0.536382D-5, 2.50662827465D0/
      DATA HALF, ONE, FPF /0.5D0,1.0D0,5.5D0/
C
      X  =XX-ONE
      TMP=X+FPF
      TMP=(X+HALF)*LOG(TMP)-TMP
      SER=ONE
      DO 11 J=1,6
        X  =X+ONE
11      SER=SER+COF(J)/X
      DLGAMA=TMP+LOG(STP*SER)
      END
